@@ -29,6 +29,7 @@ from coremltools.converters.mil.input_types import (
     InputType,
     RangeDim,
     Shape,
+    StateType,
     TensorType,
 )
 from coremltools.converters.mil.mil import Program, types
@@ -51,9 +52,7 @@ if _HAS_TF_2:
 if _HAS_TORCH:
     import torch
 
-    from coremltools.converters.mil.frontend.torch.load import (
-        _torchscript_from_spec as try_load_torchscript,
-    )
+    from coremltools.converters.mil.frontend.torch.load import is_torch_model
 
     if _HAS_TORCH_EXPORT_API:
         from torch.export import ExportedProgram
@@ -75,10 +74,11 @@ def convert(
     package_dir=None,
     debug=False,
     pass_pipeline: Optional[PassPipeline] = None,
+    states=None,
 ):
     """
     Convert a TensorFlow or PyTorch model to the Core ML model format as either
-    a neural network or an `ML program <https://coremltools.readme.io/docs/ml-programs>`_.
+    a neural network or an `ML program <https://apple.github.io/coremltools/docs-guides/source/convert-to-ml-program.html>`_.
     Some parameters and requirements differ for TensorFlow and PyTorch
     conversions.
 
@@ -112,7 +112,8 @@ def convert(
                 - Path to a ``.pt`` file
 
             - Torch Exported Models:
-                - A `ExportedProgram <https://pytorch.org/docs/stable/export.html#torch.export.ExportedProgram> ` object with `EDGE` dialect
+                - An `ExportedProgram <https://pytorch.org/docs/stable/export.html#torch.export.ExportedProgram>`_
+                  object with ``EDGE`` dialect.
 
     source : str (optional)
 
@@ -154,6 +155,7 @@ def convert(
               )
 
         * TensorFlow 1 and 2 (including tf.keras):
+
             - The ``inputs`` parameter is optional. If not provided, the inputs
               are placeholder nodes in the model (if the model is a frozen graph)
               or function inputs (if the model is a ``tf.function``).
@@ -162,31 +164,29 @@ def convert(
               in the TF model.
             - If ``name`` is specified with ``TensorType`` and ``ImageType``, it
               must correspond to a placeholder op in the TF graph. The input names
-              in the converted Core ML model can later be modifed using the
+              in the converted Core ML model can later be modified using the
               ``ct.utils.rename_feature`` API.
             - If ``dtype`` is not specified, it defaults to the ``dtype`` of the
               inputs in the TF model.
             - For ``minimum_deployment_target >= ct.target.macOS13``, and with ``compute_precision`` in float 16 precision.
-              When ``inputs`` not provided or ``dtype`` not specified. The float 32 inputs defaults to float 16.
+              When ``inputs`` not provided or ``dtype`` not specified, the float 32 inputs default to float 16.
 
         * PyTorch:
 
             - TorchScript Models:
                 - The ``inputs`` parameter is required.
-                - Number of elements in ``inputs`` must match the number of inputs
-                of the PyTorch model.
+                - Number of elements in ``inputs`` must match the number of inputs of the PyTorch model.
                 - ``inputs`` may be a nested list or tuple.
                 - ``TensorType`` and ``ImageType`` must have the ``shape`` specified.
                 - If the ``name`` argument is specified with ``TensorType`` or
-                ``ImageType``, the converted Core ML model will have inputs with
-                the same name.
+                  ``ImageType``, the converted Core ML model will have inputs with the same name.
                 - If ``dtype`` is missing:
-                * For ``minimum_deployment_target <= ct.target.macOS12``, it defaults to float 32.
-                * For ``minimum_deployment_target >= ct.target.macOS13``, and with ``compute_precision`` in float 16 precision.
-                    It defaults to float 16.
-
+                    * For ``minimum_deployment_target <= ct.target.macOS12``, it defaults to float 32.
+                    * For ``minimum_deployment_target >= ct.target.macOS13``, and with ``compute_precision`` in float 16 precision.
+                      It defaults to float 16.
             - Torch Exported Models:
-                - The ``inputs`` parameter is not supported. ``inputs`` parameter is inferred from Torch ExportedProgram.
+                - The ``inputs`` parameter is not supported.
+                - The ``inputs`` parameter is inferred from the Torch `ExportedProgram <https://pytorch.org/docs/stable/export.html#torch.export.ExportedProgram>`_.
 
     outputs : list of ``TensorType`` or ``ImageType`` (optional)
 
@@ -228,22 +228,21 @@ def convert(
               will be converted up to that node.
             - For ``minimum_deployment_target >= ct.target.macOS13``, and with ``compute_precision`` in float 16 precision.
               If ``dtype`` not specified, the outputs inferred of type float 32
-              defaults to float 16.
+              default to float 16.
 
-        * PyTorch:
-
-            - TorchScript Models:
-                - If specified, the length of the list must match the number of
-                outputs returned by the PyTorch model.
-                - If ``name`` is specified, it is applied to the output names of the
-                converted Core ML model.
-                - For ``minimum_deployment_target >= ct.target.macOS13``, and with ``compute_precision`` in float 16 precision.
-                If ``dtype`` not specified, the outputs inferred of type float 32
-                defaults to float 16.
-
-            - Torch Exported Models:
-                - The ``outputs`` parameter is not supported. ``outputs`` parameter is inferred from Torch ExportedProgram.
+        * PyTorch: TorchScript Models
+            - If specified, the length of the list must match the number of
+              outputs returned by the PyTorch model.
+            - If ``name`` is specified, it is applied to the output names of the
+              converted Core ML model.
+            - For ``minimum_deployment_target >= ct.target.macOS13``,
+              and with ``compute_precision`` in float 16 precision.
+            - If ``dtype`` not specified, the outputs inferred of type float 32
+              default to float 16.
 
+        * PyTorch: Torch Exported Models:
+            - The ``outputs`` parameter is not supported.
+            - The ``outputs`` parameter is inferred from Torch `ExportedProgram <https://pytorch.org/docs/stable/export.html#torch.export.ExportedProgram>`_.
 
     classifier_config : ClassifierConfig class (optional)
         The configuration if the MLModel is intended to be a classifier.
@@ -253,17 +252,21 @@ def convert(
         The value of this parameter determines the type of the model
         representation produced by the converter. To learn about the differences
         between ML programs and neural networks, see
-        `ML Programs <https://coremltools.readme.io/docs/ml-programs>`_.
+        `ML Programs <https://apple.github.io/coremltools/docs-guides/source/convert-to-ml-program.html>`_.
 
         - The converter produces a neural network (``neuralnetwork``) if:
-          ::
+
+          .. sourcecode:: python
+
              minimum_deployment_target <= coremltools.target.iOS14/
                                           coremltools.target.macOS11/
                                           coremltools.target.watchOS7/
                                           coremltools.target.tvOS14:
 
         - The converter produces an ML program (``mlprogram``) if:
-          ::
+
+          .. sourcecode:: python
+
              minimum_deployment_target >= coremltools.target.iOS15/
                                            coremltools.target.macOS12/
                                            coremltools.target.watchOS8/
@@ -274,7 +277,9 @@ def convert(
           model type with as minimum of a deployment target as possible.
         - If this parameter is specified and ``convert_to`` is also specified,
           they must be compatible. The following are examples of invalid values:
-          ::
+
+          .. sourcecode:: python
+
             # Invalid:
             convert_to="mlprogram", minimum_deployment_target=coremltools.target.iOS14
 
@@ -286,7 +291,7 @@ def convert(
         The value of this parameter determines the type of the model
         representation produced by the converter. To learn about the
         differences between ML programs and neural networks, see
-        `ML Programs <https://coremltools.readme.io/docs/ml-programs>`_.
+        `ML Programs <https://apple.github.io/coremltools/docs-guides/source/convert-to-ml-program.html>`_.
 
         - ``'mlprogram'`` : Returns an MLModel (``coremltools.models.MLModel``)
           containing a MILSpec.Program proto, which is the Core ML program format.
@@ -302,7 +307,9 @@ def convert(
           (``coremltools.converters.mil.Program``). An MIL program is primarily
           used for debugging and inspection. It can be converted to an MLModel for
           execution by using one of the following:
-          ::
+
+          .. sourcecode:: python
+
              ct.convert(mil_program, convert_to="neuralnetwork")
              ct.convert(mil_program, convert_to="mlprogram")
 
@@ -319,7 +326,9 @@ def convert(
           applied to produce a float 16 program; that is, a program in which all
           the intermediate float tensors are of type float 16 (for ops that
           support that type).
-          ::
+
+          .. sourcecode:: python
+
               coremltools.transform.FP16ComputePrecision(op_selector=
                                                          lambda op:True)
 
@@ -343,7 +352,9 @@ def convert(
           but you can customize this.
 
           For example:
-          ::
+
+          .. sourcecode:: python
+
              coremltools.transform.FP16ComputePrecision(op_selector=
                                          lambda op: op.op_type != "linear")
 
@@ -393,14 +404,13 @@ def convert(
         conversion, the model is loaded with the provided set of compute units and
         returned.
 
-        An enum with the following possible values.
-            - ``coremltools.ComputeUnit.ALL``: Use all compute units available, including the
-              neural engine.
-            - ``coremltools.ComputeUnit.CPU_ONLY``: Limit the model to only use the CPU.
-            - ``coremltools.ComputeUnit.CPU_AND_GPU``: Use both the CPU and GPU, but not the
-              neural engine.
-            - ``coremltools.ComputeUnit.CPU_AND_NE``: Use both the CPU and neural engine, but
-              not the GPU. Available only for macOS >= 13.0.
+        An enum with the following possible values:
+
+        * ``coremltools.ComputeUnit.ALL``: Use all compute units available, including the neural engine.
+        * ``coremltools.ComputeUnit.CPU_ONLY``: Limit the model to only use the CPU.
+        * ``coremltools.ComputeUnit.CPU_AND_GPU``: Use both the CPU and GPU, but not the neural engine.
+        * ``coremltools.ComputeUnit.CPU_AND_NE``: Use both the CPU and neural engine, but
+          not the GPU. Available only for macOS >= 13.0.
 
     package_dir : str
         Post conversion, the model is saved at a temporary location and
@@ -414,11 +424,10 @@ def convert(
     debug : bool
         This flag should generally be ``False`` except for debugging purposes.
         Setting this flag to ``True`` produces the following behavior:
-          - For Torch conversion, it will print the list of supported and
-            unsupported ops found in the model if conversion fails due to an
-            unsupported op.
-          - For Tensorflow conversion, it will cause to display extra logging
-            and visualizations.
+
+        * For Torch conversion, it will print the list of supported and
+          unsupported ops found in the model if conversion fails due to an unsupported op.
+        * For Tensorflow conversion, it will cause to display extra logging and visualizations.
 
     pass_pipeline : PassPipeline
         Manage graph passes. You can control which graph passes to run and the order of the
@@ -430,18 +439,18 @@ def convert(
 
           .. sourcecode:: python
 
-            pipeline = ct.PassPipeline()
-            pipeline.remove_passes({"common::fuse_conv_batchnorm"})
-            mlmodel = ct.convert(model, pass_pipeline=pipeline)
+             pipeline = ct.PassPipeline()
+             pipeline.remove_passes({"common::fuse_conv_batchnorm"})
+             mlmodel = ct.convert(model, pass_pipeline=pipeline)
 
         * To avoid folding too-large ``const`` ops that lead to a large model, set pass option
           as shown in the following example:
 
           .. sourcecode:: python
 
-            pipeline = ct.PassPipeline()
-            pipeline.set_options("common::const_elimination", {"skip_const_by_size": "1e6"})
-            mlmodel = ct.convert(model, pass_pipeline=pipeline)
+             pipeline = ct.PassPipeline()
+             pipeline.set_options("common::const_elimination", {"skip_const_by_size": "1e6"})
+             mlmodel = ct.convert(model, pass_pipeline=pipeline)
 
         We also provide a set of predefined pass pipelines that you can directly call.
 
@@ -470,6 +479,50 @@ def convert(
 
              mlmodel = ct.convert(model, pass_pipeline=ct.PassPipeline.DEFAULT_PALETTIZATION)
 
+    states:
+        Create a stateful ``mlprogram`` model
+        by providing the ``StateType`` in the ``states`` argument (for details see `MIL Input Types <https://apple.github.io/coremltools/source/coremltools.converters.mil.input_types.html>`_).
+        The stateful model is useful when converting a large language model with KV-Cache.
+        The name of ``StateType`` must match the key of the PyTorch ``named_buffers()`` method in the source traced model.
+
+        The following example converts a torch model with a buffer called ``state_1``.
+
+        .. sourcecode:: python
+
+            class UpdateBufferModel(torch.nn.Module):
+                def __init__(self):
+                    super(UpdateBufferModel, self).__init__()
+                    self.register_buffer(
+                        "state_1", torch.tensor(np.array([0, 0, 0], dtype=np.float32))
+                    )
+
+                def forward(self, x):
+                    # In place update of the model state
+                    self.state_1.add_(x)
+                    return self.state_1
+
+
+            model = UpdateBufferModel()
+            traced_model = torch.jit.trace(model, torch.tensor([1, 2, 3], dtype=torch.float32))
+
+            inputs = [
+                ct.TensorType(shape=(1, 2)),
+            ]
+            states = [
+                ct.StateType(
+                    wrapped_type=ct.TensorType(
+                        shape=(1, 2),
+                    ),
+                    name="state_1",
+                ),
+            ]
+            mlmodel = ct.convert(
+                traced_model,
+                inputs=inputs,
+                states=states,
+                minimum_deployment_target=ct.target.iOS18,
+            )
+
     Returns
     -------
 
@@ -519,8 +572,7 @@ def convert(
             >>> results = mlmodel.predict({"input": example_input.numpy()})
             >>> print(results['1651']) # 1651 is the node name given by PyTorch's JIT
 
-    See `Conversion Options <https://coremltools.readme.io/docs/neural-network-conversion>`_ for
-    more advanced options.
+    For more options see `Conversion Options <https://apple.github.io/coremltools/docs-guides/source/conversion-options.html>`_.
     """
     _check_deployment_target(minimum_deployment_target)
     outputs_as_strings, outputs_as_tensor_or_image_types = _validate_outputs_argument(outputs)
@@ -571,6 +623,15 @@ def convert(
         and need_fp16_cast_pass
     )
 
+    # Verify the inputs cannot contains state
+    if states is None:
+        states = []
+    _verify_inputs_doesnot_contains_states(inputs)
+
+    # states can only passed if the source is pytorch
+    if len(states) > 0 and exact_source != "pytorch":
+        raise ValueError("'states' can only be passed with pytorch source model.")
+
     mlmodel = mil_convert(
         model,
         convert_from=exact_source,
@@ -585,6 +646,7 @@ def convert(
         specification_version=specification_version,
         main_pipeline=pass_pipeline,
         use_default_fp16_io=use_default_fp16_io,
+        states=states,
     )
 
     if exact_target == "mlprogram" and mlmodel._input_has_infinite_upper_bound():
@@ -651,6 +713,20 @@ def _check_deployment_target(minimum_deployment_target):
         raise TypeError(msg.format(minimum_deployment_target))
 
 
+def _verify_inputs_doesnot_contains_states(
+    inputs: List[InputType],
+) -> None:
+    """
+    Verify that StateType is not present in the inputs.
+    """
+    if inputs is None:
+        return
+
+    for val in inputs:
+        if isinstance(val, StateType):
+            raise ValueError("'inputs' cannot contain an instance of StateType.")
+
+
 def _validate_outputs_argument(outputs):
     """
     - validate properties that the "outputs" argument must satisfy, for instance, it should either be a list
@@ -724,7 +800,7 @@ def _validate_conversion_arguments(
     compute_precision,
     convert_to,
     minimum_deployment_target,
-):
+) -> None:
     """
     Validate and process model, inputs, classifier_config based on
     `exact_source` (which cannot be `auto`) and `exact_target`.
@@ -841,9 +917,9 @@ def _validate_conversion_arguments(
 
     elif exact_source == "pytorch":
         if _HAS_TORCH_EXPORT_API and isinstance(model, ExportedProgram):
-            if model.dialect != "EDGE":
+            if model.dialect not in ("ATEN", "EDGE"):
                 raise NotImplementedError(
-                    f"Conversion for models with only EDGE dialect is supported/tested. Provided Dialect: {model.dialect}"
+                    f"Conversion for models with only ATEN or EDGE dialect is supported/tested. Provided Dialect: {model.dialect}"
                 )
 
             # TODO: rdar://115845792 ([Executorch] Handle user provided inputs/outputs in the convert API)
@@ -854,13 +930,7 @@ def _validate_conversion_arguments(
                 raise AssertionError("'outputs' argument should be None for ExportedProgram")
 
         else:
-            is_torch_load_successful = False
-            try:
-                try_load_torchscript(model)
-                is_torch_load_successful = True
-            except:
-                pass
-            if is_torch_load_successful:
+            if is_torch_model(model):
                 if inputs is None:
                     raise ValueError(
                         'Expected argument "inputs" for TorchScript models not provided'
@@ -875,7 +945,7 @@ def _validate_conversion_arguments(
                     )
             else:
                 raise TypeError(
-                    "@model must either be a TorchScript object (or .pt or .pth file) or an ExportedProgram object (if using torch.export based API), received: {}".format(
+                    "Model must either be a TorchScript object (or .pt or .pth file) or an ExportedProgram object (if using torch.export based API), received: {}".format(
                         type(model)
                     )
                 )
@@ -903,14 +973,14 @@ def _determine_source_dialect(model, exact_source):
 def _determine_source(model, source,
                       output_names,
                       outputs_as_tensor_or_image_types,
-                      output_argument_as_specified_by_user):
+                      output_argument_as_specified_by_user) -> str:
     """
     Infer source (which can be auto) to the precise framework.
     """
     source = source.lower()
     if source not in {"auto", "tensorflow", "pytorch", "milinternal"}:
         raise ValueError(
-            f'Unrecognized value of argument "source": {source}. It must be one of ["auto", "tensorflow", "pytorch"].'
+            f'Unrecognized value of argument "source": {source}. It must be one of ["auto", "tensorflow", "pytorch", "milinternal"].'
         )
 
     # Determine tensorflow version
@@ -942,13 +1012,7 @@ def _determine_source(model, source,
         if _HAS_TORCH_EXPORT_API and isinstance(model, ExportedProgram):
             return "pytorch"
 
-        is_torch_load_successful = False
-        try:
-            try_load_torchscript(model)
-            is_torch_load_successful = True
-        except:
-            pass
-        if is_torch_load_successful:
+        if is_torch_model(model):
             # validate that the outputs passed by the user are of type ImageType/TensorType
             if output_argument_as_specified_by_user is not None and not all(
                 [
@@ -977,18 +1041,18 @@ def _determine_source(model, source,
     raise ValueError(msg)
 
 
-def _determine_target(convert_to, minimum_deployment_target):
+def _determine_target(convert_to, minimum_deployment_target) -> str:
     """
     Infer the precise backend target, which could be one of ``milinternal``, ``neuralnetwork`` or ``mlprogram``
     """
     if minimum_deployment_target is None and convert_to is None:
         logger.warning(
             "When both 'convert_to' and 'minimum_deployment_target' not specified, "
-            "'convert_to' is set to \"mlprogram\" and 'minimum_deployment_targer' is set to "
+            "'convert_to' is set to \"mlprogram\" and 'minimum_deployment_target' is set to "
             "ct.target.iOS15 (which is same as ct.target.macOS12). "
             "Note: the model will not run on systems older than iOS15/macOS12/watchOS8/tvOS15. "
             "In order to make your model run on older system, please set the 'minimum_deployment_target' to iOS14/iOS13. "
-            "Details please see the link: https://coremltools.readme.io/docs/unified-conversion-api#target-conversion-formats"
+            "Details please see the link: https://apple.github.io/coremltools/docs-guides/source/target-conversion-formats.html"
         )
     if minimum_deployment_target is not None:
         if convert_to == "mlprogram" and minimum_deployment_target < AvailableTarget.iOS15:
