@@ -3,9 +3,5 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from . import (
-    adjust_io_to_supported_types,
-    fuse_activation_silu,
-    insert_image_preprocessing_op,
-    sanitize_name_strings
-)
+from . import (adjust_io_to_supported_types, fuse_activation_silu,
+               insert_image_preprocessing_op, sanitize_name_strings)
