@@ -3,8 +3,7 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from coremltools.converters.mil.mil.passes.name_sanitization_utils import \
-    NameSanitizer as _NameSanitizer
+from coremltools.converters.mil.mil.passes.defs.preprocess import NameSanitizer as _NameSanitizer
 
 
 class TestNameSanitizer:
