@@ -3,6 +3,12 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from . import (alert_return_type_cast, commingle_loop_vars,
-               handle_return_inputs_as_outputs, handle_return_unused_inputs,
-               handle_unused_inputs, mlmodel_passes)
+from . import (
+    alert_return_type_cast,
+    commingle_loop_vars,
+    conv1d_decomposition,
+    handle_return_inputs_as_outputs,
+    handle_return_unused_inputs,
+    handle_unused_inputs,
+    mlmodel_passes,
+)
