@@ -1,22 +1,22 @@
-# -*- coding: utf-8 -*-
-
 #  Copyright (c) 2021, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 import os
+
 import numpy as np
 
 from coremltools.converters.mil import Builder as mb
-from coremltools.converters.mil.experimental.passes.generic_pass_infrastructure import register_generic_pass
+from coremltools.converters.mil.experimental.passes.generic_pass_infrastructure import \
+    register_generic_pass
 from coremltools.converters.mil.mil import get_new_symbol
 
 if os.getenv("ENABLE_EXPERIMENTAL_PASSES") == "1":
     arbitrary_shape = (get_new_symbol(), get_new_symbol())
     np.random.seed()
-    arbitrary_weight = np.random.rand(4,3)
-    arbitrary_bias =  np.random.rand(4)
+    arbitrary_weight = np.random.rand(4, 3)
+    arbitrary_bias = np.random.rand(4)
 
 if os.getenv("ENABLE_EXPERIMENTAL_PASSES") == "1":
     @mb.program(input_specs=[mb.TensorSpec(shape=arbitrary_shape)])
@@ -87,8 +87,10 @@ def transform_pattern(pattern):
     linear_bias, bias, Dout = _get_linear_bias_bias_Dout(pattern, is_first_input)
     bias = np.reshape(bias, (Dout,))
 
-    if is_sub and is_first_input: bias = -bias
-    if is_sub and not is_first_input: linear_bias = -linear_bias
+    if is_sub and is_first_input:
+        bias = -bias
+    if is_sub and not is_first_input:
+        linear_bias = -linear_bias
 
     new_bias = linear_bias + bias
 
@@ -128,4 +130,4 @@ if os.getenv('ENABLE_EXPERIMENTAL_PASSES') == '1':
         transform_pattern=transform_pattern,
         pass_name="fuse_linear_bias",
         namespace="common",
-    )
\ No newline at end of file
+    )
