@@ -2,14 +2,22 @@
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
+
 import itertools
+import math as math
+from typing import List, Optional, Union
 
-from typing import List, Optional
+import numpy as np
 
+from coremltools.converters.mil._deployment_compatibility import AvailableTarget as target
 from coremltools.converters.mil.input_types import InputType
 from coremltools.converters.mil.mil import Builder as mb
-from coremltools.converters.mil.mil import Var, types
-from coremltools.converters.mil.mil.ops.defs._utils import parse_einsum_equation
+from coremltools.converters.mil.mil import Operation, Var, types
+from coremltools.converters.mil.mil.block import is_current_opset_version_compatible_with
+from coremltools.converters.mil.mil.ops.defs._utils import (
+    parse_einsum_equation,
+    promote_input_dtypes,
+)
 from coremltools.converters.mil.mil.types.symbolic import any_symbolic, is_symbolic
 
 
@@ -33,9 +41,50 @@ def value_at(x: Var, idx: int, name=None, before_op=None):
     return mb.slice_by_index(**args)
 
 
+def _construct_gather_op(
+    op_type: str, x: Var, indices: Var, axis: Var = None, name: str = None
+) -> Var:
+    """
+    This utility is a more general gather in the sense that:
+    1. Both mb.gather and mb.gather_nd are handled
+    2. x is allowed to be bool, while mb.gather and mb.gather_nd only allow float or int
+    """
+    assert (
+        op_type in {"gather", "gather_nd"}
+    ), f"This utility only handles gather or gather_nd, but got {op_type}"
+    if op_type == "gather_nd":
+        assert axis is None, "mb.gather_nd should not have input axis"
+
+    # if is gathering bool:
+    #     cast bool input to a smallest supported dtype to gather, then cast back gather result
+    #     the back cast carries the specified name
+    # else:
+    #     usual gather, and gather carries the specified name
+    is_gathering_bool = x.dtype == types.bool
+    if is_gathering_bool:
+        gather_name_kwarg = {}
+        cast_name_kwarg = {} if name is None else {"name": name}
+    else:
+        gather_name_kwarg = {} if name is None else {"name": name}
+
+    if is_gathering_bool:
+        work_dtype = "int8" if is_current_opset_version_compatible_with(target.iOS17) else "fp16"
+        x = mb.cast(x=x, dtype=work_dtype)
+
+    if op_type == "gather":
+        result = mb.gather(x=x, indices=indices, axis=axis, **gather_name_kwarg)
+    else:
+        result = mb.gather_nd(x=x, indices=indices, **gather_name_kwarg)
+
+    if is_gathering_bool:
+        result = mb.cast(x=result, dtype="bool", **cast_name_kwarg)
+
+    return result
+
+
 def _reverse_input_einsum_eq(equation: str) -> str:
     """
-    Reverse the input order of the einsum eqaution
+    Reverse the input order of the einsum equation
     e.g.:
     input : "nchw,nwhu->nchu"
     returns : "nwhu,nchw->nchu"
@@ -230,6 +279,31 @@ def get_output_names(outputs) -> Optional[List[str]]:
     return output_names
 
 
+# This is a workaround in Core ML for topk with dynamic `k`:
+#     * Core ML topk supports only constant `k`
+#     * Luckily, Core ML gather supports dynamic `end`, so we workaround by argsort then gather
+# This leads to a slightly different behaviour, though: top-k elements are always sorted
+def dynamic_topk(
+    x: Var, k: Var, axis: int, ascending: Optional[bool] = False, name: Optional[str] = None
+):
+    assert k.val is None, "Please use mb.topk directly if k is compile time known"
+
+    indices = mb.argsort(x=x, axis=axis, ascending=ascending)
+    if name is None:
+        values = mb.gather_along_axis(x=x, indices=indices, axis=axis)
+    else:
+        values = mb.gather_along_axis(x=x, indices=indices, axis=axis, name=name)
+
+    k_indices = mb.range_1d(end=k, start=0, step=1)
+    values = mb.gather(x=values, indices=k_indices, axis=axis)
+    if name is None:
+        indices = mb.gather(x=indices, indices=k_indices, axis=axis)
+    else:
+        indices = mb.gather(x=indices, indices=k_indices, axis=axis, name=name)
+
+    return values, indices
+
+
 def solve_diagonal_einsum(parsed_vectors, vars):
     def solve_diagonal_einsum_one_step(parsed_vector, x):
         for i in range(len(parsed_vector)):
@@ -436,3 +510,148 @@ def solve_binary_generic_einsum(parsed_vectors, a_var, b_var, name) -> Var:
         else:
             ab = mb.transpose(x=ab, perm=get_perm_transpose_einsum(ab_reshaped_axes, out_axes), name=name)
         return ab
+
+
+def _decompose_scaled_dot_product_attention(
+    q: Var,
+    k: Var,
+    v: Var,
+    mask: Var,
+    name: str,
+    scale: Optional[Var] = None,
+    before_op: Optional[Operation] = None,
+) -> Var:
+    # scale the query input
+    embed_size = q.shape[-1]
+    if is_symbolic(embed_size):
+        raise ValueError(
+            "The embedding size, i.e. last dimension of the shape of query tensor"
+            " cannot be symbolic, in scaled_dot_product_attention op"
+        )
+
+    q, k, v = promote_input_dtypes([q, k, v])
+    if scale is None:
+        multiplicative_scale_factor = 1 / math.sqrt(embed_size)
+        if types.builtin_to_string(q.dtype) == "fp16":
+            multiplicative_scale_factor = np.float16(multiplicative_scale_factor)
+    else:
+        multiplicative_scale_factor = scale
+    q = mb.mul(x=q, y=multiplicative_scale_factor, before_op=before_op)
+
+    # multiply query and key input tensors
+    # shape of output: (target_seq, source_seq) or (B,...,target_seq, source_seq)
+    attn_weights = mb.matmul(x=q, y=k, transpose_y=True, before_op=before_op)
+
+    # add mask if applicable
+    if mask is not None:
+        attn_weights = mb.add(x=attn_weights, y=mask, before_op=before_op)
+
+    # do softmax
+    attn_weights_normalized = mb.softmax(x=attn_weights, axis=-1, before_op=before_op)
+
+    # multiply attn_weights and value tensor
+    res = mb.matmul(x=attn_weights_normalized, y=v, name=name, before_op=before_op)
+    return res
+
+
+def _construct_constexpr_dequant_op(
+    quantized_weights: np.ndarray,
+    zero_point: Optional[Union[Var, np.ndarray, np.generic]],
+    scale: Union[Var, np.ndarray, np.generic],
+    axis: Optional[Union[Var, int]] = None,
+    name: Optional[str] = None,
+    before_op: Optional[Operation] = None,
+) -> Var:
+    """
+    Constructs the constexpr op to represent the quantized weight.
+
+    Use constexpr_affine_dequantize for pre-iOS18 and constexpr_blockwise_shift_scale for others.
+    """
+    if not is_current_opset_version_compatible_with(target.iOS18):
+        # The constexpr_affine_dequantize op requires axis.
+        if axis is None:
+            # Infer the axis based on scale's shape.
+            non_single_dim = [dim for dim, dim_size in enumerate(scale.shape) if dim_size > 1]
+            if len(non_single_dim) > 2:
+                raise ValueError(
+                    "The constexpr_affine_dequantize op doesn't support scale which "
+                    "have more than one non-single dimensions. Got scale with shape "
+                    f"{scale.shape}"
+                )
+            # Empty non_single_dim means per-tensor quantization, just use a dummy axis.
+            axis = 0 if len(non_single_dim) == 0 else non_single_dim[0]
+        if isinstance(axis, int):
+            axis = np.int32(axis)
+
+        # The constexpr_affine_dequantize op requires zero_point.
+        if zero_point is None:
+            zero_point = np.zeros_like(scale).astype(quantized_weights.dtype)
+
+        # The constexpr_affine_dequantize op requires scale and zero_point to have rank 0 or 1.
+        if isinstance(scale, (np.ndarray, np.generic)):
+            scale = np.squeeze(scale)
+        if isinstance(zero_point, (np.ndarray, np.generic)):
+            zero_point = np.squeeze(zero_point)
+        if len(scale.shape) > 1 or len(zero_point.shape) > 1:
+            raise ValueError(
+                "The more fine-grained quantization (such as blockwise) is only supported since iOS18."
+                "Please set minimum_deployment_target to iOS18 for using it."
+            )
+
+        kwargs = {
+            "quantized_data": quantized_weights,
+            "zero_point": zero_point,
+            "scale": scale,
+            "axis": axis,
+        }
+        if name is not None:
+            kwargs["name"] = name
+        if before_op is not None:
+            kwargs["before_op"] = before_op
+        return mb.constexpr_affine_dequantize(**kwargs)
+
+    # For iOS18 constexpr_blockwise_shift_scale op, the data/scale/offset need to have same rank.
+    if len(quantized_weights.shape) != len(scale.shape):
+        if axis is not None:
+            target_shape = [1] * len(quantized_weights.shape)
+            target_shape[axis] = quantized_weights.shape[axis]
+        else:
+            target_shape = list(scale.shape) + [1] * (
+                len(quantized_weights.shape) - len(scale.shape)
+            )
+        if np.prod(scale.shape) != np.prod(target_shape):
+            raise ValueError(
+                "Unable to infer scale's shape. Please provide a scale that has the "
+                "same rank as the weight."
+            )
+        scale = scale.reshape(target_shape)
+
+    # Check the value range to determine the true data type (such as int4/uint4).
+    sub_byte_type = (
+        types.uint4
+        if types.numpy_type_to_builtin_type(quantized_weights.dtype).is_unsigned()
+        else types.int4
+    )
+    sub_byte_range = types.type_mapping._TYPES_TO_RANGE[sub_byte_type]
+    if (
+        np.max(quantized_weights) <= sub_byte_range.high
+        and np.min(quantized_weights) >= sub_byte_range.low
+    ):
+        quantized_weights = quantized_weights.astype(types.nptype_from_builtin(sub_byte_type))
+
+    kwargs = {
+        "data": quantized_weights,
+        "scale": scale,
+    }
+    if zero_point is not None and np.any(zero_point):
+        # Only pass the offset parameter when not all elements in `zero_point` are zeroes.
+        zero_point = zero_point.reshape(scale.shape)
+        # When zero_point is integer, it's required to have the same dtype as the quantized weight.
+        if np.issubdtype(zero_point.dtype, np.integer):
+            zero_point = zero_point.astype(quantized_weights.dtype)
+        kwargs["offset"] = zero_point
+    if name is not None:
+        kwargs["name"] = name
+    if before_op is not None:
+        kwargs["before_op"] = before_op
+    return mb.constexpr_blockwise_shift_scale(**kwargs)
