@@ -4,10 +4,14 @@
 # found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 import os
+from typing import Tuple
 
 import numpy as np
 
+import coremltools.converters.mil.frontend.milproto.load
 from coremltools import _logger as logger
+from coremltools import proto
+from coremltools.converters.mil import mil
 from coremltools.converters.mil._deployment_compatibility import AvailableTarget as _target
 from coremltools.converters.mil.backend.mil import helper
 from coremltools.converters.mil.mil import Block
@@ -16,7 +20,6 @@ from coremltools.converters.mil.mil import (
     Function,
     ListVar,
     Placeholder,
-    Program,
     TupleInputType,
     Var,
     mil_list,
@@ -24,8 +27,7 @@ from coremltools.converters.mil.mil import (
 )
 from coremltools.converters.mil.mil.block import curr_block
 from coremltools.converters.mil.mil.ops.registry import SSAOpRegistry as _SSAOpRegistry
-from coremltools.proto import MIL_pb2 as pm
-from coremltools.proto import Model_pb2 as ml
+from coremltools.converters.mil.mil.program import StateTensorPlaceholder
 
 from .helper import proto_to_types
 
@@ -63,7 +65,7 @@ class TranscriptionContext:
 
 
 def _load_tensorvalue(tensorvalue_spec):
-    if not isinstance(tensorvalue_spec, pm.TensorValue):
+    if not isinstance(tensorvalue_spec, proto.MIL_pb2.TensorValue):
         raise TypeError("Invalid TensorValue spec object")
 
     if tensorvalue_spec.WhichOneof("value") == "floats":
@@ -84,14 +86,19 @@ def _load_tensorvalue(tensorvalue_spec):
         raise ValueError("Invalid dtype for TensorValue type")
 
 
-def _load_immediate_value(immediatevalue_spec):
-    if not isinstance(immediatevalue_spec, pm.Value.ImmediateValue):
+def _load_immediate_value(context: TranscriptionContext, immediatevalue_spec):
+    if not isinstance(immediatevalue_spec, proto.MIL_pb2.Value.ImmediateValue):
         raise TypeError("Invalid ImmedidateValue spec object")
 
     if immediatevalue_spec.WhichOneof("value") == "tensor":
         return _load_tensorvalue(immediatevalue_spec.tensor)
     elif immediatevalue_spec.WhichOneof("value") == "list":
         return immediatevalue_spec.list.values
+    elif immediatevalue_spec.WhichOneof("value") == "dictionary":
+        result = {}
+        for value in immediatevalue_spec.dictionary.values:
+            result[_load_value(context, value.key)] = _load_value(context, value.value)
+        return result
     else:
         raise NotImplementedError(
             "Immediate value type not supported yet."
@@ -101,7 +108,7 @@ def _load_immediate_value(immediatevalue_spec):
 def _load_file_value(context, filevalue_spec, dtype):
     if BlobReader is None:
         raise RuntimeError("BlobReader not loaded")
-    if not isinstance(filevalue_spec, pm.Value.BlobFileValue):
+    if not isinstance(filevalue_spec, proto.MIL_pb2.Value.BlobFileValue):
         raise TypeError("Invalid BlobFileValue spec object")
 
     filename = os.path.join(context.weights_dir, filevalue_spec.fileName.split("/")[-1])
@@ -113,8 +120,20 @@ def _load_file_value(context, filevalue_spec, dtype):
         blob_reader = BlobReader(filename)
         context.blob_reader_from_filename[filename] = blob_reader
 
-    if dtype == types.uint8:
+    if dtype == types.uint1:
+        np_value = blob_reader.read_uint1_data(offset)
+    elif dtype == types.uint2:
+        np_value = blob_reader.read_uint2_data(offset)
+    elif dtype == types.uint3:
+        np_value = blob_reader.read_uint3_data(offset)
+    elif dtype == types.uint4:
+        np_value = blob_reader.read_uint4_data(offset)
+    elif dtype == types.uint6:
+        np_value = blob_reader.read_uint6_data(offset)
+    elif dtype == types.uint8:
         np_value = blob_reader.read_uint8_data(offset)
+    elif dtype == types.int4:
+        np_value = blob_reader.read_int4_data(offset)
     elif dtype == types.int8:
         np_value = blob_reader.read_int8_data(offset)
     elif dtype == types.uint16:
@@ -126,20 +145,42 @@ def _load_file_value(context, filevalue_spec, dtype):
         np_value = np.frombuffer(np_value_uint16.tobytes(), np.float16)
     elif dtype == types.fp32:
         np_value = blob_reader.read_float_data(offset)
+    elif dtype == types.int32:
+        np_value = blob_reader.read_int32_data(offset)
+    elif dtype == types.uint32:
+        np_value = blob_reader.read_uint32_data(offset)
     else:
         raise ValueError("Invalid dtype for blob file value type")
 
     return np_value
 
 
+def _restore_np_from_bytes_value(value: bytes, dtype: types, shape: Tuple[int]) -> np.ndarray:
+    # Import _utils here to avoid circular import.
+    from coremltools.optimize.coreml import _utils as optimize_utils
+
+    if types.is_sub_byte(dtype) and isinstance(value, bytes):
+        result = np.frombuffer(value, types.nptype_from_builtin(dtype))
+        # For sub-byte data, the np array restored from bytes is packed, so we need to unpack it.
+        nbits = dtype.get_bitwidth()
+        element_num = np.prod(shape)
+        are_packed_values_signed = not dtype.is_unsigned()
+        return optimize_utils.restore_elements_from_packed_bits(
+            result, nbits, element_num, are_packed_values_signed
+        ).reshape(shape)
+
+    return np.frombuffer(value, types.nptype_from_builtin(dtype)).reshape(shape)
+
+
 def _load_value(context, value_spec):
-    if not isinstance(value_spec, pm.Value):
+    if not isinstance(value_spec, proto.MIL_pb2.Value):
         raise TypeError("Invalid Value spec object")
 
     if value_spec.docString:
         raise ValueError("Docstring would get lost in the process.")
 
-    if value_spec.type.WhichOneof("type") == "tensorType":
+    value_spec_type = value_spec.type.WhichOneof("type")
+    if value_spec_type == "tensorType":
         valuetype = proto_to_types(value_spec.type)
 
         is_tensor = types.is_tensor(valuetype)
@@ -148,24 +189,36 @@ def _load_value(context, value_spec):
         shape = () if not is_tensor else valuetype.get_shape()
 
         if value_spec.WhichOneof("value") == "immediateValue":
-            value = _load_immediate_value(value_spec.immediateValue)
+            value = _load_immediate_value(context, value_spec.immediateValue)
         else:
             value = _load_file_value(context, value_spec.blobFileValue, dtype)
 
+        target_np_dtype = types.nptype_from_builtin(dtype)
         if dtype in helper.IMMEDIATE_VALUE_TYPES_IN_BYTES:
-            value = np.frombuffer(value, types.nptype_from_builtin(dtype)).reshape(
-                shape
-            )
+            value = _restore_np_from_bytes_value(value, dtype, shape).astype(target_np_dtype)
         elif dtype == types.str and shape == ():
             value = str(value[0])
-        elif dtype in (types.fp32, types.str, types.bool, types.int32, types.int64):
-            value = (
-                np.array(value).astype(types.nptype_from_builtin(dtype)).reshape(shape)
-            )
+        elif dtype in (
+            types.fp32,
+            types.str,
+            types.bool,
+            types.int16,
+            types.uint16,
+            types.int32,
+            types.int64,
+        ):
+            value = np.array(value).astype(target_np_dtype).reshape(shape)
         else:
             raise ValueError("Invalid dtype for tensor value")
+
+    elif value_spec_type == "dictionaryType":
+        assert value_spec.WhichOneof("value") == "immediateValue", "dict must be immediate value"
+        return _load_immediate_value(context, value_spec.immediateValue)
+
     else:
-        raise NotImplementedError("Only value of tensorType implemented yet")
+        raise NotImplementedError(
+            f"Deserialization from milproto {value_spec_type} to pymil is not implemented yet"
+        )
 
     if not is_tensor and not isinstance(value, str):
         value = types.nptype_from_builtin(dtype)(value.item())
@@ -176,9 +229,9 @@ def _load_value(context, value_spec):
 def _create_var_from_spec(spec):
     """
     This helper function is used for creating PyMIL Var/ListVar from the proto spec.
-    Mainly used for the contruction of the control flow ops.
+    Mainly used for the construction of the control flow ops.
     """
-    assert isinstance(spec, pm.NamedValueType)
+    assert isinstance(spec, proto.MIL_pb2.NamedValueType)
     sym_type = proto_to_types(spec.type)
     name = spec.name
     if types.is_list(sym_type):
@@ -214,7 +267,7 @@ def _create_nested_blocks(context, op_spec):
         for v in input_vars:
             context.register_var_with_name(v.name, v)
 
-        # In pymil, the outer_op for a block can only be None if the block is a Functino.
+        # In pymil, the outer_op for a block can only be None if the block is a Function.
         # As the result, we use a dummy outer_op here for block creation, and set it to
         # the legit op later on in _set_outer_op_for_nested_blocks
         dummy = mb.const(val=0.)
@@ -255,20 +308,51 @@ def _set_inputs_for_control_flow_op(inputs, blocks, op_type):
 
 def _load_const_op(context, op_spec):
     inputs = {k: _load_value(context, v) for k, v in op_spec.attributes.items()}
-    pymil_var = getattr(mb, op_spec.type)(**inputs)
-    context.register_var_with_name(op_spec.outputs[0].name, pymil_var)
+    if len(op_spec.inputs) > 0:
+        for param_name, argument in op_spec.inputs.items():
+            vars = []
+            for binding in argument.arguments:
+                binding_type = binding.WhichOneof("binding")
+                if binding_type == "name":
+                    vars.append(context.get_var_from_name(binding.name))
+                elif binding_type == "value":
+                    vars.append(_load_value(context, binding.value))
+                else:
+                    raise ValueError(f"Invalid binding_type {binding_type}")
+            if len(vars) == 1:
+                inputs[param_name] = vars[0]
+            else:
+                inputs[param_name] = vars
+
+    output_var = getattr(mb, op_spec.type)(**inputs)
+    if "val" in op_spec.attributes:
+        if hasattr(op_spec.attributes["val"], "blobFileValue"):
+            filevalue_spec = op_spec.attributes["val"].blobFileValue
+            filename = filevalue_spec.fileName.split("/")[-1]
+            if filename != "weight.bin":
+                output_var.op.weight_key = filename.split(".")[0]
 
 
-def _load_operation(context, op_spec):
-    if not isinstance(op_spec, pm.Operation):
+    if not isinstance(output_var, (tuple, list)):
+        output_var = [output_var]
+    if len(output_var) != len(op_spec.outputs):
+        raise AssertionError(
+            "Mismatch between number of outputs in operation specification vs PyMIL outputs"
+        )
+    for spec, var in zip(op_spec.outputs, output_var):
+        context.register_var_with_name(spec.name, var)
+
+
+def _load_operation(context: TranscriptionContext, op_spec: proto.MIL_pb2.Operation):
+    if not isinstance(op_spec, proto.MIL_pb2.Operation):
         raise TypeError("Invalid Operation spec object")
 
     op_type = op_spec.type
     if op_type == "const" or "constexpr_" in op_type:
         if op_spec.blocks:
             raise ValueError("const / constexpr operation can't have any block")
-        if op_spec.inputs:
-            raise ValueError("const / constexpr operation can't have any input")
+        if op_type == "const" and op_spec.inputs:
+            raise ValueError("const operation can't have any input")
         _load_const_op(context, op_spec)
 
     else:
@@ -286,7 +370,7 @@ def _load_operation(context, op_spec):
         #       For that case, we directly create a constant variable.
 
         # (ii)  Create nested blocks for control flow operations:
-        #       The Python functinoal input arguments for control flow ops cannot be recovered from milproto -> pymil conversion,
+        #       The Python functional input arguments for control flow ops cannot be recovered from milproto -> pymil conversion,
         #       for instance, the _body, _cond for mb.while_loop and _true_fn, _false_fn for mb.cond are not invertible
         #       Hence, here we directly create the nested blocks from the proto, and set them to mb.while_loop.blocks / mb.cond.blocks.
         #       Note that, when creating a block, PyMIL required an outer_op, which should be the control flow operation itself. However,
@@ -295,7 +379,7 @@ def _load_operation(context, op_spec):
         #       for the creation of the block.
 
         # (iii) Create PyMIL operation using inputs / blocks
-        #       Note that for the control flow cases, we create dummy functional inputs, and use the exisiting block to create the op.
+        #       Note that for the control flow cases, we create dummy functional inputs, and use the existing block to create the op.
 
         # (iv)  Set the outer_op for control flow
         #       Once the operation is created, we replace the dummy outer_op with the legit one, to make it a valid PyMIL program
@@ -317,7 +401,7 @@ def _load_operation(context, op_spec):
                     value_spec = binding.value
                     assert value_spec.WhichOneof("value") == "immediateValue"
                     assert value_spec.immediateValue.WhichOneof("value") == "list"
-                    list_value = _load_immediate_value(value_spec.immediateValue)
+                    list_value = _load_immediate_value(context, value_spec.immediateValue)
                     values = []
                     for value_spec in list_value:
                         values.append(_load_value(context, value_spec))
@@ -325,18 +409,32 @@ def _load_operation(context, op_spec):
                     vars.append(var)
                 else:
                     raise NotImplementedError("Binding {} not yet implemented".format(binding_type))
-            op_cls = _SSAOpRegistry._get_core_op_cls(op_type)
-            if len(vars) == 1 and not isinstance(
-                op_cls.input_spec.input_types[param_name], TupleInputType
-            ):
+
+            if op_type == "write_state":
                 inputs[param_name] = vars[0]
             else:
-                inputs[param_name] = vars
+                op_cls = _SSAOpRegistry._get_core_op_cls(op_type)
+                if len(vars) == 1 and not isinstance(
+                    op_cls.input_spec.input_types[param_name], TupleInputType
+                ):
+                    inputs[param_name] = vars[0]
+                else:
+                    inputs[param_name] = vars
 
         blocks = _create_nested_blocks(context, op_spec)
         _set_inputs_for_control_flow_op(inputs, blocks, op_type)
 
-        output_var = getattr(mb, op_type)(**inputs)
+        # write_state is translated into coreml_update_state
+        if op_type == "write_state":
+            new_inputs = {
+                "state": inputs["input"],
+                "value": inputs["data"],
+            }
+            getattr(mb, "coreml_update_state")(**new_inputs)
+            return
+        else:
+            output_var = getattr(mb, op_type)(**inputs)
+
         if not isinstance(output_var, (tuple, list)):
             output_var = [output_var]
 
@@ -363,7 +461,34 @@ def _load_operation(context, op_spec):
 
 
 def _load_block(context, block_spec):
-    if not isinstance(block_spec, pm.Block):
+    def _try_to_merge_state_ops():
+        """
+        We detect the pattern of:
+
+            %1 = coreml_update_state(state=%state, value=%value)
+            %2 = read_state(input=%state)
+
+        and transform it into:
+
+            %2 = coreml_update_state(state=%state, value=%value)
+        """
+        block = curr_block()
+
+        if len(block.operations) < 2:
+            return
+
+        op_1, op_2 = block.operations.end.prev.op, block.operations.end.op
+        if op_1.op_type != "coreml_update_state" or op_2.op_type != "read_state":
+            return
+        if op_1.state != op_2.input:
+            return
+
+        var_1, var_2 = op_1.outputs[0], op_2.outputs[0]
+        var_1.name = var_2.name
+        context.register_var_with_name(var_1.name, var_1)
+        block.remove_ops([op_2])
+
+    if not isinstance(block_spec, proto.MIL_pb2.Block):
         raise TypeError("Invalid Block spec object")
 
     if block_spec.attributes:
@@ -373,6 +498,7 @@ def _load_block(context, block_spec):
     output_vars = []
     for op_spec in block_spec.operations:
         _load_operation(context, op_spec)
+        _try_to_merge_state_ops()
 
     for proto_output_name in block_outputs:
         output_vars.append(context.get_var_from_name(proto_output_name))
@@ -383,7 +509,7 @@ def _load_block(context, block_spec):
 
 
 def _load_function(context, func_spec, spec_version):
-    if not isinstance(func_spec, pm.Function):
+    if not isinstance(func_spec, proto.MIL_pb2.Function):
         raise TypeError("Invalid Function spec object")
 
     if func_spec.attributes:
@@ -394,11 +520,19 @@ def _load_function(context, func_spec, spec_version):
         name = named_value_type.name
         valuetype = proto_to_types(named_value_type.type)
 
-        if not types.is_tensor(valuetype):
-            raise ValueError("Functions inputs can only be tensors")
-        func_inputs[name] = Placeholder(
-            sym_shape=valuetype.get_shape(), dtype=valuetype.get_primitive(), name=name
-        )
+        if types.is_tensor(valuetype):
+            func_inputs[name] = Placeholder(
+                sym_shape=valuetype.get_shape(), dtype=valuetype.get_primitive(), name=name
+            )
+        elif types.is_state(valuetype):
+            func_inputs[name] = StateTensorPlaceholder(
+                sym_shape=valuetype.wrapped_type().get_shape(),
+                dtype=valuetype.wrapped_type().get_primitive(),
+                name=name,
+            )
+        else:
+            raise ValueError(f"Functions input of type {valuetype} not supported.")
+
         context.register_var_with_name(name, func_inputs[name].outputs[0])
 
     opset = func_spec.opset
@@ -411,11 +545,24 @@ def _load_function(context, func_spec, spec_version):
     return pymil_func
 
 
+def _load_program_spec_attributes(
+    context: TranscriptionContext,
+    program_spec: proto.MIL_pb2.Program,
+    pymil_program: mil.Program,
+) -> None:
+    for attr_name, attr_spec in program_spec.attributes.items():
+        # No need to load these attributes
+        if attr_name in ("buildInfo",):
+            pass
+        else:
+            raise ValueError(f"Invalid attribute {attr_name} for program")
+
+
 def load_mil_proto(program_spec, specification_version, file_weights_dir=""):
     """
     Load in-memory Proto specification of MILSpec.Program(.Proto) object to PyMIL
     """
-    if not isinstance(program_spec, pm.Program):
+    if not isinstance(program_spec, proto.MIL_pb2.Program):
         raise TypeError("Invalid Program spec object")
 
     if program_spec.docString:
@@ -425,15 +572,13 @@ def load_mil_proto(program_spec, specification_version, file_weights_dir=""):
         raise ValueError("Invalid program version")
 
     context = TranscriptionContext(file_weights_dir)
-    pymil_program = Program()
+    pymil_program = mil.Program()
     for func_name, func_spec in program_spec.functions.items():
         pymil_program.add_function(
             func_name, _load_function(context, func_spec, specification_version)
         )
 
-    for attr_name, attr_spec in program_spec.attributes.items():
-        if attr_name not in ("buildInfo",):
-            raise ValueError("Invalid attribute for program")
+    coremltools.converters.mil.frontend.milproto.load._load_program_spec_attributes(context, program_spec, pymil_program)
 
     return pymil_program
 
@@ -444,7 +589,7 @@ def load(model_spec, specification_version, file_weights_dir="", **kwargs):
 
     Set force_spec_version to force override the spec version.
     """
-    if not isinstance(model_spec, ml.Model):
+    if not isinstance(model_spec, proto.Model_pb2.Model):
         raise TypeError("Invalid Model sepc object")
 
     if specification_version < model_spec.specificationVersion:
