@@ -70,7 +70,7 @@ class TestCustomMatMul:
             x_type = self.x.dtype
             x_shape = self.x.shape
             y_shape = self.y.shape
-            # For illustration purpose, assumming getting valid shape
+            # For illustration purpose, assuming getting valid shape
             # Ideally, should consider transpose_?, ?_is_sparse parameters into consideration
             # for computing output shape
             return types.tensor(x_type, [x_shape[0], y_shape[1]])
@@ -159,7 +159,7 @@ class TestCustomMatMul:
         assert layers[-1].custom is not None, "Expecting a custom layer"
         assert (
             "SparseMatMul" == layers[-1].custom.className
-        ), "Custom Layer class name mis-match"
+        ), "Custom Layer class name mismatch"
         assert (
             transpose_a == layers[-1].custom.parameters["transpose_x"].boolValue
         ), "Incorrect parameter value k"
@@ -279,7 +279,7 @@ class TestCustomTopK:
         assert layers[-1].custom is not None, "Expecting a custom layer"
         assert (
             "TopK" == layers[-1].custom.className
-        ), "Custom Layer class name mis-match"
+        ), "Custom Layer class name mismatch"
         assert (
             k == layers[-1].custom.parameters["k"].intValue
         ), "Incorrect parameter value k"
