@@ -15,10 +15,10 @@ import coremltools.converters as converter
 import coremltools.proto.FeatureTypes_pb2 as ft
 from coremltools import EnumeratedShapes, ImageType, RangeDim, TensorType
 from coremltools._deps import _HAS_TF_1, _IS_MACOS, MSG_TF1_NOT_FOUND
-from coremltools.converters.mil.frontend.tensorflow.converter import \
-    TFConverter
+from coremltools.converters.mil.frontend.tensorflow.converter import TFConverter
 from coremltools.converters.mil.frontend.tensorflow.test.testing_utils import (
     TensorFlowBaseTest, get_tf_keras_io_names, make_tf_graph)
+from coremltools.converters.mil.testing_reqs import backends
 from coremltools.converters.mil.testing_utils import random_gen
 
 tf = pytest.importorskip("tensorflow")
@@ -38,7 +38,11 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
         if os.path.exists(self.saved_model_dir):
             shutil.rmtree(self.saved_model_dir)
 
-    def test_infer_inputs(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_infer_inputs(self, backend):
         x_shape = (3, 4, 5)
 
         @make_tf_graph([x_shape])
@@ -49,17 +53,19 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
         if not isinstance(outputs, (tuple, list)):
             outputs = [outputs]
 
-        output_names = [
-            j if isinstance(j, str) else j.op.name for j in outputs
-        ]
-        mlmodel = converter.convert(model, outputs=output_names)
+        output_names = [j if isinstance(j, str) else j.op.name for j in outputs]
+        mlmodel = converter.convert(model, outputs=output_names, convert_to=backend[0])
         assert mlmodel is not None
 
         input_values = [random_gen(x_shape, -10.0, 10.0)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(model, input_dict, outputs)
 
-    def test_infer_outputs(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_infer_outputs(self, backend):
         x_shape = (3, 4, 5)
 
         @make_tf_graph([x_shape])
@@ -67,17 +73,21 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
             return tf.nn.relu(x)
 
         model, inputs, outputs = build_model
-        input_name = (
-            inputs[0] if isinstance(inputs[0], str) else inputs[0].op.name
+        input_name = inputs[0] if isinstance(inputs[0], str) else inputs[0].op.name
+        mlmodel = converter.convert(
+            model, inputs=[TensorType(input_name, (3, 4, 5))], convert_to=backend[0]
         )
-        mlmodel = converter.convert(model, inputs=[TensorType(input_name, (3, 4, 5))])
         assert mlmodel is not None
 
         input_values = [random_gen(x_shape, -10.0, 10.0)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(model, input_dict, outputs)
 
-    def test_infer_inputs_and_outputs(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_infer_inputs_and_outputs(self, backend):
         x_shape = (3, 4, 5)
 
         @make_tf_graph([x_shape])
@@ -85,14 +95,18 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
             return tf.nn.relu(x)
 
         model, inputs, outputs = build_model
-        mlmodel = converter.convert(model)
+        mlmodel = converter.convert(model, convert_to=backend[0])
         assert mlmodel is not None
 
         input_values = [random_gen(x_shape, -10.0, 10.0)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(model, input_dict, outputs)
 
-    def test_extract_sub_model(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_extract_sub_model(self, backend):
         x_shape = (3, 4, 5)
         y_shape = (3, 4, 5)
 
@@ -105,11 +119,15 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
             first_output_name = outputs[0]
         else:
             first_output_name = outputs[0].name.split(":")[0]
-        mlmodel = converter.convert(model, outputs=[first_output_name])
+        mlmodel = converter.convert(model, outputs=[first_output_name], convert_to=backend[0])
         assert mlmodel is not None
 
-    def test_auto_image_nhwc_input_names(self):
-        x_shape = (4, 5, 3)
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_auto_image_nhwc_input_names(self, backend):
+        x_shape = (4, 5, 3) if backend[0] == "neuralnetwork" else (1, 4, 5, 3)
 
         @make_tf_graph([x_shape])
         def build_model(x):
@@ -117,11 +135,15 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
 
         model, inputs, outputs = build_model
 
-        mlmodel = converter.convert(model, inputs=[ImageType()])
+        mlmodel = converter.convert(model, inputs=[ImageType()], convert_to=backend[0])
         assert mlmodel is not None
 
-    def test_auto_image_nchw_input_names(self):
-        x_shape = (3, 4, 5)
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_auto_image_nchw_input_names(self, backend):
+        x_shape = (3, 4, 5) if backend[0] == "neuralnetwork" else (1, 3, 4, 5)
 
         @make_tf_graph([x_shape])
         def build_model(x):
@@ -129,12 +151,14 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
 
         model, inputs, outputs = build_model
 
-        mlmodel = converter.convert(model, inputs=[ImageType(channel_first=True)])
+        mlmodel = converter.convert(
+            model, inputs=[ImageType(channel_first=True)], convert_to=backend[0]
+        )
         assert mlmodel is not None
 
     @pytest.mark.parametrize(
         "target",
-        [ct.target.iOS13, ct.target.macOS15, ct.target.watchOS6, ct.target.tvOS13],
+        [ct.target.iOS13, ct.target.macOS10_15, ct.target.watchOS6, ct.target.tvOS13],
     )
     def test_invalid_deployment_target_cumsum(self, target):
         x_shape = (3, 4, 5)
@@ -155,7 +179,7 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
 
     @pytest.mark.parametrize(
         "target",
-        [ct.target.iOS14, ct.target.macOS16, ct.target.watchOS7, ct.target.tvOS14],
+        [ct.target.iOS14, ct.target.macOS10_16, ct.target.watchOS7, ct.target.tvOS14],
     )
     def test_valid_deployment_target_cumsum(self, target):
         x_shape = (3, 4, 5)
@@ -169,7 +193,11 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
         # successful conversion
         converter.convert(model, minimum_deployment_target=target)
 
-    def test_invalid_output_names(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_invalid_output_names(self, backend):
         x_shape = (3, 4, 5)
 
         @make_tf_graph([x_shape])
@@ -178,10 +206,16 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
 
         model, inputs, outputs = build_model
         with pytest.raises(AssertionError) as e:
-            converter.convert(model, source=frontend, outputs=["invalid_name"])
+            converter.convert(
+                model, source=frontend, outputs=["invalid_name"], convert_to=backend[0]
+            )
         e.match(r".* is not in graph")
 
-    def test_missing_placeholder_shape(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_missing_placeholder_shape(self, backend):
         x_shape = None  # Missing Placeholder shape
 
         @make_tf_graph([x_shape])
@@ -190,15 +224,20 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
 
         model, inputs, outputs = build_model
         with pytest.raises(ValueError) as e:
-            converter.convert(model, source=frontend)
+            converter.convert(model, source=frontend, convert_to=backend[0])
             e.match(r"Unable to determine the shape of input .*")
 
-        mlmodel = converter.convert(model, source=frontend,
-                                    inputs=[ct.TensorType(shape=(1,))])
+        mlmodel = converter.convert(
+            model, source=frontend, inputs=[ct.TensorType(shape=(1,))], convert_to=backend[0]
+        )
         assert mlmodel is not None
 
     @pytest.mark.skip(reason="Rank-0 input is not supported")
-    def test_scalar_placeholder_shape(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_scalar_placeholder_shape(self, backend):
         x_shape = ()  # Scalar Placeholder Shape
 
         @make_tf_graph([x_shape])
@@ -206,14 +245,18 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
             return tf.nn.relu(x)
 
         model, inputs, outputs = build_model
-        mlmodel = converter.convert(model, source=frontend)
+        mlmodel = converter.convert(model, source=frontend, convert_to=backend[0])
         assert mlmodel is not None
 
         input_values = [random_gen(x_shape, -10.0, 10.0)]
         input_dict = dict(zip(inputs, input_values))
         TensorFlowBaseTest.run_compare_tf(model, input_dict, outputs)
 
-    def test_shaping_utils(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_shaping_utils(self, backend):
         @make_tf_graph([(None, 4, 5)])
         def build_flexible_model(x):
             return tf.nn.relu(x)
@@ -223,13 +266,16 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
         output_name = TFConverter._get_tensor_name(outputs[0])
 
         # static-Flexible shape
-        mlmodel = converter.convert(
-            model,
-            inputs=[
+        if backend[0] == "neuralnetwork":
+            inputs = [
                 # Use TF's input shapes (None, 4, 5)
                 TensorType(name=input_name)
-            ],
-            outputs=[output_name]
+            ]
+        else:
+            inputs = [TensorType(name=input_name, shape=(RangeDim(upper_bound=3), 4, 5))]
+
+        mlmodel = converter.convert(
+            model, inputs=inputs, outputs=[output_name], convert_to=backend[0]
         )
         assert mlmodel is not None
         input_values = [random_gen((3, 4, 5), -10.0, 10.0)]
@@ -239,10 +285,10 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
             np.allclose(ret[output_name], np.maximum(input_values[0], 0.0))
 
         # Enumerate shape
-        inputs_shape = [
-            TensorType(input_name, EnumeratedShapes(shapes=[(3, 4, 5), (4, 4, 5)]))
-        ]
-        mlmodel = converter.convert(model, inputs=inputs_shape, outputs=[output_name])
+        inputs_shape = [TensorType(input_name, EnumeratedShapes(shapes=[(3, 4, 5), (4, 4, 5)]))]
+        mlmodel = converter.convert(
+            model, inputs=inputs_shape, outputs=[output_name], convert_to=backend[0]
+        )
         assert mlmodel is not None
         input_values = [random_gen((3, 4, 5), -10.0, 10.0)]
         input_dict = {input_name: input_values[0]}
@@ -264,7 +310,9 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
 
         # Ranged shape
         inputs_shape = [TensorType(input_name, [RangeDim(3, 5), 4, 5])]
-        mlmodel = converter.convert(model, inputs=inputs_shape, outputs=[output_name])
+        mlmodel = converter.convert(
+            model, inputs=inputs_shape, outputs=[output_name], convert_to=backend[0]
+        )
         assert mlmodel is not None
         input_values = [random_gen((3, 4, 5), -10.0, 10.0)]
         input_dict = {input_name: input_values[0]}
@@ -284,13 +332,17 @@ class TestTfModelInputsOutputs(TensorFlowBaseTest):
                 input_dict = {input_name: input_values[0]}
                 ret = mlmodel.predict(input_dict)
 
-    def test_default_data_types(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_default_data_types(self, backend):
         @make_tf_graph([(2, 2)])
         def build_model(x):
             return tf.nn.relu(x)
 
         model, inputs, outputs = build_model
-        mlmodel = converter.convert(model)
+        mlmodel = converter.convert(model, convert_to=backend[0])
         assert mlmodel is not None
         spec = mlmodel.get_spec()
 
@@ -316,16 +368,27 @@ class TestTf1ModelFormats:
         if os.path.exists(self.saved_model_dir):
             shutil.rmtree(self.saved_model_dir)
 
-    def test_graph_def(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_graph_def(self, backend):
         with tf.Graph().as_default() as graph:
             x = tf.placeholder(tf.float32, shape=(3, 4, 5))
             out = tf.nn.relu(x)
             mlmodel = converter.convert(
-                graph, inputs=[TensorType(x.op.name, (3, 4, 5))], outputs=[out.op.name]
+                graph,
+                inputs=[TensorType(x.op.name, (3, 4, 5))],
+                outputs=[out.op.name],
+                convert_to=backend[0],
             )
             assert mlmodel is not None
 
-    def test_graph_def_file(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_graph_def_file(self, backend):
         with tf.Graph().as_default() as graph:
             x = tf.placeholder(tf.float32, shape=(3, 4, 5))
             out = tf.nn.relu(x)
@@ -336,38 +399,46 @@ class TestTf1ModelFormats:
             self.model_path_pb,
             inputs=[TensorType(x.op.name, (3, 4, 5))],
             outputs=[out.op.name],
+            convert_to=backend[0],
         )
         assert mlmodel is not None
 
-    def test_saved_model_from_simple_save(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_saved_model_from_simple_save(self, backend):
         with tf.compat.v1.Session() as sess:
             x = tf.placeholder(shape=(1, 3, 5), dtype=tf.float32)
             y = tf.nn.relu(x)
             inputs = {"x": x}
             outputs = {"y": y}
-            tf.compat.v1.saved_model.simple_save(
-                sess, self.saved_model_dir, inputs, outputs
-            )
-        mlmodel = converter.convert(self.saved_model_dir)
+            tf.compat.v1.saved_model.simple_save(sess, self.saved_model_dir, inputs, outputs)
+        mlmodel = converter.convert(self.saved_model_dir, convert_to=backend[0])
         assert mlmodel is not None
 
-    def test_tf_keras(self):
-        keras_model = tf.keras.Sequential(
-            [tf.keras.layers.ReLU(input_shape=(4, 5), batch_size=3)]
-        )
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_tf_keras(self, backend):
+        keras_model = tf.keras.Sequential([tf.keras.layers.ReLU(input_shape=(4, 5), batch_size=3)])
         input_names, output_names = get_tf_keras_io_names(keras_model)
         mlmodel = converter.convert(
             keras_model,
             inputs=[TensorType(input_names[0], (3, 4, 5))],
             outputs=["Identity"],
             source=frontend,
+            convert_to=backend[0],
         )
         assert mlmodel is not None
 
-    def test_tf_keras_hdf5_file(self):
-        keras_model = tf.keras.Sequential(
-            [tf.keras.layers.ReLU(input_shape=(4, 5), batch_size=3)]
-        )
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_tf_keras_hdf5_file(self, backend):
+        keras_model = tf.keras.Sequential([tf.keras.layers.ReLU(input_shape=(4, 5), batch_size=3)])
         keras_model.save(self.model_path_h5)
         input_names, output_names = get_tf_keras_io_names(keras_model)
         mlmodel = converter.convert(
@@ -375,37 +446,55 @@ class TestTf1ModelFormats:
             inputs=[TensorType(input_names[0], (3, 4, 5))],
             outputs=["Identity"],
             source=frontend,
+            convert_to=backend[0],
         )
         assert mlmodel is not None
 
-    def test_model_metadata(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_model_metadata(self, backend):
         with tf.Graph().as_default() as graph:
             x = tf.placeholder(tf.float32, shape=(3, 4, 5))
             out = tf.nn.relu(x)
             mlmodel = converter.convert(
-                graph, inputs=[TensorType(x.op.name, (3, 4, 5))], outputs=[out.op.name]
+                graph,
+                inputs=[TensorType(x.op.name, (3, 4, 5))],
+                outputs=[out.op.name],
+                convert_to=backend[0],
             )
             metadata_keys = mlmodel.get_spec().description.metadata.userDefined
             assert "com.github.apple.coremltools.version" in metadata_keys
             assert "com.github.apple.coremltools.source" in metadata_keys
             assert "tensorflow==1." in metadata_keys["com.github.apple.coremltools.source"]
 
-    def test_invalid_format_none(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_invalid_format_none(self, backend):
         with pytest.raises(NotImplementedError) as e:
-            converter.convert(None, source="tensorflow")
+            converter.convert(None, source="tensorflow", convert_to=backend[0])
             e.match(r"Expected model format: .* .pb")
 
-    def test_invalid_format_invalid_extension(self):
-        _, invalid_filename = tempfile.mkstemp(
-            suffix=".invalid", prefix=self.saved_model_dir
-        )
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_invalid_format_invalid_extension(self, backend):
+        _, invalid_filename = tempfile.mkstemp(suffix=".invalid", prefix=self.saved_model_dir)
         with pytest.raises(NotImplementedError) as e:
-            converter.convert(invalid_filename, source="tensorflow")
+            converter.convert(invalid_filename, source="tensorflow", convert_to=backend[0])
             e.match(r"Expected model format: .* .pb")
 
-    def test_invalid_converter_source(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_invalid_converter_source(self, backend):
         with pytest.raises(ValueError) as e:
-            converter.convert(None, source="invalid")
+            converter.convert(None, source="invalid", convert_to=backend[0])
             expected_msg = r'Unrecognized value of argument "source": .*'
             e.match(expected_msg)
 
@@ -428,8 +517,12 @@ class TestTf1ModelFormats:
             converter.convert(graph, convert_to="invalid", source="tensorflow")
             e.match(r"Backend converter .* not implemented")
 
-    def test_invalid_format_non_exist(self):
+    @pytest.mark.parametrize(
+        "backend",
+        backends,
+    )
+    def test_invalid_format_non_exist(self, backend):
         non_exist_filename = self.model_path_pb.replace(".pb", "_non_exist.pb")
         with pytest.raises(ValueError) as e:
-            converter.convert(non_exist_filename, source="tensorflow")
+            converter.convert(non_exist_filename, source="tensorflow", convert_to=backend[0])
             e.match(r"Input model .* does not exist")
