@@ -8,17 +8,8 @@ from coremltools.converters.mil.frontend.tensorflow.basic_graph_ops import \
 from coremltools.converters.mil.frontend.tensorflow.converter import \
     TFConverter
 
-from .ssa_passes.tf_passes import tensorflow_passes as tensorflow2_passes
-
 
 class TF2Converter(TFConverter):
-    def __init__(self, tf_ssa, inputs=None, outputs=None, opset_version=None):
-        TFConverter.__init__(self, tf_ssa, inputs, outputs, opset_version)
-
-        # Overwrite tensorflow_passes
-        # TF 2.x uses different set of graph passes
-        self.tensorflow_passes = tensorflow2_passes
-
     def _get_stack(self, tfssa, root="main"):
         """
         Overwrite TFConverter._get_stack() as TF2 generates different sub-graphs.
