@@ -3,13 +3,13 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-import logging
-
+from coremltools import _logger as logger
 from coremltools.converters.mil.mil import Builder as mb
 from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
 from coremltools.converters.mil.mil.passes.helper import block_context_manager
 from coremltools.converters.mil.mil.passes.pass_registry import register_pass
 
+
 @block_context_manager
 def _remove_vacuous_cond_block(block):
     num_changes = 0
@@ -115,4 +115,4 @@ class remove_vacuous_cond(AbstractGraphPass):
         for f_name, f in prog.functions.items():
             num_changes = _remove_vacuous_cond_block(f)
             msg = "remove_vacuous_cond: changed {} ops in function '{}'"
-            logging.info(msg.format(num_changes, f_name))
+            logger.info(msg.format(num_changes, f_name))
