@@ -4,29 +4,34 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 from collections import OrderedDict
+from typing import Any, Dict, List, Optional, Tuple, Union
 
+import numpy as np
 import torch
-from torch.fx.node import Node
 
 from coremltools import _logger as logger
+from coremltools.converters.mil.input_types import TensorType
 
-from .edgeir_utils import extract_inputs_from_edge_program
+from .exir_utils import extract_io_from_exir_program
+from .torch_op_registry import _TORCH_OPS_REGISTRY
 from .torchscript_utils import _expand_and_optimize_ir
-
-_DEFAULT_OP_NAMESPACES = set(["aten", "prim"])
+from .utils import TORCH_DTYPE_TO_NUM, sanitize_op_kind
 
 
-def _make_ssa_name(name):
+def _make_ssa_name(name: Optional[Union[str, int]]) -> str:
     """
     Converts a symbol name (string) into an SSA name, by prepending '%'.
+    If the name is a parameter value (int), directly printing it without prepending '%'.
     Only used for pretty printing the graph.
     """
     if name is None:
         return "None"
+    if type(name) is int:
+        return str(name)
     return "%" + name
 
 
-def _ssa_name_list(names):
+def _ssa_name_list(names: List[Optional[Union[str, int]]]) -> List[str]:
     """
     Take a list of symbol names (strings) and return them as SSA names. Only
     used for pretty printing the graph.
@@ -34,7 +39,7 @@ def _ssa_name_list(names):
     return [_make_ssa_name(x) for x in names]
 
 
-def _find_new_name(old_name, node_names):
+def _find_new_name(old_name: str, node_names: List[str]) -> str:
     """
     Disambiguate a node's name from a list of existing node names by adding
     successively larger integers.
@@ -47,7 +52,7 @@ def _find_new_name(old_name, node_names):
     return new_name
 
 
-def _replace_in_list(ls, old_val, new_val):
+def _replace_in_list(ls: List[Any], old_val: Any, new_val: Any) -> None:
     """Helper function to replace a value in a list."""
     try:
         idx = ls.index(old_val)
@@ -62,11 +67,17 @@ class InternalTorchIRBlock:
     coremltools internal representation of a torch IR block.
     """
 
-    def __init__(self, parent=None, nodes=None, inputs=None, outputs=None):
+    def __init__(
+        self,
+        parent: Optional["InternalTorchIRNode"] = None,
+        nodes: Optional[List["InternalTorchIRNode"]] = None,
+        inputs: Optional[List[str]] = None,
+        outputs: Optional[List[str]] = None,
+    ):
         """
         Arguments:
             parent: The InternalTorchIRNode this block belongs to.
-            nodes: list of InternalTorchIRNodes in the block
+            nodes: list of InternalTorchIRNode in the block
             inputs: list of input symbols.
             outputs: list of output symbols.
         """
@@ -77,10 +88,10 @@ class InternalTorchIRBlock:
         self.parent = parent
 
     @classmethod
-    def from_edgeir_block(cls, block, parent):
+    def from_exir_block(cls, block, parent):
         raise NotImplementedError(
-            "EdgeIR: Support for Ops containing blocks not implemented yet"
-        )  # TODO: rdar://115846569 ([Executorch] Handle control flow ops from edge ir)
+            "EXIR: Support for Ops containing blocks not implemented yet"
+        )  # TODO: rdar://115846569 ([Executorch] Handle control flow ops from EXIR)
 
     @classmethod
     def from_torchscript_block(cls, block, parent):
@@ -151,13 +162,16 @@ class InternalTorchIRNode:
 
     def __init__(
         self,
-        kind,
-        inputs,
-        outputs,
-        name=None,
-        parent=None,
-        attr=None,
-        blocks=None,
+        kind: str,
+        inputs: List[str],
+        outputs: List[str],
+        kwinputs: Optional[Dict[str, str]] = None,
+        name: Optional[str] = None,
+        parent: Optional[Union["InternalTorchIRGraph", "InternalTorchIRBlock"]] = None,
+        attr: Optional[Dict[str, Any]] = None,
+        blocks: Optional[List["InternalTorchIRBlock"]] = None,
+        model_hierarchy: Optional[str] = None,
+        meta: Optional[Dict] = None,
     ):
         """
         Arguments:
@@ -165,9 +179,12 @@ class InternalTorchIRNode:
             kind: the kind (op) of the node.
             inputs: list of input symbols.
             outputs: list of output symbols.
+            kwinputs: dict of keyword input symbols.
             parent: The InternalTorchIRGraph/Block this node belongs to.
             attr:  dict of named attributes.
             blocks: list of InternalTorchIRBlock.
+            model_hierarchy: str represents TorchScript node's model hierarchy.
+            meta: A dictionary of torch fx node metadata inherited from torch.fx.Node.meta
         """
         if not name and not outputs:
             self.name = ""
@@ -177,20 +194,18 @@ class InternalTorchIRNode:
         self.kind = kind
         self.inputs = inputs
         self.outputs = outputs
+        self.kwinputs = kwinputs
         self.parent = parent
         self.attr = attr if attr is not None else {"value": None}
         self.blocks = blocks if blocks is not None else []
+        self.model_hierarchy = model_hierarchy
+        self.meta = meta
 
     @classmethod
     def from_torchscript_node(cls, node, parent):
         inputs = [_input.debugName() for _input in node.inputs()]
         outputs = [output.debugName() for output in node.outputs()]
-        namespace = node.kind().split("::")[0].lower()
-        if namespace in _DEFAULT_OP_NAMESPACES:
-            # We conventionally skip the aten/prim namespaces in our naming.
-            kind = node.kind().split("::")[-1].lower()
-        else:
-            kind = node.kind().lower()
+        kind = sanitize_op_kind(node.kind())
 
         attr = {name: getattr(node, node.kindOf(name))(name) for name in node.attributeNames()}
         if "value" not in attr:
@@ -201,6 +216,7 @@ class InternalTorchIRNode:
             attr["value"] = bool(attr["value"])
 
         # On rare occassions, a node has no outputs. In that case, the node's
+        # On rare occasions, a node has no outputs. In that case, the node's
         # name will be its kind. However, this no longer guarantees the node's
         # name is unique. It will be up to the graph constructing the node to
         # make sure names are unique.
@@ -214,6 +230,7 @@ class InternalTorchIRNode:
             outputs=outputs,
             attr=attr,
             blocks=None,
+            model_hierarchy=node.getModuleHierarchy(),
         )
         internal_node.blocks = [
             InternalTorchIRBlock.from_torchscript_block(block=b, parent=internal_node)
@@ -222,27 +239,40 @@ class InternalTorchIRNode:
         return internal_node
 
     @classmethod
-    def from_edgeir_node(cls, node):
-        def get_arguments(alist):
+    def from_exir_node(cls, node):
+        def get_arguments(alist: List) -> Tuple:
             args = []
             for i in alist:
-                if isinstance(i, Node):
+                if isinstance(i, torch.fx.Node):
                     args.append(i.name)
                 elif isinstance(i, torch.fx.immutable_collections.immutable_list):
                     args.append(get_arguments(i))
-                elif isinstance(i, (int, float)):
+                elif isinstance(i, (int, float, str)):
                     args.append(i)
+                # This is necessitated by backward compatibility:
+                # * TorchScript used to store dtype as integers/enums
+                # * Subsequently, we built our PyTorch converter based on numbered dtypes
+                # * Now EXIR uses dtype directly...
+                # * Until refactoring EXIR converter to be independent from TorchScript converter,
+                #   we have to map dtype to number ourselves
+                #   to leverage the existing TorchScript converter infra
+                elif isinstance(i, torch.dtype):
+                    args.append(TORCH_DTYPE_TO_NUM[i])
+                elif (
+                    isinstance(i, torch.device)
+                    or isinstance(i, torch.layout)
+                    or isinstance(i, torch.memory_format)
+                ):
+                    # PyMIL graph does not care about these things
+                    pass
                 elif i is None:
                     args.append(None)
                 else:
-                    raise AssertionError(f"Unhandled type of the node: {type(i)}")
+                    raise AssertionError(
+                        f"Unhandled node type {type(i)}. Node content is: {str(i)}"
+                    )
             return tuple(args)
 
-        inputs = get_arguments(node.args)
-        outputs = [
-            node.name
-        ]  # TODO: rdar://115846125 ([Executorch] Handle Models/Layers with Multiple outputs)
-
         try:
             kind = node.target.name()
         except:
@@ -250,13 +280,21 @@ class InternalTorchIRNode:
                 kind = node.target.__name__
             else:
                 kind = str(node.target)
+        kind = sanitize_op_kind(kind)
+        if not kind in _TORCH_OPS_REGISTRY:
+            raise ValueError(f"Unsupported fx node {str(node)}, kind {kind}")
 
-        namespace = kind.split("::")[0].lower()
-        if namespace in _DEFAULT_OP_NAMESPACES:
-            # We conventionally skip the aten/prim namespaces in our naming.
-            kind = kind.split("::")[-1].lower()
-        else:
-            kind = kind.lower()
+        # TODO (rdar://134015126) handle kwargs
+        inputs = get_arguments(node.args)
+        # TODO: rdar://115846125 ([Executorch] Handle Models/Layers with Multiple outputs)
+        outputs = [node.name]
+
+        kwinputs = {}
+        for keyword, arg in node.kwargs.items():
+            if arg is not None:
+                kwinputs[keyword] = get_arguments([arg])
+        if len(kwinputs) == 0:
+            kwinputs = None
 
         name = node.name
         return cls(
@@ -264,9 +302,11 @@ class InternalTorchIRNode:
             kind=kind,
             inputs=inputs,
             outputs=outputs,
+            kwinputs=kwinputs,
             parent=None,
             attr=None,
             blocks=None,
+            meta=node.meta,
         )
 
     def __str__(self, indent=2):
@@ -297,18 +337,95 @@ class InternalTorchIRNode:
         for block in self.blocks:
             block.replace_name(old_name, new_name)
 
+    def get_scope_info(self) -> Tuple[List[str], List[str]]:
+        """
+        Get the scope information (``scope_name``, ``scope_type``) of a TorchScript node.
+        In a TorchScript node, a model hierarchy is represented in a string of format:
+            ``scope_name_1(scope_type_1).scope_name_2(scope_type_1).<...>.scope_name_n(scope_type_n)``
+        For instance, given a torch model:
+
+            class SubModule(torch.nn.Module):
+                def __init__(self):
+                    super().__init__()
+                    self.linear_1 = torch.nn.Linear(2, 3)
+
+                def forward(self, x):
+                    x_1 = self.linear(x)
+                    x_2 = torch.relu(x_1)
+                    return x_2
+
+            class Model(torch.nn.Module):
+                def __init__(self):
+                    super().__init__()
+                    self.submodule_1 = SubModule()
+
+                def forward(self, x):
+                    return self.submodule_1(x)
+
+        The model hierarchy of ``x_1`` is ``submodule_1(SubModule).linear_1(Linear)``,
+        and ``x_2`` has ``submodule_1(SubModule)``.
+
+        We consider the ``node.name`` as the most inner ``scope_name``, and
+        ``node.kind`` (aten op type) as the most inner ``scope_type``.
+
+        ``x_1`` results in:
+            {
+                "scope_name": ["submodule_1", "linear_1", "x_1"],
+                "scope_type": ["SubModule", "Linear", "linear"],
+            },
+        and ``x_2`` gets:
+            {
+                "scope_name": ["submodule_1", "x_2"],
+                "scope_type": ["SubModule", "relu"],
+            }.
+
+        Note that, for the model weight const ops, the names are in the following format:
+        "submodule_1.linear_1.weight", which would result in a long ``scope_name``:
+        ``["submodule_1", "linear_1", "submodule_1.linear_1.weight"]``.
+        This function does a special handling to trim it to:
+        ``["submodule_1", "linear_1", "weight"]``
+        """
+
+        def _trim_scopename_for_weight(scope_names: List[str]) -> List[str]:
+            weight_name = scope_names[-1]
+            if scope_names[:-1] != weight_name.split(".")[:-1]:
+                return scope_names
+            scope_names[-1] = weight_name.split(".")[-1]
+            return scope_names
+
+        if self.model_hierarchy == "" or self.model_hierarchy is None:
+            scopes = []
+        else:
+            scopes = self.model_hierarchy.split(".")
+        scope_names, scope_types = [], []
+        for val in scopes:
+            if val == "":
+                scope_names.append("UNKNOWN_SCOPE_NAME")
+                scope_types.append("UNKNOWN_SCOPE_TYPE")
+                continue
+            if val.count("(") != 1 or val.count(")") != 1:
+                raise ValueError(f"{val} is not a valid model hierarchy string.")
+            lower_idx, upper_idx = val.index("("), val.index(")")
+            scope_names.append(val[:lower_idx])
+            scope_types.append(val[lower_idx + 1 : upper_idx])
+        scope_names.append(self.name)
+        scope_types.append(self.kind)
+        if self.kind == "getattr":
+            scope_names = _trim_scopename_for_weight(scope_names)
+        return scope_names, scope_types
+
 
 class InternalTorchIRGraph:
     """
-    CoreML internal representation of a torch IR graph. A torch._C.Graph
+    Core ML internal representation of a torch IR graph. A torch._C.Graph
     object is not an ideal structure to use in converting to CoreML. Conversion
     to an InternalTorchIRGraph is inserted between the original graph and the
-    final CoreML model to address several issues:
+    final Core ML model to address several issues:
         1. A torch._C.graph is hard to work with. For example, its .inputs()
           and .outputs() functions return iterators, so the only way to
           determine the number of inputs/outputs is by counting to the end.
           There are other examples of why the torch structure is hard to work
-          with, and this structure alleviates those isses.
+          with, and this structure alleviates those issues.
         2. torch._C.graph is an internal API and so we can't count on its
           stability. By inserting a layer in between, we can handle any changes
           to torch._C.graph here and isolate the ops code that processes the
@@ -322,33 +439,46 @@ class InternalTorchIRGraph:
 
     def __init__(
         self,
-        params,
-        inputs,
-        outputs,
-        nodes=None,
+        params: Dict[str, np.ndarray],
+        inputs: Dict[str, TensorType],
+        outputs: List[str],
+        nodes: Optional[List["InternalTorchIRNode"]] = None,
+        buffers: Optional[Dict[str, torch.Tensor]] = None,
+        input_name_to_source_buffer_name: Optional[Dict[str, str]] = None,
+        output_name_to_target_buffer_name: Optional[Dict[str, str]] = None,
     ):
         """
         Arguments:
             params: dict mapping parameter names to their numpy value.
-            inputs: OrderedDict mapping input names to their example values.
+            inputs: OrderedDict mapping input names to their input types.
             outputs: list[str], list of outputs from the graph.
-            nodes: list of InternalTorchIRNodes in the graph.
+            nodes: list of InternalTorchIRNode in the graph.
+            buffers: Dict mapping torch model buffers to their names.
+            input_name_to_source_buffer_name: Dict[str, str] (EXIR only)
+                dictionary mapping input variable names to underlying mutable buffer names,
+                i.e. these input variables are "read" from mutable buffer
+            output_name_to_target_buffer_name: Dict[str, str] (EXIR only)
+                dictionary mapping output variable names to underlying mutable buffer names,
+                i.e. these output variables are "written" to mutable buffer
         """
         self.nodes = nodes
         self.params = params
         self.inputs = inputs
         self.outputs = outputs
+        self.buffers = buffers
+        self.input_name_to_source_buffer_name = input_name_to_source_buffer_name
+        self.output_name_to_target_buffer_name = output_name_to_target_buffer_name
+
+        self.params_scope = {}
 
     @classmethod
-    def from_torchscript(cls, torchscript, input_values=None, cut_at_symbols=None):
+    def from_torchscript(cls, torchscript, inputs=None, cut_at_symbols=None):
         """
         Arguments:
             torchscript: TorchScript object representing the model to convert.
-            input_values: A list of inputs to the graph. Must be given is
-                @raw_graph if not None.
+            inputs: A list of input types to the graph.
             cut_at_symbols: The list of desired outputs from the graph. Symbols
-                must be present in the graph. For debugging use only. Can only
-                be given if @raw_graph is not None.
+                must be present in the graph. For debugging use only.
         """
         if not isinstance(torchscript, torch.jit.ScriptModule):
             raise AssertionError(
@@ -367,34 +497,21 @@ class InternalTorchIRGraph:
             )
 
         nodes = []
-        params = {}
-        inputs = OrderedDict()
+        inputs_name_to_type = OrderedDict()
         outputs = []
 
-        raw_graph, params_dict, buffer_dict = _expand_and_optimize_ir(torchscript)
-
-        # Add params
-        for name, param in params_dict.items():
-            if isinstance(param, torch.Tensor):
-                if param.is_quantized:
-                    value = param
-                else:
-                    value = param.detach().cpu().numpy()
-            else:
-                value = param
-            params[name] = value
+        raw_graph, params, buffers = _expand_and_optimize_ir(torchscript)
 
         # Add inputs
         # The first element of the raw_graph.inputs() is the 'self' of the module, which is not used.
         graph_inputs = list(raw_graph.inputs())[1:]
-        if len(graph_inputs) != len(input_values):
-                raise ValueError(
-                    f"Number of TorchScript inputs ({len(graph_inputs)}) must match the user provided inputs ({len(input_values)})."
-                )
+        if len(graph_inputs) != len(inputs):
+            raise ValueError(
+                f"Number of TorchScript inputs ({len(graph_inputs)}) must match the user provided inputs ({len(inputs)})."
+            )
         for index, _input in enumerate(graph_inputs):
             name = _input.debugName()
-            value = input_values[index]
-            inputs[name] = value
+            inputs_name_to_type[name] = inputs[index]
 
         # Add outputs, cutting if @cut_at_symbols is set
         output_names = cut_at_symbols
@@ -403,10 +520,16 @@ class InternalTorchIRGraph:
         for output in output_names:
             outputs.append(output)
 
-        internal_graph = cls(nodes=nodes, params=params, inputs=inputs, outputs=outputs)
+        internal_graph = cls(
+            nodes=nodes,
+            params=params,
+            inputs=inputs_name_to_type,
+            outputs=outputs,
+            buffers=buffers,
+        )
 
-        node_names = set()
         # Add nodes
+        node_names = set()
         for raw_node in raw_graph.nodes():
             new_node = InternalTorchIRNode.from_torchscript_node(
                 node=raw_node, parent=internal_graph
@@ -416,56 +539,64 @@ class InternalTorchIRGraph:
             internal_graph.nodes.append(new_node)
             node_names.add(new_node.name)
 
-        return internal_graph, params_dict, buffer_dict
+        internal_graph._cache_model_hierarchy_for_params()
+
+        return internal_graph
+
+    def _cache_model_hierarchy_for_params(self) -> None:
+        # We cache the model hierarchy information for model weights in self.params_scope,
+        # since self.params doesn't contain the information.
+        def cache_model_hierarchy_block(block):
+            for node in block.nodes:
+                for b in node.blocks:
+                    cache_model_hierarchy_block(b)
+                if node.name in self.params:
+                    self.params_scope[node.name] = node.get_scope_info()
+        cache_model_hierarchy_block(self)
 
     @classmethod
-    def from_edgeir(cls, edgeir):
-        exported_program = edgeir
+    def from_exir(cls, exir):
+        exported_program: torch.export.ExportedProgram = exir
+        (
+            user_inputs,
+            user_outputs,
+            params,
+            buffers,
+            input_name_to_source_buffer_name,
+            output_name_to_target_buffer_name,
+        ) = extract_io_from_exir_program(exported_program)
+
+        inputs = OrderedDict([(i.name, i) for i in user_inputs])
 
         nodes = []
-        params = {}
-        outputs = []
-        inputs = OrderedDict(
-            [
-                (i.name, i)
-                for i in extract_inputs_from_edge_program(exported_program=exported_program)
-            ]
-        )
-
-        inputs_to_parameters = exported_program.graph_signature.inputs_to_parameters
-        inputs_to_buffers = exported_program.graph_signature.inputs_to_buffers
-
-        inputs_to_consts = {**inputs_to_parameters, **inputs_to_buffers}
-
-        parameters_to_inputs = {
-            v: k if not k.startswith("%") else k[1:] for k, v in inputs_to_consts.items()
-        }
-
-        # Add params
-        for name, param in exported_program.state_dict.items():
-            if isinstance(param, torch.Tensor):
-                value = param.detach().cpu().numpy()
-            else:
-                raise NotImplementedError("Only torch.Tensor handled yet")
-
-            params[name if name not in parameters_to_inputs else parameters_to_inputs[name]] = value
-
-        graph = exported_program.graph
-
-        outputs = []
-        for node in graph.nodes:
+        for node in exported_program.graph_module.graph.nodes:
             if node.op == "call_function":
-                nodes.append(InternalTorchIRNode.from_edgeir_node(node=node))
+                nodes.append(InternalTorchIRNode.from_exir_node(node=node))
+            elif node.op == "get_attr":
+                name = node.target
+                attr = exported_program.graph_module.__getattr__(name)
+                # Only handle simple tensor attribute for now
+                # There may be unconvertible advanced attributes,
+                # e.g. higher-level callables such as "call_delegate"
+                if not isinstance(attr, torch.Tensor):
+                    raise NotImplementedError("Only torch.Tensor attr handled yet")
+                params[name] = attr
             elif node.op == "placeholder":
                 continue
             elif node.op == "output":
-                outputs = [
-                    node.name for node in node.args[0]
-                ]  # TODO: rdar://115846125 ([Executorch] Handle Models/Layers with Multiple outputs)
+                continue
             else:
                 raise NotImplementedError(f"Nodes of type {node.op} not yet implemented")
 
-        return cls(nodes=nodes, params=params, inputs=inputs, outputs=outputs)
+        return cls(
+            nodes=nodes,
+            params=params,
+            inputs=inputs,
+            outputs=user_outputs,
+            buffers=buffers,
+            input_name_to_source_buffer_name=input_name_to_source_buffer_name,
+            output_name_to_target_buffer_name=output_name_to_target_buffer_name,
+        )
 
     def __str__(self):
         graph_str = "graph(\n"
