@@ -9,16 +9,22 @@ from typing import List, Optional, Union
 import torch as _torch
 from torch.jit._script import RecursiveScriptModule
 
-from coremltools._deps import _HAS_TORCH_EXPORT_API
+from coremltools import _logger as logger
+from coremltools._deps import _HAS_EXECUTORCH, _HAS_TORCH_EXPORT_API
 from coremltools.converters.mil.frontend.torch.converter import TorchConverter
-from coremltools.converters.mil.input_types import TensorType
+from coremltools.converters.mil.input_types import StateType, TensorType
 from coremltools.converters.mil.mil.program import Program
 
 from .converter import TorchConverter
+from .utils import TorchFrontend
 
 if _HAS_TORCH_EXPORT_API:
     from torch.export import ExportedProgram
 
+if _HAS_EXECUTORCH:
+    import executorch.exir
+
+
 def load(
     spec: Union[RecursiveScriptModule, "ExportedProgram", str],
     inputs: List[TensorType],
@@ -27,6 +33,7 @@ def load(
     outputs: Optional[List[TensorType]] = None,
     cut_at_symbols: Optional[List[str]] = None,
     use_default_fp16_io: bool = False,
+    states: Optional[List[StateType]] = None,
     **kwargs
 ) -> Program:
     """
@@ -37,7 +44,7 @@ def load(
     spec: It could be one of the following:
         - String path to .pt file containing serialized torchscript model
         - In memory TorchScript model of type torch.jit.ScriptModule
-        - In memory EdgeIR program of type ExportedProgram
+        - In memory EXIR program of type ExportedProgram
     inputs: Can be a singular element or list of elements of the following form
         1. Any subclass of InputType
         2. torch.Tensor (only shape and dtype will be used)
@@ -64,6 +71,9 @@ def load(
     """
 
     if _HAS_TORCH_EXPORT_API and isinstance(spec, ExportedProgram):
+        # TODO: rdar://115845792 ([Executorch] Handle user provided inputs/outputs in the convert API)
+        if states:
+            raise AssertionError("'states' argument should be None for ExportedProgram")
         model = spec
     else:
         model = _torchscript_from_spec(spec)
@@ -75,25 +85,70 @@ def load(
         cut_at_symbols,
         specification_version,
         use_default_fp16_io,
+        states,
     )
 
     return _perform_torch_convert(converter, debug)
 
 
-def _torchscript_from_spec(model_spec: RecursiveScriptModule) -> RecursiveScriptModule:
+def is_torch_model(model_spec: Union[str, RecursiveScriptModule]) -> bool:
+    if isinstance(model_spec, str) and (model_spec.endswith(".pt") or model_spec.endswith(".pth")):
+        # PyTorch file path
+        return True
+    elif isinstance(model_spec, _torch.jit.ScriptModule):
+        # PyTorch object
+        return True
+    return False
+
+
+def _torchscript_from_spec(model_spec: Union[str, RecursiveScriptModule]) -> RecursiveScriptModule:
     if isinstance(model_spec, str) and (model_spec.endswith(".pt") or model_spec.endswith(".pth")):
         filename = _os_path.abspath(model_spec)
-        return _torch.jit.load(filename)
+        try:
+            return _torch.jit.load(filename)
+        except Exception as e:
+            logger.error("\n\nERROR - Could not load the PyTorch model. Got the following error:\n")
+            raise e
+
     elif isinstance(model_spec, _torch.jit.ScriptModule):
         return model_spec
+
     else:
         raise TypeError(
-            "@model must either be a PyTorch .pt or .pth file or a TorchScript object, received: {}".format(
-                type(model_spec)
-            )
+            "A PyTorch model must either be a .pt or .pth file, or a TorchScript object. "
+            f"Received: {type(model_spec)}"
         )
 
 
+if _HAS_TORCH_EXPORT_API:
+
+    def _torchexport_from_spec(
+        model_spec: Union[str, ExportedProgram],
+        frontend=TorchFrontend.TORCHEXPORT,
+    ) -> ExportedProgram:
+        # Load torch.export serialization
+        if isinstance(model_spec, str) and model_spec.endswith(".pt2"):
+            filename = _os_path.abspath(model_spec)
+            try:
+                model = _torch.export.load(filename)
+            except Exception as e:
+                logger.error(
+                    "\n\nERROR - Could not load the PyTorch model. Got the following error:\n"
+                )
+                raise e
+        elif isinstance(model_spec, ExportedProgram):
+            model = model_spec
+        else:
+            raise TypeError(
+                "A PyTorch model must either be a .pt2 file, or an ExportedProgram object. "
+                f"Received: {type(model_spec)}"
+            )
+        # To edge if edge dialect is desired
+        if frontend == TorchFrontend.EXECUTORCH and model.dialect != "EDGE":
+            model = executorch.exir.to_edge(model).exported_program()
+        return model
+
+
 def _perform_torch_convert(converter: TorchConverter, debug: bool) -> Program:
     try:
         prog = converter.convert()
