@@ -14,7 +14,7 @@ class torch_tensor_assign_to_core(AbstractGraphPass):
     """
     Map Torch dialect ops `torch_tensor_assign` into core opset.
 
-    Currently, we tranform the torch_tensor_assign op using mb.scatter.
+    Currently, we transform the torch_tensor_assign op using mb.scatter.
     """
     def apply(self, prog):
         for f in prog.functions.values():
@@ -22,7 +22,7 @@ class torch_tensor_assign_to_core(AbstractGraphPass):
 
 @block_context_manager
 def _torch_tensor_assign_to_core_block(block):
-    for op in block.operations[:]:
+    for op in list(block.operations):
         for b in op.blocks:
             _torch_tensor_assign_to_core_block(b)
 
