@@ -5,17 +5,30 @@
 
 import numbers
 from collections import defaultdict
-from typing import Callable, List, Optional
+from typing import Any, Callable, List, Optional, Tuple, Type
 
 import numpy as np
 
 from coremltools import _logger as logger
+from coremltools.converters.mil import mil
 from coremltools.converters.mil._deployment_compatibility import AvailableTarget
 from coremltools.converters.mil.mil.types.symbolic import any_symbolic
 
 from .block import Function, curr_block
-from .input_type import InternalInputType, ListOrTensorInputType, TensorInputType, TupleInputType
-from .program import Placeholder, Program
+from .input_type import (
+    InternalInputType,
+    ListOrTensorOrDictInputType,
+    TensorInputType,
+    TupleInputType,
+)
+from .program import Placeholder, StateTensorPlaceholder
+from .scope import (
+    SCOPE_STACK,
+    VALID_OPS_TO_COPY_SCOPE_INFO,
+    ScopeContextManger,
+    ScopeInfo,
+    ScopeSource,
+)
 from .var import InternalVar, Var
 
 
@@ -74,15 +87,15 @@ class Builder:
     @classmethod
     def _add_const(cls, val, name, before_op):
         if not is_python_value(val):
-            raise ValueError("Cannot add const {}".format(val))
-        if any_symbolic(val):
-            msg = (
-                "Python native vals (list, tuple), np.array that are"
-                + "operation inputs cannot have symbolic values. Consider feeding"
-                + "symbolic shape in through placeholder and use mb.shape() "
-                + "operator. Input {}: {}"
-            )
-            raise ValueError(msg.format(name, val))
+            err_msg = f"Cannot add const {val}"
+            if any_symbolic(val):
+                err_msg += (
+                    "\nPython native vals (list, tuple), np.array that are"
+                    + "operation inputs cannot have symbolic values. Consider feeding"
+                    + "symbolic shape in through placeholder and use mb.shape() "
+                    + f"operator. Input {name}: {val}"
+                )
+            raise ValueError(err_msg)
         const_name = cls._get_free_name(name)
         logger.debug("Adding const op '{}'".format(const_name))
         output_var = cls.const(val=val, name=const_name,
@@ -95,7 +108,7 @@ class Builder:
         candidate_kv):
         """
         For each key K in `candidate_kv`, create a Var if the
-        followings are satisfied:
+        following are satisfied:
 
         - K exists in input_spec and is not an InternalInputType
         - candidate_kv[K] is not already a Var
@@ -131,6 +144,8 @@ class Builder:
             new_var_name = op_name + "_" + k
             if isinstance(in_type, TupleInputType):
                 var = []
+                if not isinstance(val, (list, tuple)):
+                    raise ValueError(f"Invalid type {type(val)} for TupleInputType param.")
                 for i, v in enumerate(val):
                     if isinstance(v, Var):
                         var.append(v)
@@ -142,7 +157,7 @@ class Builder:
                 update_dict[k] = var
                 continue
 
-            if isinstance(in_type, (TensorInputType, ListOrTensorInputType)):
+            if isinstance(in_type, (TensorInputType, ListOrTensorOrDictInputType)):
                 var = cls._add_const(val, new_var_name, before_op)
                 update_dict[k] = var
 
@@ -154,7 +169,7 @@ class Builder:
         Add an op of type `op_cls` (e.g., convolution) to current block.
         """
         kwargs = cls._maybe_set_name(kwargs, op_cls.__name__)
-        logger.info(
+        logger.debug(
             "Adding op '{}' of type {}".format(kwargs["name"], op_cls.__name__)
         )
         before_op = kwargs.get("before_op", None)
@@ -165,8 +180,16 @@ class Builder:
             op_name=kwargs["name"], before_op=before_op,
             candidate_kv=kwargs))
         kwargs["enclosing_block"] = curr_block()
+
+        # Add scope information
+        current_scopes = SCOPE_STACK.get_curr_scopes()
+        kwargs["scopes"] = current_scopes
         new_op = op_cls(**kwargs)
 
+        # We record if the op is created under graph pass
+        if len(current_scopes) == 1 and ScopeSource.COREMLTOOLS_GRAPH_PASS in current_scopes:
+            VALID_OPS_TO_COPY_SCOPE_INFO[-1].add(new_op)
+
         # Initialize optional input Vars if it wasn't in kwargs
         default_inputs = new_op.default_inputs()
         # Shallow copy list inputs to ensure op inputs are immutable
@@ -187,13 +210,26 @@ class Builder:
         return new_op.outputs
 
     @staticmethod
-    def placeholder(shape, dtype=None, allow_rank0_input=False):
-        return Placeholder(shape, dtype, allow_rank0_input=allow_rank0_input)
+    def placeholder(
+        shape: Tuple[Any],
+        dtype: Optional[Type] = None,
+        allow_rank0_input: Optional[bool] = False,
+        name: Optional[str] = None,
+    ) -> Placeholder:
+        return Placeholder(shape, dtype, allow_rank0_input=allow_rank0_input, name=name)
 
     @staticmethod
     def TensorSpec(shape, dtype=None):
         return Placeholder(shape, dtype)
 
+    @staticmethod
+    def StateTensorSpec(shape, dtype=None):
+        return StateTensorPlaceholder(shape, dtype)
+
+    @staticmethod
+    def state_tensor_placeholder(shape, dtype=None):
+        return StateTensorPlaceholder(shape, dtype)
+
     @staticmethod
     def _create_function(
         main_block: Callable,
@@ -285,6 +321,8 @@ class Builder:
         Examples
         --------
         >>> import coremltools as ct
+        >>> from coremltools.converters.mil.mil import Builder as mb
+        >>>
         >>> @mb.program(input_specs=[mb.TensorSpec(shape=(1,2))], opset_version=ct.target.iOS16)
         >>> def prog(a):
         >>>     return mb.add(x=a, y=2)
@@ -292,7 +330,65 @@ class Builder:
         """
         def wrapper(main_block):
             function = Builder._create_function(main_block, input_specs, opset_version)
-            program = Program()
+            program = mil.Program()
             program.add_function(function_name, function)
             return program
         return wrapper
+
+    @staticmethod
+    def scope(
+        *scopes: List[ScopeInfo],
+    ) -> ScopeContextManger:
+        """
+        The ``mb.scope`` creates a context manager, which makes the operations created within it have the corresponding scope information.
+
+        Parameters
+        ----------
+        scopes: Optional[List[ScopeInfo]] (Optional)
+            * A list of ScopeInfo under the context manager.
+            * The source in each ScopeInfo cannot be duplicated.
+            * If not provided, this context manager does no affects.
+
+        Examples
+        --------
+        The following is an example of creating a scope for torchscript module heirarchy with type and name information.
+
+        .. sourcecode:: python
+
+            @mb.program(input_specs=[mb.TensorSpec(shape=(2, 3))])
+            def prog(x):
+                with mb.scope(
+                    ScopeInfo(source=ScopeSource.TORCHSCRIPT_MODULE_TYPE, data=["Module1"]),
+                    ScopeInfo(source=ScopeSource.TORCHSCRIPT_MODULE_NAME, data=["module_1"]),
+                ):
+                    return mb.add(x=x, y=4.3, name="add_1")
+
+
+        In the previous example, the "add_1" op will have two scope attributes, for torchscipt module type and name:
+            * TORCHSCRIPT_MODULE_TYPE: ["Module1"]
+            * TORCHSCRIPT_MODULE_NAME: ["module_1"]
+
+        The following is an example of creating nested scopes:
+
+        .. sourcecode:: python
+
+            @mb.program(input_specs=[mb.TensorSpec(shape=(2, 3))])
+            def prog(x):
+                with mb.scope(
+                    ScopeInfo(source=ScopeSource.TORCHSCRIPT_MODULE_TYPE, data=["Module1"]),
+                ):
+                    x = mb.add(x=x, y=4.3, name="add_1")
+                    with mb.scope(
+                        ScopeInfo(source=ScopeSource.TORCHSCRIPT_MODULE_TYPE, data=["Module2"]),
+                        ScopeInfo(source=ScopeSource.TORCHSCRIPT_MODULE_NAME, data=["module_2"]),
+                    ):
+                        return mb.add(x=x, y=3.2, name="add_2")
+
+        In the previous example, the "add_1" op would have a scope attribute:
+            * TORCHSCRIPT_MODULE_TYPE: ["Module1"]
+
+        while the "add_2" op would have scope attributes:
+            * TORCHSCRIPT_MODULE_TYPE: ["Module1", "Module2"]
+            * TORCHSCRIPT_MODULE_NAME: ["module_2"]
+        """
+        return ScopeContextManger(*scopes)
