@@ -3,12 +3,11 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from typing import Any, Dict, Tuple
+from typing import Any, Dict, Optional, Tuple
 
 import numpy as np
 
 from coremltools.converters.mil.mil import types
-from coremltools.converters.mil.mil.types import is_compatible_type
 from coremltools.converters.mil.mil.types.symbolic import any_symbolic, is_symbolic
 
 from . import SPACES
@@ -167,6 +166,7 @@ class Operation:
         self._input_vars = {}
         self.blocks = []
         self.enclosing_block = kwargs["enclosing_block"]
+        self.scopes = kwargs["scopes"]
 
         # Initialize inputs as object attributes (all None)
         for k in self._input_types.keys():
@@ -205,10 +205,10 @@ class Operation:
             "value",
             "version",
             "before_op",
-            "no_check_var_visibility",  # no_check_var_visibility==True to deviate from SSA
             "no_check_var_types",
             # no_check_var_types==True to force set inputs, even if type does not match with earlier ones
             "enclosing_block",
+            "scopes",
         ]
         for k in kwargs.keys():
             if k not in non_attributes and k not in self._input_types:
@@ -325,7 +325,7 @@ class Operation:
                 # Check type inference
                 if overwrite_output:
                     out_var._sym_type = sym_type
-                elif not is_compatible_type(sym_type, out_var.sym_type):
+                elif not types.is_compatible_type(sym_type, out_var.sym_type):
                     msg = "Output Var {} in op {} type changes with new input Vars"
                     raise ValueError(msg.format(out_var.name, self.name))
 
@@ -347,7 +347,7 @@ class Operation:
 
     def _auto_val(self, output_types):
         """
-        # Evaluation is two stage:
+        # Evaluation has two stages:
         #
         # Stage 1: Check whether the method value_inference() is implemented
         #
@@ -465,7 +465,7 @@ class Operation:
 
     def _validate_and_set_inputs(self, input_kvs, no_check_var_types=False):
         """
-        For each k, v in `input_kvs`, perform the followings:
+        For each k, v in `input_kvs`, perform the following:
 
         - Check k exists in `self.input_specs`
         - Check that v satisfies the correspodning `InputType`
@@ -493,10 +493,13 @@ class Operation:
         def check_and_detach(v_new, v_old, op, no_check_var_types):
             # Check new var's sym_type is compatible with the
             # existing's sym_type.
-            if not is_compatible_type(v_new.sym_type, v_old.sym_type) and not no_check_var_types:
+            if (
+                not types.is_compatible_type(v_new.sym_type, v_old.sym_type)
+                and not no_check_var_types
+            ):
                 raise ValueError(
-                    f"New var type `{v_new.sym_type}` not a "
-                    f"subtype of existing var type `{v_old.sym_type}`."
+                    f"New var {v_new} doesn't have compatible "
+                    f"subtype of existing var `{v_old}`."
                 )
             v_old.remove_child_op(op, no_check_var_types)
 
@@ -541,6 +544,13 @@ class Operation:
             if not isinstance(v, InternalVar) and v is not None
         }
 
+    @property
+    def internal_inputs(self) -> Dict[str, InternalVar]:
+        """
+        Get internal var inputs of an op.
+        """
+        return {k: v for k, v in self._input_vars.items() if isinstance(v, InternalVar)}
+
     @property
     def outputs(self):
         return self._output_vars
@@ -584,23 +594,26 @@ class Operation:
 
         return "%" + v.name
 
-    def indented_str(self, indent=""):
+    def indented_str(self, indent: Optional[str] = "", print_attr: Optional[bool] = False) -> str:
         if self.op_type == "const":
             return ""
         s = indent
         if self.outputs is not None:
             s += ", ".join([str(o) for o in self.outputs])
-        s += " = " + self.op_type + "("
-        s += ", ".join(
-            [
-                k + "=" + Operation.var_to_str(self.inputs[k])
-                for k in self._input_types.keys()
-                if k in self.inputs and not is_internal_input(k)
-            ]
-        )
+
+        if print_attr:
+            attr = "["
+            for k, v in self.scopes.items():
+                attr += f"{k}: {v}, "
+            attr = attr[:-2] + "]"
+        else:
+            attr = ""
+
+        s += " = " + self.op_type + attr + "("
+        s += ", ".join([k + "=" + Operation.var_to_str(v) for k, v in self.inputs.items()])
         s += ', name="{}")\n'.format(self.name)
         for b in self.blocks:
-            s += b.indented_str(indent=indent + SPACES)
+            s += b.indented_str(indent=indent + SPACES, print_attr=print_attr)
         return s
 
     def __repr__(self):
