@@ -224,10 +224,10 @@ class matmul(Operation):
     def value_inference(self):
         x = self.x.val
         if self.transpose_x.val:
-            x = np.transpose(x)
+            x = np.swapaxes(x, -1, -2)
         y = self.y.val
         if self.transpose_y.val:
-            y = np.transpose(y)
+            y = np.swapaxes(y, -1, -2)
         return np.matmul(x, y)
 
 
@@ -235,7 +235,7 @@ class matmul(Operation):
 class einsum(Operation):
     """
     Perform tensor multiplication expressed according to the einsum notation.
-    The mode/equation that is currently supported is mutiplying matrices that are laid out on
+    The mode/equation that is currently supported is multiplying matrices that are laid out on
     dimensions -1 and -3, treating all the other dimensions as batch. Broadcasting is supported along batch dimensions.
     In particular, the inputs must be of the following shapes:
 
@@ -291,7 +291,8 @@ class einsum(Operation):
         x_shape = x.shape
         y_shape = y.shape
         assert len(x_shape) == len(y_shape), "inputs not of the same rank"
-        assert x_shape[-1] == y_shape[-3], "input shapes incompatible"
+        if not (is_symbolic(x_shape[-1]) or is_symbolic(y_shape[-3])):
+            assert x_shape[-1] == y_shape[-3], f"input shapes incompatible: {x_shape[-1]} and {y_shape[-3]}"
         if x_shape[-2] != 1 and y_shape[-2] != 1:
             assert x_shape[-2] == y_shape[-2], "input shapes incompatible"
         if len(x_shape) == 4:
