@@ -45,13 +45,13 @@ class random_bernoulli(RandomDistribution):
     r"""
     Returns a tensor with the specified shape, with random values from a Bernoulli
     distribution.
-    
+
     .. math::
        f(k) = \begin{cases}1-p  &\text{if } k = 0\\
                         p    &\text{if } k = 1\end{cases}
 
     for :math:`k` in :math:`\{0, 1\}`.
-    
+
     Parameters
     ----------
     shape: <K, i32> (Required)
@@ -62,7 +62,7 @@ class random_bernoulli(RandomDistribution):
         * The probability of sampling ``1``. Defaults to ``0.5``.
     seed: const<i32> (Optional)
         * Seed to create a reproducible sequence of values across multiple invokes.
-    
+
     Returns
     -------
     <\*, T>
@@ -76,7 +76,7 @@ class random_bernoulli(RandomDistribution):
     --------
     random_categorical, random_normal, random_uniform
     """
-    
+
     input_spec = (
         InputSpec(
             shape=TensorInputType(type_domain=types.int32),
@@ -85,7 +85,7 @@ class random_bernoulli(RandomDistribution):
         )
         + RandomDistribution.input_spec
     )
-    
+
     type_domains = {
         "T": (types.fp16, types.fp32),
     }
@@ -106,23 +106,29 @@ class random_bernoulli(RandomDistribution):
 class random_categorical(Operation):
     """
     Returns random values from a categorical distribution.
-    
+
     Parameters
     ----------
-    shape: <\*D_in, T>
-        * N-dimensional tensor, one of ``logits`` (event log-probabilities) or ``probs``
-          (event probabilities). The first ``N - 1`` dimensions specifies distributions,
-          and the last dimension represents a vector of probabilities.
+    x: <\*D_in, T>
+        * N-dimensional tensor which represents ``logits`` (event log-probabilities) or ``probs``
+          (event probabilities) depending on ``mode``. The first ``N - 1`` dimensions specifies
+          distributions, and the last dimension represents a vector of probabilities.
 
     mode: const<str> (Optional)
         One of ``['logits', 'probs']``. Defaults to ``logits``.
+        When set to ``probs``, an element-wise log layer will be added to calculate logits.
 
     size: const<i32> (Optional)
         Number of samples to draw. Defaults to ``1``.
+        When set as ``1``, it's categorical distribution.
+        When set larger than ``1``, it's actually multinomial distribution by drawing with
+        replacement. It means that when a sample index is drawn, it can be drawn again.
+        The categorical distribution is a special case of the multinomial distribution, giving
+        the probabilities of potential outcomes of a single drawing rather than multiple drawings.
 
     seed: const<i32> (Optional)
         Seed to create a reproducible sequence of values across multiple invokes.
-    
+
     Returns
     -------
     <\*D_in[:-1] + [size], T>
@@ -136,14 +142,14 @@ class random_categorical(Operation):
     --------
     random_bernoulli, random_normal, random_uniform
     """
-    
+
     input_spec = InputSpec(
         x=TensorInputType(type_domain="T"),
         mode=TensorInputType(const=True, optional=True, type_domain=types.str),
         size=TensorInputType(const=True, optional=True, type_domain=types.int32),
         seed=TensorInputType(const=True, optional=True, type_domain=types.int32),
     )
-    
+
     type_domains = {
         "T": (types.fp16, types.fp32),
     }
@@ -166,7 +172,7 @@ class random_normal(RandomDistribution):
     r"""
     Returns a tensor with the specified shape, with random values from a normal
     distribution.
-    
+
     Parameters
     ----------
     shape: <K, i32> (Required)
@@ -179,7 +185,7 @@ class random_normal(RandomDistribution):
         The standard deviation (width) of the normal distribution. Defaults to ``1.0``.
     seed: const<i32> (Optional)
         Seed to create a reproducible sequence of values across multiple invokes.
-    
+
     Returns
     -------
     <\*, T>
@@ -193,7 +199,7 @@ class random_normal(RandomDistribution):
     --------
     random_categorical, random_bernoulli, random_uniform
     """
-    
+
     input_spec = (
         InputSpec(
             shape=TensorInputType(type_domain=types.int32),
@@ -203,7 +209,7 @@ class random_normal(RandomDistribution):
         )
         + RandomDistribution.input_spec
     )
-    
+
     type_domains = {
         "T": (types.fp16, types.fp32),
     }
@@ -229,15 +235,15 @@ class random_uniform(RandomDistribution):
     Returns a tensor with the specified shape with random values from a uniform
     distribution. Samples are uniformly distributed over the half-open interval
     ``[low, high)`` (includes low, but excludes high).
-    
+
     .. math::
        p(x) = \frac{1}{high - low}
-    
+
     For a real number :math:`x`.
-    
+
     When ``high == low``, values of ``low`` will be returned. If ``high < low``,
     the results are officially undefined and may eventually raise an error.
-    
+
     Parameters
     ----------
     shape: <K, i32> (Required)
@@ -250,7 +256,7 @@ class random_uniform(RandomDistribution):
         * Upper boundary of the output interval (exclusive). Defaults to ``1.0``.
     seed: const<i32> (Optional)
         * Seed to create a reproducible sequence of values across multiple invokes.
-    
+
     Returns
     -------
     <\*, T>
@@ -264,7 +270,7 @@ class random_uniform(RandomDistribution):
     --------
     random_categorical, random_bernoulli, random_normal
     """
-    
+
     input_spec = (
         InputSpec(
             shape=TensorInputType(type_domain=types.int32),
@@ -274,7 +280,7 @@ class random_uniform(RandomDistribution):
         )
         + RandomDistribution.input_spec
     )
-    
+
     type_domains = {
         "T": (types.fp16, types.fp32),
     }
