@@ -7,25 +7,15 @@ import math
 
 import numpy as np
 
-from coremltools.converters.mil.mil import (
-    get_new_symbol,
-    get_new_variadic_symbol,
-    types,
-)
+from coremltools.converters.mil.mil import get_new_symbol, get_new_variadic_symbol, types
 from coremltools.converters.mil.mil.input_type import (
     DefaultInputs,
     InputSpec,
-    ListOrTensorInputType,
+    ListOrTensorOrDictInputType,
     TensorInputType,
     TupleInputType,
 )
-from coremltools.converters.mil.mil.operation import (
-    NONE,
-    SYMBOL,
-    VALUE,
-    Operation,
-    precondition,
-)
+from coremltools.converters.mil.mil.operation import NONE, SYMBOL, VALUE, Operation, precondition
 from coremltools.converters.mil.mil.ops.defs._op_reqs import register_op
 from coremltools.converters.mil.mil.ops.defs._utils import MAX_SIZE_CONSTANT_FOLDING
 from coremltools.converters.mil.mil.types.symbolic import (
@@ -365,7 +355,7 @@ class one_hot(Operation):
     ----------
     indices: tensor<[D], i32> (Required)
         * Tensor, values indicate the locations for each one-hot vector to take the ``on_value``.
-    one_got_vector_size: i32 (Required)
+    one_hot_vector_size: i32 (Required)
         * Indicates the number of returning vectors.
     axis: const i32 (Optional)
         * Indicates which dimension to append the new axis.
@@ -444,7 +434,7 @@ class one_hot(Operation):
 @register_op
 class pad(Operation):
     """
-    Pad a tensor.
+    Pads a tensor.
 
     Parameters
     ----------
@@ -570,10 +560,10 @@ class range_1d(Operation):
 
     Parameters
     ----------
-    end: <T> (Required)
-        * The upper limit of the sequence, exclusive.
     start: <T> (Required)
         * The start point of the sequence.
+    end: <T> (Required)
+        * The upper limit of the sequence, exclusive.
     step: <T> (Required)
         * Number that increments ``start``.
 
@@ -692,8 +682,13 @@ class tile(Operation):
                 if rep <= 0:
                     raise ValueError("All entries of reps parameter must be greater than 0")
 
-            if is_symbolic(rep) or is_symbolic(x_shape[i]):
+            if is_symbolic(rep):
                 out_shape.append(get_new_symbol())
+            elif is_symbolic(x_shape[i]):
+                if rep == 1:
+                    out_shape.append(x_shape[i])
+                else:
+                    out_shape.append(get_new_symbol())
             else:
                 out_shape.append(rep * x_shape[i])
 
@@ -1117,6 +1112,10 @@ class split(Operation):
         * The tensors may be variadic, but the number of tensors must be determined
           at compile time (i.e. a tuple).
 
+    axis: const<i32> (Required)
+        * The dimension along which to concatenate. Must be in the
+          range ``[-rank(x), rank(x))``.
+
     num_splits: <i32> (Optional)
         If specified, divide ``x`` into ``num_splits`` tensors along ``axis``.
         Its behavior depends on ``split_sizes``:
@@ -1134,10 +1133,6 @@ class split(Operation):
         * Sizes to split to. The sum of ``split_sizes`` must equal to
           ``value.shape[axis]``.
 
-    axis: const<i32> (Required)
-        * The dimension along which to concatenate. Must be in the
-          range ``[-rank(x), rank(x))``.
-
     Returns
     -------
     Tuple[tensor<\*?, T>]
@@ -1339,9 +1334,7 @@ class identity(Operation):
     T: fp16, fp32, i32, bool
     """
 
-    input_spec = InputSpec(
-        x=ListOrTensorInputType()
-    )
+    input_spec = InputSpec(x=ListOrTensorOrDictInputType())
 
     def type_inference(self):
         return self.x.sym_type
