@@ -167,7 +167,7 @@ class reshape(Operation):
     ----------
     x: tensor<\*?, T> (Required)
 
-        * A n-D tensor or a scalar.
+        * An n-D tensor or a scalar.
         * If ``x`` is fixed rank (and possibly contains symbolic dimension),
           shape may contain elements that are not positive integers (see below).
         * If ``x`` is variadic rank, shape can only contain positive integers.
@@ -181,7 +181,7 @@ class reshape(Operation):
               The new symbol that is not present in ``x.shape`` represent a dimension
               such that the total size remains constant. Symbol is illegal
               if ``x`` is variadic rank.
-            * ``-1``: ``-1`` introduces a new symbol (see Symbols). Therefore, ``-1`` is
+            * ``-1``: ``-1`` introduces a new symbol (see Symbols above). Therefore, ``-1`` is
               allowed if all symbols in the shape appear in ``x.shape``. ``-1`` is illegal
               if ``x`` is variadic rank.
             * ``0``: If ``K == rank(x)`` then ``0`` means inheriting from the corresponding
@@ -585,9 +585,9 @@ class slice_by_size(Operation):
     ----------
     x: tensor<*?, T> (Required)
         * Input tensor.
-    begin: tensor<[rank(x)], i32> Required
+    begin: tensor<[rank(x)], i32> (Required)
         * The begin index for slice.
-    size: tensor<[rank(x)], i32> Required
+    size: tensor<[rank(x)], i32> (Required)
         * The size that is to be sliced. If ``size`` is ``-1``,
           all the remaining elements starting with "begin" are sliced.
 
@@ -719,28 +719,28 @@ class space_to_depth(Operation):
 @register_op
 class space_to_batch(Operation):
     """
-    Rearrange elements in a tensor from spatial into batch dimension.
+    Rearrange elements in a tensor from spatial into batch dimensions.
 
     Parameters
     ----------
     x: tensor<[n, C, H, W], T> (Required)
-        * Input tensor must have rank 4.
-        * The first and the second dimension are batch, channel, respectively
-        * The remaining dimensions (H, W) are treated as "spatial dimensions"
+        * Input tensor must have rank ``4``.
+        * The first and the second dimension are batch, channel; respectively.
+        * The remaining dimensions ``(H, W)`` are treated as "spatial dimensions".
     block_shape: const tensor<[2], i32> (Required)
-        * The length of the block_shape must be `2`
-        * It defines the shapes of the block in which the spatial dimensions are divided
+        * The length of the ``block_shape`` must be ``2``.
+        * It defines the shapes of the block in which the spatial dimensions are divided.
     paddings: const tensor<[2, 2], i32> (Required)
-        * It must have shape `(2, 2)`
-        * It defines the padding for each spatial dimensions
+        * It must have shape ``(2, 2)``.
+        * It defines the padding for each spatial dimension.
 
     Returns
     -------
     tensor<[new_n, C, new_H, new_W], T>
-        * new_n = n * block_shape[0] * block_shape[1]
-        * new_H = (H + paddings[0][0] + padding[0][1])/block_shape[0]
-        * new_W = (W + paddings[1][0] + padding[1][1])/block_shape[1]
-        * The output has the same rank as the input
+        * ``new_n = n * block_shape[0] * block_shape[1]``
+        * ``new_H = (H + paddings[0][0] + padding[0][1])/block_shape[0]``
+        * ``new_W = (W + paddings[1][0] + padding[1][1])/block_shape[1]``
+        * The output has the same rank as the input.
 
     Attributes
     ----------
@@ -795,28 +795,28 @@ class space_to_batch(Operation):
 @register_op
 class batch_to_space(Operation):
     """
-    Rearrange elements in a tensor from batch into spatial dimension.
+    Rearrange elements in a tensor from batch into spatial dimensions.
 
     Parameters
     ----------
     x: tensor<[n, C, H, W], T> (Required)
-        * Input tensor must have rank 4.
-        * The first and the second dimension are batch, channel, respectively
-        * The remaining dimensions (H, W) are treated as "spatial dimensions"
+        * Input tensor must have rank ``4``.
+        * The first and the second dimension are batch, channel; respectively.
+        * The remaining dimensions ``(H, W)`` are treated as "spatial dimensions".
     block_shape: const tensor<[2], i32> (Required)
-        * The length of the block_shape must be `2`
-        * It defines the shapes of the block in which the spatial dimensions are multiplied
+        * The length of the ``block_shape`` must be ``2``.
+        * It defines the shapes of the block in which the spatial dimensions are multiplied.
     crops: const tensor<[2, 2], i32> (Required)
-        * It must have shape `(2, 2)`
-        * It defines the amount to crop from each spatial dimensions
+        * It must have shape ``(2, 2)``.
+        * It defines the amount to crop from each spatial dimension.
 
     Returns
     -------
     tensor<[new_n, C, new_H, new_W], T>
-        * new_n = n / (block_shape[0] * block_shape[1])
-        * new_H = (H * block_shape[0]) - paddings[0][0] - padding[0][1]
-        * new_W = (W * block_shape[1]) - paddings[1][0] - padding[1][1]
-        * The output has the same rank as the input
+        * ``new_n = n / (block_shape[0] * block_shape[1])``
+        * ``new_H = (H * block_shape[0]) - paddings[0][0] - padding[0][1]``
+        * ``new_W = (W * block_shape[1]) - paddings[1][0] - padding[1][1]``
+        * The output has the same rank as the input.
 
     Attributes
     ----------
@@ -884,6 +884,9 @@ class squeeze(Operation):
         * Must be at least 1-D.
     axes: const<K,i32> (Optional)
         * Axes to squeeze out.
+        * The behaviour of squeezing non-single dimensions follow PyTorch instead of NumPy, where
+          it ignores non-single dimensions instead of erroring out. More specifically, if x has
+          shape (2, 3, 4) and axes is [0, 1], the output will be a tensor with shape (2, 3, 4).
         * Default to remove all single-dimensions.
 
     Returns
@@ -923,9 +926,11 @@ class squeeze(Operation):
             for i in sorted(axes)[::-1]:  # descending order
                 if len(squeezed_shape) <= i:
                     raise ValueError(
-                        "Cannot squeeze dim {} for shape {}".format(i, squeezed_shape)
+                        f"Invalid axis {i} in squeeze. The axis should be smaller than {len(squeezed_shape)}"
                     )
-                squeezed_shape.pop(i)
+                if squeezed_shape[i] == 1:
+                    # Only remove the dim_size=1 dimension.
+                    squeezed_shape.pop(i)
 
         return types.tensor(x_type, tuple(squeezed_shape)) if len(squeezed_shape) != 0 else x_type
 
