@@ -9,7 +9,6 @@ from coremltools.converters.mil.mil import types
 from coremltools.converters.mil.mil.input_type import InputSpec, TensorInputType
 from coremltools.converters.mil.mil.operation import Operation
 from coremltools.converters.mil.mil.ops.defs._op_reqs import register_op
-from coremltools.converters.mil.mil.ops.defs._utils import restore_elements_from_packed_bits
 from coremltools.converters.mil.mil.ops.defs.iOS16 import _IOS16_TARGET
 
 
@@ -22,8 +21,10 @@ class constexpr_affine_dequantize(Operation):
     The quantized data is stored in the parameter ``quantized_data``.
     The other parameters -- ``scale``, ``zero_point``, and ``axis`` -- describe how
     unquantized values can be extracted from it, using the equation for affine/linear quantization:
-    ::
-                unquantized_data = scale * (quantized_data - zero_point)
+
+    .. sourcecode:: python
+
+        unquantized_data = scale * (quantized_data - zero_point)
 
     Although all of the parameters of this op are constants, this op is not constant folded
     to a single const op at the time of model serialization. The unquantized output will
@@ -38,15 +39,15 @@ class constexpr_affine_dequantize(Operation):
  	   * ``zero_point`` follows similar broadcasting rules and size constraints as ``scale``.
 
     scale: const tensor<DstT, [0..1]> (Required)
-       * ``scale`` can be either a scalar or a vector. If ``scale`` is a vector,
-         for implementation it is broadcast to the following shape:
-           * The rank of ``scale`` becomes the same as the rank of ``quantized_data``.
-           * The constraint: ``size(scale-vector) == quantized_data.shape[axis]``.
-           * For ``i == axis``, ``scale.shape[i] == quantized_data.shape[i]``.
-           * For ``i != axis``, ``scale.shape == 1``.
-         For example, assume ``quantized_data.shape = (2, 3, 4, 5)`` and ``axis = 1``.
-         If ``scale`` is a vector, then ``scale.size`` needs to be equal to
-         ``quantized_data.shape[axis] i.e = 3``, which would be broadcast to ``(1, 3, 1, 1)``.
+       * ``scale`` can be either a scalar or a vector.
+       * If ``scale`` is a vector, for implementation it is broadcast to the following shape:
+          * The rank of ``scale`` becomes the same as the rank of ``quantized_data``.
+          * The constraint: ``size(scale-vector) == quantized_data.shape[axis]``.
+          * For ``i == axis``, ``scale.shape[i] == quantized_data.shape[i]``.
+          * For ``i != axis``, ``scale.shape == 1``.
+            For example, assume ``quantized_data.shape = (2, 3, 4, 5)`` and ``axis = 1``.
+            If ``scale`` is a vector, then ``scale.size`` needs to be equal to
+            ``quantized_data.shape[axis] i.e = 3``, which would be broadcast to ``(1, 3, 1, 1)``.
 
     axis: const tensor<int32, []> (Required)
 
@@ -114,20 +115,37 @@ class constexpr_affine_dequantize(Operation):
             self.quantized_data.val, self.zero_point.val, self.scale.val, self.axis.val
         )
 
+    def is_all_zeros(self) -> bool:
+        zero_point = self.promote_rank_to_same_as_quantized_data(
+            self.zero_point.val, self.quantized_data.val, self.axis.val
+        )
+        return np.all(self.quantized_data.val == zero_point)
+
     @staticmethod
-    def decompress(quantized_data, zero_point, scale, axis):
+    def promote_rank_to_same_as_quantized_data(
+        param: np.ndarray, quantized_data: np.ndarray, axis: int
+    ) -> np.ndarray:
+        """
+        Promote param (i.e. zero point or scale) rank to same as quantized data,
+        so subtraction or multiplication can happen properly on the specified axis
+        """
+        if len(param.shape) == 0:
+            return np.reshape(param, np.ones(len(quantized_data.shape), np.int32))
+        else:
+            axes = [i for i in range(len(quantized_data.shape)) if i != axis]
+            return np.expand_dims(param, axis=tuple(axes))
 
+    @staticmethod
+    def decompress(
+        quantized_data: np.ndarray, zero_point: np.ndarray, scale: np.ndarray, axis: int
+    ) -> np.ndarray:
         axis = axis if axis >= 0 else axis + len(quantized_data.shape)
-
-        def rank_promoted_to_same_as_quantized_data(param):
-            if len(param.shape) == 0:
-                return np.reshape(param, np.ones(len(quantized_data.shape), np.int32))
-            else:
-                axes = [i for i in range(len(quantized_data.shape)) if i != axis]
-                return np.expand_dims(param, axis=tuple(axes))
-
-        sc = rank_promoted_to_same_as_quantized_data(scale)
-        zp = rank_promoted_to_same_as_quantized_data(zero_point)
+        sc = constexpr_affine_dequantize.promote_rank_to_same_as_quantized_data(
+            scale, quantized_data, axis
+        )
+        zp = constexpr_affine_dequantize.promote_rank_to_same_as_quantized_data(
+            zero_point, quantized_data, axis
+        )
         val = sc * (quantized_data.astype(np.float32) - zp.astype(np.float32))
         return val.astype(scale.dtype)
 
@@ -136,8 +154,10 @@ class constexpr_affine_dequantize(Operation):
 class constexpr_cast(Operation):
     """
     A compile-time operation that returns a constant output value upon casting its constant input.
-    ::
-                Expression: output = constexpr_cast(source_val, output_dtype="fp32")
+
+    .. sourcecode:: python
+
+        Expression: output = constexpr_cast(source_val, output_dtype="fp32")
 
     Parameters
     ----------
@@ -205,7 +225,9 @@ class constexpr_lut_to_dense(Operation):
     bits of the same index are filled in the LSBs of the next byte.
 
     For example:
-    ::
+
+    .. sourcecode:: python
+
         if n_bits = 2, shape = (5,) => M = 2 bytes
 
                     MSB             LSB
@@ -266,8 +288,11 @@ class constexpr_lut_to_dense(Operation):
 
     @staticmethod
     def decompress(lut, indices, shape):
+        # Import here to avoid circular import.
+        from coremltools.optimize.coreml import _utils as optimize_utils
+
         nbits = np.log2(lut.size).astype(np.int32)
-        indices = restore_elements_from_packed_bits(indices, nbits, np.prod(shape))
+        indices = optimize_utils.restore_elements_from_packed_bits(indices, nbits, np.prod(shape))
         flatten_val = lut[indices]
         return flatten_val.reshape(shape)
 
@@ -290,8 +315,8 @@ class constexpr_sparse_to_dense(Operation):
 
     shape: const tensor<uint32, [K]> (Required)
 
-	Notes
-	-----
+        Notes
+        -----
     * Any data is packed and read in a row-major order.
     * ``mask`` contains ``M`` bytes, where ``M = ceil( product(shape) / 8)``. That is, each bit
       field corresponds to one element in the output tensor.
@@ -301,7 +326,9 @@ class constexpr_sparse_to_dense(Operation):
     moving up to the most significant bit.
 
     For example:
-    ::
+
+    .. sourcecode:: python
+
         shape = (5,) => M = 1 bytes
 
                    MSB                  LSB
@@ -310,6 +337,7 @@ class constexpr_sparse_to_dense(Operation):
                    |--|--|--|i4|i3|i2|i1|i0|      <== tagged element ids
                    |      byte 0           |      <== tagged bytes
 
+
     Returns
     -------
     const tensor<T, [...]>
