@@ -21,10 +21,10 @@ from coremltools.converters.mil.mil.ops.defs.iOS16 import _IOS16_TARGET
 @register_op(opset_version=_IOS16_TARGET)
 class resample(_resample_iOS15):
     """
-    The iOS 16 version of ``resample`` supports float 16 coordinates.
-    
-    For the complete documentation, see the 
-    `iOS 15 version <#module-coremltools.converters.mil.mil.ops.defs.iOS15.image_resizing>`_.
+    This version of ``resample`` supports float 16 coordinates.
+
+    For complete documentation, see the
+    iOS 15 :py:class:`~.iOS15.image_resizing.resample`.
     """
     input_spec = InputSpec(
         x=TensorInputType(type_domain="T"),
@@ -47,32 +47,32 @@ class resample(_resample_iOS15):
 @register_op(opset_version=_IOS16_TARGET)
 class upsample_bilinear(_upsample_bilinear_iOS15):
     """
-    iOS16 version of upsample_bilinear supports half_pixel_centers
+    This version of ``upsample_bilinear`` supports ``half_pixel_centers``.
+    For complete documentation, see the
+    iOS 15 :py:class:`~.iOS15.image_resizing.upsample_bilinear`.
 
-    Additional Parameters
+    Parameters
     ----------
     half_pixel_centers: const<bool> (Optional)
-        * Default to !align_corners if not provided
+        * Defaults to ``!align_corners`` if not provided.
     """
 
     input_spec = _upsample_bilinear_iOS15.input_spec + InputSpec(
         half_pixel_centers=TensorInputType(const=True, optional=True, type_domain=types.bool),
     )
 
-    def default_inputs(self):
-        return super().default_inputs() + DefaultInputs(half_pixel_centers=not self.align_corners.val)
-
 @register_op(opset_version=_IOS16_TARGET)
 class crop_resize(_crop_resize_iOS15):
     """
-    iOS16 version of crop_resize, which supports ``pad_value``
-    
-    Additional Parameters
+    This version differs from the iOS 15 :py:class:`~.iOS15.image_resizing.crop_resize`
+    by supporting ``pad_value`` as an additional parameter.
+
+    Parameters
     ----------
-    pad_value : const<T> (Optional, default=1.0)
-        * If the box indexes go beyond the input boundary, the input image is padded with pad_value.
-        * Defaults to 0.
-        * It is the same as extrapolation_value in tf.image.crop_and_resize.
+    pad_value : const<T> (Optional, default=0.0)
+        * If the box indexes go beyond the input boundary, the input image is padded with ``pad_value``.
+        * Defaults to ``0``.
+        * It is the same as ``extrapolation_value`` in `tf.image.crop_and_resize <https://www.tensorflow.org/api_docs/python/tf/image/crop_and_resize>`_.
 
     Attributes
     ----------
@@ -83,4 +83,4 @@ class crop_resize(_crop_resize_iOS15):
     )
 
     def default_inputs(self):
-        return super().default_inputs() + DefaultInputs(pad_value=1.0)
+        return super().default_inputs() + DefaultInputs(pad_value=0.0)
