@@ -4,33 +4,35 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 from coremltools.converters.mil.mil import Operation, types
-from coremltools.converters.mil.mil.input_type import (DefaultInputs,
-                                                       InputSpec,
-                                                       TensorInputType)
-from coremltools.converters.mil.mil.operation import (SYMBOL, VALUE,
-                                                      precondition)
+from coremltools.converters.mil.mil.input_type import DefaultInputs, InputSpec, TensorInputType
+from coremltools.converters.mil.mil.operation import SYMBOL, VALUE, precondition
 from coremltools.converters.mil.mil.ops.defs._op_reqs import register_op
 from coremltools.converters.mil.mil.ops.defs._utils import compute_gather
+from coremltools.converters.mil.mil.ops.defs.iOS15.scatter_gather import (
+    gather_along_axis as _gather_along_axis_iOS15,
+)
 from coremltools.converters.mil.mil.ops.defs.iOS16 import _IOS16_TARGET
 
 
 @register_op(opset_version=_IOS16_TARGET)
 class gather(Operation):
     """
-    An iOS16 version of gather
-    
-    The new gather op supports `batch_dims`
-    similar to `tf.gather <https://www.tensorflow.org/api_docs/python/tf/gather>`_.
+    The iOS16 version.
+    This section documents only the differences between this version and the
+    iOS 15 :py:class:`~.iOS15.scatter_gather.gather`.
+
+    This version supports ``batch_dims``, similar to `tf.gather <https://www.tensorflow.org/api_docs/python/tf/gather>`_.
+    Input parameter ``indices`` now supports ``int16`` and ``uint16``.
 
     Parameters
     ----------
-    x: tensor<\*D, U> (Required)
+    x: tensor<\*D, T> (Required)
     indices: tensor<\*N, I> (Required)
         * Indices values may be negative. More precisely, ``-D[axis]<= v < D[axis]`` for ``v`` in ``indices``.
     axis: const i32 (Optional. Default=``0``)
         * Negative axis is supported.
     batch_dims: const i32 (Optional. Default=``0``)
-        * The number of batch dimensions
+        * The number of batch dimensions.
 
     Returns
     -------
@@ -49,14 +51,14 @@ class gather(Operation):
     """
 
     input_spec = InputSpec(
-        x=TensorInputType(type_domain="U"),
+        x=TensorInputType(type_domain="T"),
         indices=TensorInputType(type_domain="I"),
         axis=TensorInputType(const=True, optional=True, type_domain=types.int32),
         batch_dims=TensorInputType(const=True, optional=True, type_domain=types.int32)
     )
-    
+
     type_domains = {
-        "U": (types.fp16, types.fp32, types.int32),
+        "T": (types.fp16, types.fp32, types.int32),
         "I": (types.int32, types.uint16, types.int16),
     }
 
@@ -74,11 +76,11 @@ class gather(Operation):
             # only allow x to be symbolic. indices cannot.
             return None
         return compute_gather(
-                params=self.x.sym_val, 
-                indices=self.indices.val, 
-                axis=self.axis.val,
-                batch_dims=self.batch_dims.val
-            )
+            params=self.x.sym_val,
+            indices=self.indices.val,
+            axis=self.axis.val,
+            batch_dims=self.batch_dims.val,
+        )
 
     def type_inference(self):
         # validate parameters
@@ -99,7 +101,7 @@ class gather(Operation):
                 "batch_dims {} must be less or equal to than indices.rank {} for node {}".format(
                     self.batch_dims.val, self.indices.rank, self.name
                 )
-            )         
+            )
 
         output_rank = self.x.rank - 1 + self.indices.rank - self.batch_dims.val
         if output_rank == 0:
@@ -114,18 +116,60 @@ class gather(Operation):
 
         return types.tensor(self.x.dtype, out_shape)
 
+
 @register_op(opset_version=_IOS16_TARGET)
-class gather_nd(Operation):
+class gather_along_axis(_gather_along_axis_iOS15):
     """
-    An iOS16 version of gather_nd
-    The new gather_nd op supports `batch_dims`
+    The iOS16 version.
+    The only difference between this version and the iOS 15 :py:class:`~.iOS15.scatter_gather.gather_along_axis`.
+    is that input parameter ``indices`` now supports ``int16`` and ``uint16``.
 
     Parameters
     ----------
     x: tensor<\*D, T> (Required)
-    indices: tensor<\*K, i32> (Required)
+    indices: tensor<\*K, I> (Required)
+    axis: const i32 (Optional):
+        * Default to ``0``.
+
+    Returns
+    -------
+    tensor<\*D, T>:
+        * Output tensor has the same shape as ``indices``.
+
+    Attributes
+    ----------
+    T: fp16, fp32, i32
+    I: uint16, int16, int32
+    """
+
+    input_spec = InputSpec(
+        x=TensorInputType(type_domain="T"),
+        indices=TensorInputType(type_domain="I"),
+        axis=TensorInputType(const=True, optional=True, type_domain=types.int32),
+    )
+
+    type_domains = {
+        "T": (types.fp16, types.fp32, types.int32),
+        "I": (types.int32, types.uint16, types.int16),
+    }
+
+
+@register_op(opset_version=_IOS16_TARGET)
+class gather_nd(Operation):
+    """
+    The iOS16 version.
+    This section documents only the differences between this version and the
+    iOS 15 :py:class:`~.iOS15.scatter_gather.gather_nd`.
+
+    This version supports ``batch_dims``.
+    Input parameter ``indices`` now supports ``int16`` and ``uint16``.
+
+    Parameters
+    ----------
+    x: tensor<\*D, T> (Required)
+    indices: tensor<\*K, I> (Required)
     batch_dims: const i32 (Optional. Default=``0``)
-        * The number of batch dimensions
+        * The number of batch dimensions.
 
     Returns
     -------
@@ -135,6 +179,7 @@ class gather_nd(Operation):
     Attributes
     ----------
     T: fp16, fp32, i32
+    I: uint16, int16, int32
 
     References
     ----------
@@ -142,13 +187,13 @@ class gather_nd(Operation):
     """
 
     input_spec = InputSpec(
-        x=TensorInputType(type_domain="U"),
+        x=TensorInputType(type_domain="T"),
         indices=TensorInputType(type_domain="I"),
         batch_dims=TensorInputType(const=True, optional=True, type_domain=types.int32),
     )
-    
+
     type_domains = {
-        "U": (types.fp16, types.fp32, types.int32),
+        "T": (types.fp16, types.fp32, types.int32),
         "I": (types.int32, types.uint16, types.int16),
     }
 
