@@ -436,6 +436,23 @@ class TestConv:
         config,
         x_weight_dtype,
     ):
+        if (
+            backend.backend == 'neuralnetwork' and
+            conv_dim == "conv2d" and
+            config == {
+                "padding": (1, 1, 1),
+                "DHWKdKhKw": (5, 5, 5, 2, 2, 2),
+                "stride": (2, 2, 2),
+                "dilation": (2, 1, 1),
+                "has_bias": True,
+                "groups": 1,
+                "symbolic": True,
+            }
+        ):
+            pytest.xfail(
+                "rdar://129121584: NN Conv Fail when Run Multiple Faulty Models at Same Time"
+            )
+
         padding = config["padding"]
         DHWKdKhKw = config["DHWKdKhKw"]
         stride = config["stride"]
