@@ -413,6 +413,11 @@ class TestReshape:
         ),
     )
     def test_builder_to_backend_symbolic(self, compute_unit, backend):
+        if backend.backend == "mlprogram":
+            pytest.xfail(
+                "rdar://131637870 Why It Randomly Segfaults on CI but Cannot Reproduce Locally "
+            )
+
         s0 = get_new_symbol()
         s_len = get_new_symbol()
 
@@ -1058,7 +1063,7 @@ class TestSliceByIndex:
             )
             return x
 
-        x = np.random.rand(*INPUT_SHAPE)
+        x = np.float16(np.random.rand(*INPUT_SHAPE))
 
         # slice by index is x[begin[0]: end[0]: stride[0], begin[1]: end[1]: stride[1], ...]
         y_numpy = x[0:1:1, 0:2:1, 0:8:2, 0:12:2]
@@ -1268,6 +1273,63 @@ class TestSqueeze:
             assert const.val[0, 0, 0] == 112
             return x
 
+    @staticmethod
+    def test_squeeze_invalid_axis():
+        with pytest.raises(
+            ValueError, match="Invalid axis 3 in squeeze. The axis should be smaller than 3"
+        ):
+
+            @mb.program()
+            def prog():
+                const = mb.const(val=[[[2, 3], [4, 5]]])
+                x = mb.squeeze(x=const, axes=(3,))
+                return x
+
+    @pytest.mark.parametrize(
+        "compute_unit, backend, is_symbolic",
+        itertools.product(
+            compute_units,
+            backends,
+            (True, False),
+        ),
+    )
+    def test_non_single_element_dim(self, compute_unit, backend, is_symbolic):
+        if backend.backend == "neuralnetwork":
+            pytest.skip("neuralnetwork backend doesn't support squeeze a not-1 dimension")
+        if compute_unit == ct.ComputeUnit.CPU_ONLY:
+            pytest.xfail("CPU failed non-single-dim squeeze (rdar://124555262)")
+
+        x = np.arange(2 * 3 * 4, dtype=np.int32).reshape(2, 3, 4)
+        input_shape = (
+            [get_new_symbol(), get_new_symbol(), get_new_symbol()] if is_symbolic else x.shape
+        )
+        input_placeholders = {"x": mb.placeholder(shape=input_shape)}
+        input_values = {"x": x}
+
+        def build(x):
+            return [
+                mb.squeeze(x=x, axes=(-1,)),
+                mb.squeeze(x=x, axes=(-2, 0)),
+                mb.squeeze(x=x, axes=(0, 1, 2)),
+                mb.squeeze(x=x),
+            ]
+
+        # The symbolic dim won't be squeezed, so it doesn't affect the output.
+        expected_output_types = [tuple(input_shape) + (types.int32,)] * 4
+        expected_outputs = [x] * 4
+        run_compare_builder(
+            build,
+            input_placeholders,
+            input_values,
+            expected_output_types,
+            expected_outputs,
+            inputs=construct_inputs_from_placeholders(input_placeholders, 10)
+            if backend.backend == "mlprogram"
+            else None,
+            compute_unit=compute_unit,
+            backend=backend,
+        )
+
 
 class TestTranspose:
     @pytest.mark.parametrize(
