@@ -34,12 +34,15 @@ from coremltools.converters.mil.mil.passes.defs import (
     cleanup,
     lower_complex_dialect_ops,
     optimize_activation,
+    optimize_activation_quantization,
     optimize_conv,
     optimize_elementwise_binary,
     optimize_linear,
     optimize_normalization,
     optimize_quantization,
     optimize_repeat_ops,
+    optimize_state,
     optimize_tensor_operation,
     preprocess,
+    symbol_transform,
 )
