@@ -5,13 +5,16 @@
 
 from abc import abstractmethod
 from enum import Enum as _Enum
-from typing import Set, Text
+from typing import Dict, Set, Text, Tuple
 
 import numpy as np
 
+from coremltools import _logger as logger
 from coremltools.converters.mil._deployment_compatibility import AvailableTarget
+from coremltools.converters.mil.input_types import TensorType
+from coremltools.converters.mil.mil import Block
 from coremltools.converters.mil.mil import Builder as mb
-from coremltools.converters.mil.mil import Operation, types
+from coremltools.converters.mil.mil import Function, Operation, Var, types
 from coremltools.converters.mil.mil.block import is_current_opset_version_compatible_with
 from coremltools.converters.mil.mil.ops.registry import SSAOpRegistry
 from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
@@ -19,6 +22,7 @@ from coremltools.converters.mil.mil.passes.helper import block_context_manager
 from coremltools.converters.mil.mil.passes.pass_registry import register_pass
 from coremltools.converters.mil.mil.program import Program
 from coremltools.converters.mil.mil.types.symbolic import is_symbolic
+from coremltools.converters.mil.mil.types.type_mapping import string_to_builtin
 
 
 class ComputePrecision(_Enum):
@@ -47,6 +51,14 @@ class AbstractQuantizationPass(AbstractGraphPass):
             )
         self.op_selector = op_selector
 
+        # Var that feeds into multiple ops will be cast once and cached into this dict
+        # For reference: Checkout test_single_input_to_multiple_operations in `TestFP16CastTransform`.
+        # Note that, we make it a stack of dict to keep tracking the blocks
+        self._cache_vars = []
+
+    def current_cache_vars(self) -> Set[Var]:
+        return self._cache_vars[-1]
+
     def apply(self, prog):
         """
         Walks over each operation in the graph and performs following two steps,
@@ -68,6 +80,7 @@ class AbstractQuantizationPass(AbstractGraphPass):
 
         @block_context_manager
         def apply_block(block):
+            self._cache_vars.append({})
             for op in list(block.operations):
                 for b in op.blocks:
                     apply_block(b)
@@ -80,6 +93,7 @@ class AbstractQuantizationPass(AbstractGraphPass):
                         need_transform = op.op_type not in getattr(self, "skip_ops_by_type", set())
                     if need_transform:
                         self.transform_op(op)
+            self._cache_vars.pop()
 
         for f in prog.functions.values():
             apply_block(f)
@@ -135,10 +149,6 @@ class CastTypeQuantization(AbstractQuantizationPass):
     def __init__(self, op_selector=None):
         super().__init__(op_selector=op_selector)
 
-        # Var that feeds into multiple ops will be cast once and cached into this dict
-        # For reference: Checkout test_single_input_to_multiple_operations in `TestFP16CastTransform`.
-        self.cache_vars = {}
-
     @property
     @abstractmethod
     def origin_dtype(self) -> str:
@@ -151,6 +161,91 @@ class CastTypeQuantization(AbstractQuantizationPass):
         """Target dtype, such as fp16."""
         raise NotImplementedError("target_dtype must be specified in subclass.")
 
+    # TODO: rdar://122845072 ([Infra] Refactor the transform_function_signatures, adjust_io_to_supported_types and update_output_dtypes using a shared graph pass)
+    @block_context_manager
+    def transform_function_signatures(self, func: Function) -> None:
+        """
+        This utility transform a function input / output signatures from the original_dtype to
+        the target_dtype.
+
+        For instance, in the add_fp16_cast class, this member function transforms the following
+        function:
+
+            function(%input(fp32)) {
+              block0() {
+                % var_1 = op_1(x=%input)
+                ...
+                % output(fp32) = ...
+              } -> (%output)
+            }
+
+        into:
+
+            function(%input(fp16)) {
+              block0() {
+                # input_cast = cast(x=input, dtype="fp32")
+                % var_1 = op_1(x=%input_cast)
+                ...
+                % output(fp32) = ...
+              } -> (%output)
+            }
+
+        and function.output_types is set to [TensorType(dtype=types.fp16)],
+        in which will be used in common::update_output_dtypes to upgrade the function output dtype accordingly.
+
+        """
+        # reset input signatures
+        old_func_inputs = func.inputs
+        new_func_inputs = {}
+        cache_vars = {}
+
+        # cast the new input into the original dtype
+        for k, v in old_func_inputs.items():
+            if v.is_tensor_or_scalar_of(self.origin_dtype):
+                new_input = mb.placeholder(
+                    shape=v.shape,
+                    dtype=string_to_builtin(self.target_dtype),
+                    name=v.name,
+                ).outputs[0]
+
+                if v in func.outputs:
+                    new_outputs = []
+                    for val in func.outputs:
+                        new_outputs.append(new_input if val == v else val)
+                    func.set_outputs(new_outputs)
+
+                new_func_inputs[k] = new_input
+                cast_input = mb.cast(
+                    x=new_input,
+                    dtype=self.origin_dtype,
+                    before_op=func.operations[0] if len(func.operations) > 0 else None,
+                )
+                cache_vars[k] = cast_input
+            else:
+                new_func_inputs[k] = v
+                cache_vars[k] = v
+
+        # replace the use of the old input vars with the new cast var
+        for k, v in old_func_inputs.items():
+            func.replace_uses_of_var_after_op(
+                anchor_op=None,
+                old_var=v,
+                new_var=cache_vars[k],
+            )
+        func._input_dict = new_func_inputs
+
+        # reset output signatures
+        if func.output_types is None:
+            output_types = [TensorType(dtype=v.dtype) for v in func.outputs]
+        else:
+            output_types = func.output_types
+
+        for idx, v in enumerate(output_types):
+            if v.dtype == string_to_builtin(self.origin_dtype):
+                output_types[idx] = TensorType(dtype=string_to_builtin(self.target_dtype))
+
+        func.output_types = output_types
+
     def should_cast_parameter(self, op: Operation, param_name: str) -> bool:
         """
         Determines if a param of an op should be cast to target_dtype.
@@ -167,6 +262,13 @@ class CastTypeQuantization(AbstractQuantizationPass):
 
         return True
 
+    def _get_casted_outputs(self, op: Operation, casted_inputs: Dict[str, Var]) -> Tuple[Var]:
+        """
+        Given an op and casted_inputs, this utility returns the new resulting outputs.
+        """
+        return getattr(mb, op.op_type)(**casted_inputs)
+
+
     def transform_op(self, op) -> None:
         """Transform the input(s)/output(s) dtypes of the op."""
         block = op.enclosing_block
@@ -190,18 +292,28 @@ class CastTypeQuantization(AbstractQuantizationPass):
                 casted_var_name = f"{var.name}_to_{self.target_dtype}"
                 if (
                     len(var._child_ops) > 1
-                    and casted_var_name in self.cache_vars
-                    and (block.is_var_visible_in_block(self.cache_vars[casted_var_name]))
+                    and casted_var_name in self.current_cache_vars()
                 ):
-                    casted_inputs[param][i] = self.cache_vars[casted_var_name]
+                    if self.current_cache_vars()[casted_var_name].op.x != var:
+                        logger.warning(
+                            "The cached cast Var doesn't match the original Var. It's due to duplicated Var "
+                            f"names in the graph for {casted_var_name}."
+                        )
+                    casted_inputs[param][i] = self.current_cache_vars()[casted_var_name]
                 else:
-                    x = mb.cast(x=var, dtype=self.target_dtype, name=casted_var_name, before_op=op)
+                    x = mb.cast(
+                        x=var,
+                        dtype=self.target_dtype,
+                        name=casted_var_name,
+                        before_op=op,
+                    )
                     if self.target_dtype == "fp16":
                         self._check_underflow_to_zero(x, var)
+                    Block._copy_metadata(var, x)
 
                     casted_inputs[param][i] = x
                     if len(var._child_ops) > 1:
-                        self.cache_vars[casted_var_name] = casted_inputs[param][i]
+                        self.current_cache_vars()[casted_var_name] = casted_inputs[param][i]
 
             if not is_list_input:
                 casted_inputs[param] = casted_inputs[param][0]
@@ -210,7 +322,7 @@ class CastTypeQuantization(AbstractQuantizationPass):
             casted_inputs.update({k: v for k, v in op.inputs.items() if k not in casted_inputs})
             casted_inputs["name"] = f"{op.name}_cast_{self.target_dtype}"
             casted_inputs["before_op"] = op
-            quant_output = getattr(mb, op.op_type)(**casted_inputs)
+            quant_output = self._get_casted_outputs(op, casted_inputs)
 
             if not isinstance(quant_output, (list, tuple)):
                 quant_output = [quant_output]
@@ -232,6 +344,7 @@ class CastTypeQuantization(AbstractQuantizationPass):
                         force_replace=True,
                     )
                 else:
+
                     op.enclosing_block.replace_uses_of_var_after_op(
                         anchor_op=op,
                         old_var=old_output_var,
@@ -272,6 +385,8 @@ class FP16ComputePrecision(CastTypeQuantization):
         "list_read",
         "list_write",
         "list_length",
+        "read_state",
+        "coreml_update_state",
     }
 
     def __init__(self, op_selector=None):
@@ -401,17 +516,24 @@ class add_fp16_cast(FP16ComputePrecision):
 @register_pass(namespace="common")
 class add_int16_cast(CastTypeQuantization):
     """
-    This transform does the following, for each op that supports int16:
-    - For each input of dtype int32 which actually supports int16, inject a "cast" op to change it
-      to int16 dtype.
-    - For each output of dtype int16, inject a "cast" op to change it back to int32.
-    It's mainly for int16 op ANE residency.
+    This transform does the following, for each op that supports int16/uint16:
+    - For each input of dtype int32 which supports int16/uint16, inject a "cast" op to change it
+      to int16/uint16 dtype.
+    - For each output of dtype int16/uint16, inject a "cast" op to change it back to int32.
+    Notice that the cast will not be inserted if the const value is out of int16/uint16 range.
     """
     # Ops that prefer int16 params.
-    _PREFER_INT16_OPS: Set[str] = {"gather", "gather_along_axis", "gather_nd"}
+    # If an op supports 16-bit only in later iOS (e.g. gather started to support 16-bit from iOS16)
+    # then int16 cast will be inserted only if the iOS version is high enough
+    # (e.g. nothing will happen for iOS15 gather)
+    # This is achieved by type domain confirmation in `CastTypeQuantization.should_cast_parameter`
+    _PREFER_INT16_OPS: Set[str] = {"gather", "gather_along_axis", "gather_nd", "squeeze"}
 
     def __init__(self, op_selector=None):
         super().__init__(op_selector=op_selector)
+        # Use variable instead of hard-coded "int16" because the target dtype could be uint16
+        # depending on if the param is non-negative const and within uint16 range.
+        self._target_dtype: str = "int16"
 
     @property
     def origin_dtype(self) -> str:
@@ -419,38 +541,56 @@ class add_int16_cast(CastTypeQuantization):
 
     @property
     def target_dtype(self) -> str:
-        return "int16"
+        return self._target_dtype
 
-    @staticmethod
-    def int16_overflow(op: Operation) -> bool:
+    @target_dtype.setter
+    def target_dtype(self, target_dtype: str):
+        if target_dtype not in {"int16", "uint16"}:
+            raise ValueError("The target_dtype in add_int16_cast must be int16 or uint16")
+        self._target_dtype = target_dtype
+
+    def should_cast_parameter(self, op: Operation, param_name: str) -> bool:
         """
-        Determines if any of the op's input will overflow when represented by int16. Constants with
-        values more than np.iinfo(np.int16).max or less than np.iinfo(np.int16).min overflows in int16.
+        Determine if a parameter should be cast or not.
+        If should be cast, determine whether to use int16 or uint16.
         """
         _INT16_MAX = np.iinfo(np.int16).max
         _INT16_MIN = np.iinfo(np.int16).min
-        for _, inputs in op.inputs.items():
-            is_list_input = isinstance(inputs, (list, tuple))
-            if not is_list_input:
-                inputs = [inputs]
-            for var in inputs:
-                if var.val is not None and var.is_tensor_or_scalar_of(dtype="int32"):
-                    if np.any(var.val > _INT16_MAX) or np.any(var.val < _INT16_MIN):
-                        return True
+        _UINT16_MAX = np.iinfo(np.uint16).max
+        _UINT16_MIN = np.iinfo(np.uint16).min
 
-        # In `gather` and `gather_along_axis`, if the dim size of x is larger than int16 upperbound,
-        # the dynamic indices could overflow.
-        if (
-            op.op_type in {"gather", "gather_along_axis"}
-            and op.indices.val is None
-            and op.x.shape is not None
-        ):
-            dim_size = op.x.shape[op.axis.val]
-            if not is_symbolic(dim_size) and dim_size > _INT16_MAX:
-                return True
+        input_var = op.inputs[param_name]
+        if not input_var.is_tensor_or_scalar_of(dtype="int32"):
+            return False
 
-        return False
+        input_op = input_var.op
+        if input_op is not None and input_op.op_type == "const":
+            if (
+                input_op.outputs[0].val.min() >= _UINT16_MIN
+                and input_op.outputs[0].val.max() <= _UINT16_MAX
+            ):
+                self._target_dtype = "uint16"
+            elif (
+                input_op.outputs[0].val.min() >= _INT16_MIN
+                and input_op.outputs[0].val.max() <= _INT16_MAX
+            ):
+                self._target_dtype = "int16"
+            else:
+                return False
+
+        # In `gather` and `gather_along_axis`, if the dim size of x is larger than int16
+        # upperbound, the dynamic indices could overflow, so it shouldn't be cast.
+        if op.op_type in {"gather", "gather_along_axis"} and param_name == "indices":
+            if op.indices.val is None and op.x.shape is not None:
+                dim_size = op.x.shape[op.axis.val]
+                if not is_symbolic(dim_size) and dim_size > _INT16_MAX:
+                    return False
+
+        if not super().should_cast_parameter(op, param_name):
+            return False
+
+        return True
 
     def is_valid_op(self, op: Operation) -> bool:
-        """Determines if op is valid for int16 casting."""
-        return op.op_type in self._PREFER_INT16_OPS and not self.int16_overflow(op)
+        """Determines if op is valid for int16/uint16 casting."""
+        return op.op_type in self._PREFER_INT16_OPS
