@@ -117,3 +117,10 @@ class TestPassPipeline:
         assert len(available_pipelines) == 12
         assert "default" in available_pipelines
         assert "default_palettization" in available_pipelines
+
+    @staticmethod
+    def test_get_pipeline_should_use_copy():
+        pipeline = PassPipeline.DEFAULT_PRUNING
+        pipeline.append_pass("compression::palettize_weights")
+        pipeline_2 = PassPipeline.DEFAULT_PRUNING
+        assert "compression::palettize_weights" not in pipeline_2.passes
