@@ -4,11 +4,13 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 from collections import namedtuple
+from typing import Optional, Union
 
 import numpy as _np
 import numpy as np
 import sympy as sm
 
+import coremltools.converters.mil.backend.mil.helper as mil_helper
 import coremltools.proto.MIL_pb2 as _mil_pm
 
 from .get_type_info import get_type_info
@@ -22,11 +24,27 @@ from .type_double import fp16 as types_fp16
 from .type_double import fp32 as types_fp32
 from .type_double import fp64 as types_fp64
 from .type_double import is_float
+from .type_int import SUB_BYTE_DTYPE_METADATA_KEY
+from .type_int import int4 as types_int4
 from .type_int import int8 as types_int8
 from .type_int import int16 as types_int16
 from .type_int import int32 as types_int32
 from .type_int import int64 as types_int64
-from .type_int import is_int
+from .type_int import (
+    is_int,
+    is_sub_byte,
+    np_int4_dtype,
+    np_uint1_dtype,
+    np_uint2_dtype,
+    np_uint3_dtype,
+    np_uint4_dtype,
+    np_uint6_dtype,
+)
+from .type_int import uint1 as types_uint1
+from .type_int import uint2 as types_uint2
+from .type_int import uint3 as types_uint3
+from .type_int import uint4 as types_uint4
+from .type_int import uint6 as types_uint6
 from .type_int import uint8 as types_uint8
 from .type_int import uint16 as types_uint16
 from .type_int import uint32 as types_uint32
@@ -37,10 +55,16 @@ from .type_unknown import unknown
 
 _TYPES_TO_NPTYPES = {
     types_bool: np.bool_,
+    types_int4: np_int4_dtype,
     types_int8: np.int8,
     types_int16: np.int16,
     types_int32: np.int32,
     types_int64: np.int64,
+    types_uint1: np_uint1_dtype,
+    types_uint2: np_uint2_dtype,
+    types_uint3: np_uint3_dtype,
+    types_uint4: np_uint4_dtype,
+    types_uint6: np_uint6_dtype,
     types_uint8: np.uint8,
     types_uint16: np.uint16,
     types_uint32: np.uint32,
@@ -73,10 +97,16 @@ _NPTYPES_TO_STRINGS = {
 
 _TYPES_TO_STRINGS = {
     types_bool: "bool",
+    types_int4: "int4",
     types_int8: "int8",
     types_int16: "int16",
     types_int32: "int32",
     types_int64: "int64",
+    types_uint1: "uint1",
+    types_uint2: "uint2",
+    types_uint3: "uint3",
+    types_uint4: "uint4",
+    types_uint6: "uint6",
     types_uint8: "uint8",
     types_uint16: "uint16",
     types_uint32: "uint32",
@@ -91,7 +121,13 @@ _TYPES_TO_STRINGS = {
 
 _TYPES_TO_RESOLUTION = {
     types_bool: 1,
+    types_int4: 1,
     types_int8: 1,
+    types_uint1: 1,
+    types_uint2: 1,
+    types_uint3: 1,
+    types_uint4: 1,
+    types_uint6: 1,
     types_uint8: 1,
     types_int16: 1,
     types_uint16: 1,
@@ -106,7 +142,13 @@ RangeTuple = namedtuple("RangeTuple", "low high")
 
 _TYPES_TO_RANGE = {
     types_bool: RangeTuple(0, 1),
+    types_int4: RangeTuple(np.iinfo(np.int8).min >> 4, np.iinfo(np.int8).max >> 4),
     types_int8: RangeTuple(np.iinfo(np.int8).min, np.iinfo(np.int8).max),
+    types_uint1: RangeTuple(np.iinfo(np.uint8).min >> 7, np.iinfo(np.uint8).max >> 7),
+    types_uint2: RangeTuple(np.iinfo(np.uint8).min >> 6, np.iinfo(np.uint8).max >> 6),
+    types_uint3: RangeTuple(np.iinfo(np.uint8).min >> 5, np.iinfo(np.uint8).max >> 5),
+    types_uint4: RangeTuple(np.iinfo(np.uint8).min >> 4, np.iinfo(np.uint8).max >> 4),
+    types_uint6: RangeTuple(np.iinfo(np.uint8).min >> 2, np.iinfo(np.uint8).max >> 2),
     types_uint8: RangeTuple(np.iinfo(np.uint8).min, np.iinfo(np.uint8).max),
     types_int16: RangeTuple(np.iinfo(np.int16).min, np.iinfo(np.int16).max),
     types_uint16: RangeTuple(np.iinfo(np.uint16).min, np.iinfo(np.uint16).max),
@@ -127,7 +169,13 @@ BUILTIN_TO_PROTO_TYPES = {
     types_fp64: _mil_pm.FLOAT64,
 
     # int
+    types_uint1: _mil_pm.UINT1,
+    types_uint2: _mil_pm.UINT2,
+    types_uint3: _mil_pm.UINT3,
+    types_uint4: _mil_pm.UINT4,
+    types_uint6: _mil_pm.UINT6,
     types_uint8: _mil_pm.UINT8,
+    types_int4: _mil_pm.INT4,
     types_int8: _mil_pm.INT8,
 
     types_uint16: _mil_pm.UINT16,
@@ -158,6 +206,16 @@ def np_dtype_to_py_type(np_dtype):
 PROTO_TO_BUILTIN_TYPE = {v: k for k, v in BUILTIN_TO_PROTO_TYPES.items()}
 _STRINGS_TO_TYPES = {v: k for k, v in _TYPES_TO_STRINGS.items()}
 _STRINGS_TO_NPTYPES = {v: k for k, v in _NPTYPES_TO_STRINGS.items()}
+_STRINGS_TO_NPTYPES.update(
+    {
+        "int4": np_int4_dtype,
+        "uint1": np_uint1_dtype,
+        "uint2": np_uint2_dtype,
+        "uint3": np_uint3_dtype,
+        "uint4": np_uint4_dtype,
+        "uint6": np_uint6_dtype,
+    }
+)
 
 def string_to_builtin(s):
     """
@@ -191,12 +249,12 @@ def nptype_from_builtin(btype):
 
 def builtin_to_resolution(builtin_type: type):
     """
-    Given a builtin type, return its corrsponding resolution.
+    Given a builtin type, return its corresponding resolution.
     """
     return _TYPES_TO_RESOLUTION[builtin_type]
 
 
-def builtin_to_range(builtin_type: type):
+def builtin_to_range(builtin_type: type) -> RangeTuple:
     """
     Given a builtin type, return its corresponding range.
     """
@@ -341,54 +399,71 @@ def is_builtin(t):
     return is_scalar(t) or is_tensor(t) or is_str(t) or is_tuple(t)
 
 
-# Converts a numpy type to its types equivalent.
-# Supports both dtypes and numpy primitive types.
-def numpy_type_to_builtin_type(nptype):
+def _numpy_dtype_instance_to_builtin_type(np_dtype: np.dtype) -> Optional[type]:
+    metadata_dict = np_dtype.metadata
+    if metadata_dict is not None and SUB_BYTE_DTYPE_METADATA_KEY in metadata_dict:
+        return metadata_dict[SUB_BYTE_DTYPE_METADATA_KEY]
+
+    if np_dtype in _NPTYPES_TO_STRINGS:
+        return string_to_builtin(_NPTYPES_TO_STRINGS[np_dtype])
+    return None
+
+
+def numpy_type_to_builtin_type(nptype) -> type:
+    """
+    Converts a numpy type to its builtin `types` equivalent.
+    Supports Python native types and numpy types.
+    """
+    if isinstance(nptype, np.dtype):
+        builtin_type = _numpy_dtype_instance_to_builtin_type(nptype)
+        if builtin_type is not None:
+            return builtin_type
+
     # If this is a data type object, use the corresponding scalar data type.
-    if np.issubclass_(type(nptype), np.dtype):
+    if issubclass(type(nptype), np.dtype):
         nptype = nptype.type
 
-    if np.issubclass_(nptype, (bool, np.bool_)):
+    if issubclass(nptype, (bool, np.bool_)):
         # numpy as 2 bool types it looks like. what is the difference?
         return types_bool
     # Because np.uint is a subclass of int,
     # we need to first check for np.uint before
     # checking for int
-    elif np.issubclass_(nptype, np.uint8):
+    elif issubclass(nptype, np.uint8):
         return types_uint8
-    elif np.issubclass_(nptype, np.int8):
+    elif issubclass(nptype, np.int8):
         return types_int8
-    elif np.issubclass_(nptype, np.uint16):
+    elif issubclass(nptype, np.uint16):
         return types_uint16
-    elif np.issubclass_(nptype, np.int16):
+    elif issubclass(nptype, np.int16):
         return types_int16
-    elif np.issubclass_(nptype, np.uint32):
+    elif issubclass(nptype, np.uint32):
         return types_uint32
-    elif np.issubclass_(nptype, np.int32):
+    elif issubclass(nptype, np.int32):
         return types_int32
-    elif np.issubclass_(nptype, np.uint64):
+    elif issubclass(nptype, np.uint64):
         return types_uint64
-    elif np.issubclass_(nptype, np.int64):
+    elif issubclass(nptype, np.int64):
         return types_int64
-    elif np.issubclass_(nptype, int) or nptype == int:
+    elif issubclass(nptype, int) or nptype == int:
         # Catch all int
         return types_int32
-    elif np.issubclass_(nptype, np.object_):
+    elif issubclass(nptype, np.object_):
         # symbolic shape is considered int32
         return types_int32
-    elif np.issubclass_(nptype, np.float16):
+    elif issubclass(nptype, np.float16):
         return types_fp16
     elif (
-        np.issubclass_(nptype, (np.float32, np.single)) or nptype == float
+        issubclass(nptype, (np.float32, np.single)) or nptype == float
     ):
         return types_fp32
-    elif np.issubclass_(nptype, (np.float64, np.double)):
+    elif issubclass(nptype, (np.float64, np.double)):
         return types_fp64
-    elif np.issubclass_(nptype, np.complex64):
+    elif issubclass(nptype, np.complex64):
         return types_complex64
-    elif np.issubclass_(nptype, (np.complex128, complex)):
+    elif issubclass(nptype, (np.complex128, complex)):
         return types_complex128
-    elif np.issubclass_(nptype, (str, np.string_, np.str_)):
+    elif issubclass(nptype, (str, np.bytes_, np.str_)):
         return types_str
     else:
         raise TypeError(f"Unsupported numpy type: {nptype}.")
@@ -402,15 +477,15 @@ def type_to_builtin_type(type):
         return numpy_type_to_builtin_type(type)
 
     # Otherwise, try to infer from a few generic python types
-    if np.issubclass_(type, bool):
+    if issubclass(type, bool):
         return types_bool
-    elif np.issubclass_(type, int):
+    elif issubclass(type, int):
         return types_int32
-    elif np.issubclass_(type, str):
+    elif issubclass(type, str):
         return types_str
-    elif np.issubclass_(type, float):
+    elif issubclass(type, float):
         return types_fp32
-    elif np.issubclass_(type, complex):
+    elif issubclass(type, complex):
         return types_complex64
     else:
         raise TypeError("Could not determine builtin type for " + str(type))
@@ -473,11 +548,21 @@ def is_subtype(type1, type2):
     return type1 == type2
 
 
+def _numpy_val_to_bytes(val: Union[np.ndarray, np.generic]) -> bytes:
+    # Import here to avoid circular import.
+    from coremltools.optimize.coreml import _utils as optimize_utils
+
+    builtin_type = numpy_type_to_builtin_type(val.dtype)
+    if is_sub_byte(builtin_type):
+        val = optimize_utils.pack_elements_into_bits(val, builtin_type.get_bitwidth())
+
+    return val.tobytes()
+
 def np_val_to_py_type(val):
     """Convert numpy val to python primitive equivalent. Ex:
 
     Given: val = np.array([True, False])
-    Returns: [True, False]
+    Returns: (True, False)
 
     Given: val = np.array(32, dtype=np.int32)
     Returns 32
@@ -485,9 +570,9 @@ def np_val_to_py_type(val):
     if not isinstance(val, (_np.ndarray, _np.generic)):
         return val
 
-    if val.dtype in (_np.float16, _np.uint8, _np.int8, _np.uint32):
-        # Serialize to bytes because MIL read them from bytes field (see TensorValue in MIL.proto).
-        return val.tobytes()
+    builtin_type = numpy_type_to_builtin_type(val.dtype)
+    if builtin_type in mil_helper.IMMEDIATE_VALUE_TYPES_IN_BYTES:
+        return _numpy_val_to_bytes(val)
     else:
         if val.dtype in (_np.uint16, _np.int16):
             # TODO (rdar://111797203): Serialize to byte after MIL changes to read from byte field.
@@ -519,3 +604,9 @@ def infer_fp_dtype_from_complex(complex_dtype):
         return types_fp64
     else:
         raise ValueError(f"Unsupported complex dtype ({complex_dtype}).")
+
+
+def get_nbits_int_builtin_type(nbits: int, signed: True) -> type:
+    """Get the nbits int built-in type."""
+    type_prefix = "u" if not signed else ""
+    return string_to_builtin(f"{type_prefix}int{nbits}")
