@@ -1,13 +1,8 @@
-# -*- coding: utf-8 -*-
-
 #  Copyright (c) 2020, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from __future__ import print_function as _
-from __future__ import division as _
-from __future__ import absolute_import as _
 from ..var import Var
 
 
@@ -37,7 +32,7 @@ def _get_input_vars(op, only_nonconst_vars=False):
     return input_vars
 
 
-class DotVisitor(object):
+class DotVisitor:
     """
     Generates a dot description of a ssa block
     """
