@@ -3,12 +3,8 @@
 # Use of this source code is governed by a BSD-3-clause license that can be
 # found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from . import flexible_shape_utils
-from . import optimization_utils
-from . import printer
-from . import quantization_utils
-from . import spec_inspection_utils
-from . import update_optimizer_utils
-from . import utils
+from . import (flexible_shape_utils, optimization_utils, printer,
+               quantization_utils, spec_inspection_utils,
+               update_optimizer_utils, utils)
 from .builder import NeuralNetworkBuilder
 from .update_optimizer_utils import AdamParams, SgdParams
