@@ -1,4 +1,4 @@
-#  Copyright (c) 2023, Apple Inc. All rights reserved.
+#  Copyright (c) 2024, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
@@ -19,6 +19,8 @@ from torch.ao.quantization.backend_config import BackendConfig as _BackendConfig
 from torch.ao.quantization.backend_config import BackendPatternConfig as _BackendPatternConfig
 from torch.ao.quantization.backend_config import DTypeWithConstraints as _DTypeWithConstraints
 
+import coremltools.optimize.torch.quantization.modules.conv_transpose as _qconv_transpose
+import coremltools.optimize.torch.quantization.modules.conv_transpose_fused as _qconv_transpose_fused
 import coremltools.optimize.torch.quantization.modules.fused_modules as _fused
 import coremltools.optimize.torch.quantization.modules.qat_modules as _qat
 import coremltools.optimize.torch.quantization.modules.quantized_modules as _quantized
@@ -27,7 +29,7 @@ from coremltools.optimize.torch.quantization._backend_config_utils import (
     activation_configs as _activation_configs,
 )
 from coremltools.optimize.torch.quantization._backend_config_utils import (
-    binary_op_act_configs as _binary_op_relu_configs,
+    binary_op_act_configs as _binary_op_act_configs,
 )
 from coremltools.optimize.torch.quantization._backend_config_utils import (
     binary_op_configs as _binary_op_configs,
@@ -103,6 +105,11 @@ _func_activations = (
     _F.hardswish,
 )
 
+# ReLU activations
+_relu_activations = (
+    _nn.ReLU,
+    _F.relu,
+)
 
 # layers which have a fixed output range and hence use fixed qparams
 _fixed_qparams_modules = {
@@ -224,6 +231,69 @@ def _conv3d_act() -> _List[_BackendPatternConfig]:
     return configs
 
 
+@_BackendConfigRegistry.register()
+def _conv_transpose1d_act() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose1d -> Act
+    qat: FakeQuant -> qat.ConvTransposeAct1d -> FakeQuant
+    """
+    configs = []
+    for act in _mod_activations + _relu_activations:
+        configs.extend(
+            _weighted_act_configs(
+                mod=_nn.ConvTranspose1d,
+                func_mod=_F.conv_transpose1d,
+                act=act,
+                fused_mod=_fused.ConvTransposeAct1d,
+                qat_mod=_qat.ConvTransposeAct1d,
+                ref_quant_mod=_quantized.QuantizedConvTransposeAct1d,
+            )
+        )
+    return configs
+
+
+@_BackendConfigRegistry.register()
+def _conv_transpose2d_act() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose2d -> Act
+    qat: FakeQuant -> qat.ConvTransposeAct2d -> FakeQuant
+    """
+    configs = []
+    for act in _mod_activations + _relu_activations:
+        configs.extend(
+            _weighted_act_configs(
+                mod=_nn.ConvTranspose2d,
+                func_mod=_F.conv_transpose2d,
+                act=act,
+                fused_mod=_fused.ConvTransposeAct2d,
+                qat_mod=_qat.ConvTransposeAct2d,
+                ref_quant_mod=_quantized.QuantizedConvTransposeAct2d,
+            )
+        )
+    return configs
+
+
+@_BackendConfigRegistry.register()
+def _conv_transpose3d_act() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose3d -> Act
+    qat: FakeQuant -> qat.ConvTransposeAct3d -> FakeQuant
+    """
+    configs = []
+    for act in _mod_activations + _relu_activations:
+        configs.extend(
+            _weighted_act_configs(
+                mod=_nn.ConvTranspose3d,
+                func_mod=_F.conv_transpose3d,
+                act=act,
+                fused_mod=_fused.ConvTransposeAct3d,
+                qat_mod=_qat.ConvTransposeAct3d,
+                ref_quant_mod=_quantized.QuantizedConvTransposeAct3d,
+            )
+        )
+    return configs
+
+
 @_BackendConfigRegistry.register()
 def _linear_act() -> _List[_BackendPatternConfig]:
     """
@@ -296,6 +366,51 @@ def _conv3d_bn() -> _List[_BackendPatternConfig]:
     )
 
 
+@_BackendConfigRegistry.register()
+def _conv_transpose1d_bn() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose1d -> BatchNorm1d
+    qat: FakeQuant -> qat.ConvTransposeBn1d -> FakeQuant
+    """
+    return _weighted_bn_configs(
+        mod=_nn.ConvTranspose1d,
+        bn_mod=_nn.BatchNorm1d,
+        fused_mod=_fused.ConvTransposeBn1d,
+        qat_mod=_qconv_transpose_fused.ConvTransposeBn1d,
+        ref_quant_mod=_nnr.ConvTranspose1d,
+    )
+
+
+@_BackendConfigRegistry.register()
+def _conv_transpose2d_bn() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose2d -> BatchNorm2d
+    qat: FakeQuant -> qat.ConvTransposeBn2d -> FakeQuant
+    """
+    return _weighted_bn_configs(
+        mod=_nn.ConvTranspose2d,
+        bn_mod=_nn.BatchNorm2d,
+        fused_mod=_fused.ConvTransposeBn2d,
+        qat_mod=_qconv_transpose_fused.ConvTransposeBn2d,
+        ref_quant_mod=_nnr.ConvTranspose2d,
+    )
+
+
+@_BackendConfigRegistry.register()
+def _conv_transpose3d_bn() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose3d -> BatchNorm3d
+    qat: FakeQuant -> qat.ConvTransposeBn3d -> FakeQuant
+    """
+    return _weighted_bn_configs(
+        mod=_nn.ConvTranspose3d,
+        bn_mod=_nn.BatchNorm3d,
+        fused_mod=_fused.ConvTransposeBn3d,
+        qat_mod=_qconv_transpose_fused.ConvTransposeBn3d,
+        ref_quant_mod=_nnr.ConvTranspose3d,
+    )
+
+
 @_BackendConfigRegistry.register()
 def _linear_bn() -> _List[_BackendPatternConfig]:
     """
@@ -395,6 +510,72 @@ def _conv3d_bn_act() -> _List[_BackendPatternConfig]:
     return configs
 
 
+@_BackendConfigRegistry.register()
+def _conv_transpose1d_bn_act() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose1d -> BatchNorm1d -> Act
+    qat: FakeQuant -> qat.ConvTransposeBnAct1d -> FakeQuant
+    """
+    configs = []
+    for act in _mod_activations + _relu_activations:
+        configs.extend(
+            _weighted_bn_act_configs(
+                mod=_nn.ConvTranspose1d,
+                act=act,
+                bn_mod=_nn.BatchNorm1d,
+                root_mod=_fused.ConvTransposeBn1d,
+                fused_mod=_fused.ConvTransposeBnAct1d,
+                qat_mod=_qat.ConvTransposeBnAct1d,
+                ref_quant_mod=_quantized.QuantizedConvTransposeAct1d,
+            )
+        )
+    return configs
+
+
+@_BackendConfigRegistry.register()
+def _conv_transpose2d_bn_act() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose2d -> BatchNorm2d -> Act
+    qat: FakeQuant -> qat.ConvTransposeBnAct2d -> FakeQuant
+    """
+    configs = []
+    for act in _mod_activations + _relu_activations:
+        configs.extend(
+            _weighted_bn_act_configs(
+                mod=_nn.ConvTranspose2d,
+                act=act,
+                bn_mod=_nn.BatchNorm2d,
+                root_mod=_fused.ConvTransposeBn2d,
+                fused_mod=_fused.ConvTransposeBnAct2d,
+                qat_mod=_qat.ConvTransposeBnAct2d,
+                ref_quant_mod=_quantized.QuantizedConvTransposeAct2d,
+            )
+        )
+    return configs
+
+
+@_BackendConfigRegistry.register()
+def _conv_transpose3d_bn_act() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose3d -> BatchNorm3d -> Act
+    qat: FakeQuant -> qat.ConvTransposeBnAct3d -> FakeQuant
+    """
+    configs = []
+    for act in _mod_activations + _relu_activations:
+        configs.extend(
+            _weighted_bn_act_configs(
+                mod=_nn.ConvTranspose3d,
+                act=act,
+                bn_mod=_nn.BatchNorm3d,
+                root_mod=_fused.ConvTransposeBn3d,
+                fused_mod=_fused.ConvTransposeBnAct3d,
+                qat_mod=_qat.ConvTransposeBnAct3d,
+                ref_quant_mod=_quantized.QuantizedConvTransposeAct3d,
+            )
+        )
+    return configs
+
+
 @_BackendConfigRegistry.register()
 def _conv1d() -> _List[_BackendPatternConfig]:
     """
@@ -437,6 +618,48 @@ def _conv3d() -> _List[_BackendPatternConfig]:
     )
 
 
+@_BackendConfigRegistry.register()
+def _conv_transpose1d() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose1d
+    qat: FakeQuant -> qat.ConvTranspose1d -> FakeQuant
+    """
+    return _weighted_configs(
+        mod=_nn.ConvTranspose1d,
+        func_mod=_F.conv_transpose1d,
+        qat_mod=_qconv_transpose.ConvTranspose1d,
+        ref_quant_mod=_nnr.ConvTranspose1d,
+    )
+
+
+@_BackendConfigRegistry.register()
+def _conv_transpose2d() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose2d
+    qat: FakeQuant -> qat.ConvTranspose2d -> FakeQuant
+    """
+    return _weighted_configs(
+        mod=_nn.ConvTranspose2d,
+        func_mod=_F.conv_transpose2d,
+        qat_mod=_qconv_transpose.ConvTranspose2d,
+        ref_quant_mod=_nnr.ConvTranspose2d,
+    )
+
+
+@_BackendConfigRegistry.register()
+def _conv_transpose3d() -> _List[_BackendPatternConfig]:
+    """
+    float: ConvTranspose3d
+    qat: FakeQuant -> qat.ConvTranspose3d -> FakeQuant
+    """
+    return _weighted_configs(
+        mod=_nn.ConvTranspose3d,
+        func_mod=_F.conv_transpose3d,
+        qat_mod=_qconv_transpose.ConvTranspose3d,
+        ref_quant_mod=_nnr.ConvTranspose3d,
+    )
+
+
 @_BackendConfigRegistry.register()
 def _linear() -> _List[_BackendPatternConfig]:
     """
@@ -501,7 +724,7 @@ def _add_act() -> _List[_BackendPatternConfig]:
     FakeQuant ->
     """
     acts = _mod_activations + _func_activations + (_nn.ReLU, _F.relu, _torch.relu)
-    return _binary_op_relu_configs(ops=[_operator.add, _torch.add], acts=list(acts))
+    return _binary_op_act_configs(ops=[_operator.add, _torch.add], acts=list(acts))
 
 
 @_BackendConfigRegistry.register()
@@ -518,7 +741,7 @@ def _mul_act() -> _List[_BackendPatternConfig]:
     FakeQuant ->
     """
     acts = _mod_activations + _func_activations + (_nn.ReLU, _F.relu, _torch.relu)
-    return _binary_op_relu_configs(ops=[_operator.mul, _torch.mul], acts=list(acts))
+    return _binary_op_act_configs(ops=[_operator.mul, _torch.mul], acts=list(acts))
 
 
 @_BackendConfigRegistry.register()
@@ -535,7 +758,23 @@ def _matmul_act() -> _List[_BackendPatternConfig]:
     FakeQuant ->
     """
     acts = _mod_activations + _func_activations + (_nn.ReLU, _F.relu, _torch.relu)
-    return _binary_op_relu_configs(ops=[_torch.matmul], acts=list(acts))
+    return _binary_op_act_configs(ops=[_torch.matmul], acts=list(acts))
+
+
+@_BackendConfigRegistry.register()
+def _einsum_act() -> _List[_BackendPatternConfig]:
+    """
+    float:
+    input_1 ->
+                einsum -> Act -> output
+    input_2 ->
+    qat:
+    FakeQuant ->
+                 einsum -> Act -> FakeQuant
+    FakeQuant ->
+    """
+    acts = _mod_activations + _func_activations + (_nn.ReLU, _F.relu, _torch.relu)
+    return _binary_op_act_configs(ops=[_torch.einsum], acts=list(acts))
 
 
 @_BackendConfigRegistry.register()
@@ -586,6 +825,21 @@ def _matmul() -> _List[_BackendPatternConfig]:
     return _binary_op_configs(ops=[_torch.matmul])
 
 
+@_BackendConfigRegistry.register()
+def _einsum() -> _List[_BackendPatternConfig]:
+    """
+    float:
+    input_1 ->
+                einsum -> output
+    input_2 ->
+    qat:
+    FakeQuant ->
+                 einsum -> FakeQuant
+    FakeQuant ->
+    """
+    return _binary_op_configs(ops=[_torch.einsum])
+
+
 @_BackendConfigRegistry.register()
 def _cat() -> _List[_BackendPatternConfig]:
     """
@@ -863,7 +1117,7 @@ def get_backend_config() -> _BackendConfig:
 
 def get_supported_modules() -> _List[_Any]:
     """
-    Returns a list of modules which are supported for quantization
+    Returns a tuple of modules which are supported for quantization
     aware training.
     """
     return tuple(_BackendConfigRegistry.supported_modules)
