@@ -1,4 +1,4 @@
-#  Copyright (c) 2023, Apple Inc. All rights reserved.
+#  Copyright (c) 2024, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
@@ -22,6 +22,9 @@ from attr import define as _define
 from attr import field as _field
 from attrs import validators as _validators
 
+from coremltools.optimize.torch._utils.torch_utils import (
+    get_n_bits_from_dtype as _get_n_bits_from_dtype,
+)
 from coremltools.optimize.torch._utils.torch_utils import (
     maybe_convert_str_to_dtype as _maybe_convert_str_to_dtype,
 )
@@ -40,10 +43,12 @@ _logger = _logging.getLogger(__name__)
 @_unique
 class ObserverType(_Enum):
     """
-    An enum indicating the type of observer. Allowed options are moving_average_min_max and mix_max.
+    An enum indicating the type of observer.
+    Allowed options are moving_average_min_max, min_max, ema_min_max, ema_percentile, mse, ema_mse, lsq and lsq_plus.
     """
+
     moving_average_min_max = "moving_average_min_max"
-    mix_max = "min_max"
+    min_max = "min_max"
 
     @staticmethod
     def get_observer(observer_type: "ObserverType", is_per_channel: bool) -> _Any:
@@ -88,16 +93,21 @@ class QuantizationScheme(_Enum):
 _default_quantization_options = {
     "weight_dtype": _torch.qint8,
     "weight_per_channel": True,
+    "weight_ch_axis": 0,
     "activation_dtype": _torch.quint8,
     "observer": ObserverType.moving_average_min_max,
     "quantization_scheme": QuantizationScheme.symmetric,
 }
 
 
+# Backends only support 4 and 8 bit quantization
+_SUPPORTED_N_BITS = [4, 8, 32]
+
+
 @_define
 class ModuleLinearQuantizerConfig(_ModuleOptimizationConfig):
     """
-    Configuration class for specifying global and module level quantization options for linear quantization
+    Configuration class for specifying global and module-level quantization options for linear quantization
     algorithm implemented in :py:class:`LinearQuantizer`.
 
     Linear quantization algorithm simulates the effects of quantization during training, by quantizing
@@ -159,17 +169,18 @@ class ModuleLinearQuantizerConfig(_ModuleOptimizationConfig):
             # mode, thus more closely simulating the inference numerics during training time.
 
     Args:
-        weight_dtype (:py:class:`torch.dtype`): The dtype to use for quantizing the weights. When dtype
-            is set to :py:class:`torch.float32`, the weights corresponding to that layer are not quantized.
-            Defaults to :py:class:`torch.qint8`.
-        weight_observer (:py:class:`ObserverType`): Type of observer to use for quantizing weights. Defaults
-            to ``moving_average_min_max``.
+        weight_dtype (:py:class:`torch.dtype`): The dtype to use for quantizing the weights. The number of bits used
+            for quantization is inferred from the dtype. When dtype is set to :py:class:`torch.float32`, the weights
+            corresponding to that layer are not quantized.  Defaults to :py:class:`torch.int8` which corresponds to
+            8-bit quantization.
+        weight_observer (:py:class:`ObserverType`): Type of observer to use for quantizing weights.
+            Defaults to ``moving_average_min_max``.
         weight_per_channel (:obj:`bool`): When ``True``, weights are quantized per channel; otherwise, per tensor.
         activation_dtype (:py:class:`torch.dtype`): The dtype to use for quantizing the activations. When dtype
             is set to :py:class:`torch.float32`, the activations corresponding to that layer are not quantized.
             Defaults to :py:class:`torch.quint8`.
-        activation_observer (:py:class:`ObserverType`): Type of observer to use for quantizing activations. Allowed
-            values are ``min_max`` and ``moving_average_min_max``. Defaults to ``moving_average_min_max``.
+        activation_observer (:py:class:`ObserverType`): Type of observer to use for quantizing activations.
+            Defaults to ``moving_average_min_max``.
         quantization_scheme: (:py:class:`QuantizationScheme`): Type of quantization configuration to use. When
             this parameter is set to :py:class:`QuantizationScheme.symmetric`, all weights are
             quantized with zero point as zero, and activations are quantized with zero point as zero for
@@ -181,16 +192,11 @@ class ModuleLinearQuantizerConfig(_ModuleOptimizationConfig):
             quantization simulation, the third to disabling observers, and the last to freezing batch norm statistics.
             Defaults to ``None``, which means the ``step`` method of :py:class:`LinearQuantizer` will be a no-op and
             all observers and quantization simulation will be turned on from the first step, batch norm layers always
-            operate in training mode, and mean and varaince statistics collection is not frozen.
+            operate in training mode, and mean and variance statistics collection is not frozen.
     """
 
-    weight_dtype: _torch.dtype = _field(
+    weight_dtype: _Union[str, _torch.dtype] = _field(
         default=_default_quantization_options["weight_dtype"],
-        converter=_maybe_convert_str_to_dtype,
-        validator=[
-            _validators.instance_of(_torch.dtype),
-            _validators.in_([_torch.qint8, _torch.quint8, _torch.float32]),
-        ],
     )
     weight_observer: ObserverType = _field(
         default=_default_quantization_options["observer"],
@@ -230,11 +236,11 @@ class ModuleLinearQuantizerConfig(_ModuleOptimizationConfig):
     )
 
     def __attrs_post_init__(self):
-        if self.weight_dtype == _torch.float32 and self.activation_dtype != _torch.float32:
+        self.weight_n_bits = _get_n_bits_from_dtype(self.weight_dtype)
+        self.weight_dtype = _maybe_convert_str_to_dtype(self.weight_dtype)
+        if self.weight_dtype not in [_torch.qint8, _torch.quint8, _torch.float32]:
             raise ValueError(
-                f"Unsupported configuration: weight_dtype = {self.weight_dtype}, "
-                f"activation_dtype = {self.activation_dtype}. When weights are not quantized,"
-                f"activations cannot be quantized."
+                f"weight_dtype must be one of (_torch.qint8, _torch.quint8, _torch.float32) not {self.weight_dtype}"
             )
 
     @milestones.validator
@@ -246,6 +252,15 @@ class ModuleLinearQuantizerConfig(_ModuleOptimizationConfig):
                 f"Refer to docs for more information."
             )
 
+    @classmethod
+    def from_dict(cls, config_dict):
+        converter = _cattrs.Converter(forbid_extra_keys=True)
+        converter.register_structure_hook(
+            _Union[str, _torch.dtype],
+            lambda obj, type: obj,
+        )
+        return converter.structure_attrs_fromdict(config_dict, cls)
+
 
 _ModuleTypeConfigType = _NewType(
     "ModuleTypeConfigType",
@@ -298,6 +313,17 @@ class LinearQuantizerConfig(_OptimizationConfig):
                 }
             )
 
+            # If model has some methods and attributes which are not used in the forward
+            # pass, but are needed to be preserved after quantization is added, they can
+            # be preserved on the quantized model by passing them in preserved_attributes
+            # parameter
+
+            model = MyModel()
+            model.key_1 = value_1
+            model.key_2 = value_2
+
+            config = LinearQuantizerConfig.from_dict({"preserved_attributes": ["key_1", "key_2"]})
+
     Args:
         global_config (:py:class:`ModuleLinearQuantizerConfig`): Config to be applied globally
             to all supported modules. Missing values are chosen from the default config.
@@ -306,11 +332,14 @@ class LinearQuantizerConfig(_OptimizationConfig):
             module class, such as :py:class:`torch.nn.Linear`. The keys can be either strings
             or module classes.
         module_name_configs (:obj:`dict` of :obj:`str` to :py:class:`ModuleLinearQuantizerConfig`):
-            Module level configs applied to specific modules.
+            Module-level configs applied to specific modules.
             The name of the module must be a fully qualified name that can be used to fetch it
             from the top level module using the ``module.get_submodule(target)`` method.
         non_traceable_module_names (:obj:`list` of :obj:`str`):
             Names of modules which cannot be traced using ``torch.fx``.
+        preserved_attributes (:obj:`list` of :obj:`str`): Names of attributes of the model
+            which should be preserved on the prepared and finalized models, even if they are not
+            used in the model's forward pass.
 
     .. note::
         The ``quantization_scheme`` parameter must be the same across all configs.
@@ -347,6 +376,12 @@ class LinearQuantizerConfig(_OptimizationConfig):
             member_validator=_validators.instance_of(str),
         ),
     )
+    preserved_attributes: _List[str] = _field(
+        factory=list,
+        validator=_validators.deep_iterable(
+            member_validator=_validators.instance_of(str),
+        ),
+    )
 
     def __attrs_post_init__(self):
         if (
@@ -365,6 +400,10 @@ class LinearQuantizerConfig(_OptimizationConfig):
     def from_dict(cls, config_dict: _Dict[str, _Any]) -> "LinearQuantizerConfig":
         super().from_dict(config_dict)
         converter = _cattrs.Converter(forbid_extra_keys=True)
+        converter.register_structure_hook(
+            _Union[str, _torch.dtype],
+            lambda obj, type: obj,
+        )
         converter.register_structure_hook(
             _ModuleTypeConfigType,
             _structure_from_dict_hook_factory(ModuleLinearQuantizerConfig),
