@@ -6,6 +6,7 @@
 import copy
 import os
 import tempfile
+from collections import Counter
 
 import numpy as np
 import pytest
@@ -13,7 +14,8 @@ import pytest
 import coremltools as ct
 from coremltools._deps import _HAS_TORCH
 from coremltools.converters.mil import Builder as mb
-from coremltools.converters.mil.mil import Function, Program, get_new_symbol
+from coremltools.converters.mil import mil
+from coremltools.converters.mil.mil import Function, get_new_symbol, types
 from coremltools.converters.mil.testing_utils import get_op_types_in_program
 
 if _HAS_TORCH:
@@ -57,7 +59,7 @@ class TestInputs:
         '''
         input name : "x/0" becomes "x_0" due to name sanitization applied during conversion
         '''
-        prog = Program()
+        prog = mil.Program()
         func_inputs = {"x/0": mb.placeholder(shape=[2, 3]),
                        "y": mb.placeholder(shape=[2, 3])}
         with Function(func_inputs) as ssa_fun:
@@ -79,7 +81,7 @@ class TestInputs:
 
     @staticmethod
     def _test_variant_input_type_prediction(to_tensor, convert_to):
-        prog = Program()
+        prog = mil.Program()
         func_inputs = {"x": mb.placeholder(shape=[2, 3]),
                        "y": mb.placeholder(shape=[2, 3])}
         with Function(func_inputs) as ssa_fun:
@@ -129,6 +131,33 @@ class TestInputs:
 ###############################################################################
 
 class TestMLProgramConverterExamples:
+    @staticmethod
+    @pytest.mark.skipif(
+        ct.utils._macos_version() < (15, 0), reason="Tests are for deployment target iOS18/macos15"
+    )
+    def test_build_stateful_model():
+        @mb.program(
+            input_specs=[
+                mb.TensorSpec((1,), dtype=types.fp16),
+                mb.StateTensorSpec((1,), dtype=types.fp16),
+            ],
+        )
+        def prog(x, accumulator_state):
+            # Read state
+            accumulator_value = mb.read_state(input=accumulator_state)
+            # Update value
+            y = mb.add(x=x, y=accumulator_value, name="y")
+            # Write state
+            mb.coreml_update_state(state=accumulator_state, value=y)
+
+            return y
+
+        mlmodel = ct.convert(prog, minimum_deployment_target=ct.target.iOS18)
+
+        # try to run prediction on the stateful model
+        state = mlmodel.make_state()
+        assert mlmodel.predict({"x": np.array([1.0])}, state=state)["y"] == 1
+        assert mlmodel.predict({"x": np.array([1.0])}, state=state)["y"] == 2
 
     @staticmethod
     def test_model_save(tmpdir):
@@ -167,7 +196,7 @@ class TestMLProgramConverterExamples:
     @staticmethod
     @pytest.mark.skipif(not ct.utils._is_macos(), reason="Platform is not Mac OS")
     def test_deepcopy_error_with_symbols_in_prog():
-        prog = Program()
+        prog = mil.Program()
         func_inputs = {"x": mb.placeholder(shape=[get_new_symbol(), 3]),
                        "y": mb.placeholder(shape=[2, 3])}
         with Function(func_inputs) as ssa_fun:
@@ -390,12 +419,21 @@ class TestGraphPassManagement:
             convert_to="mlprogram",
             pass_pipeline=pipeline,
         )
-        assert (
-            get_op_types_in_program(
-                model_converted._get_mil_internal(), skip_const_ops=False
-            ).count("const")
-            == 24
-        )
+
+        op_types = get_op_types_in_program(model_converted._mil_program, skip_const_ops=False)
+        expected_counts = {
+            "const": 26,
+            "cast": 7,
+            "conv": 1,
+            "matmul": 1,
+            "add": 1,
+            "shape": 1,
+            "slice_by_index": 2,
+            "concat": 1,
+            "reshape": 1,
+            "leaky_relu": 1,
+        }
+        assert Counter(op_types) == expected_counts
 
     def test_empty_pipeline(self):
         model = self._get_test_model()
@@ -409,6 +447,8 @@ class TestGraphPassManagement:
             inputs=[ct.TensorType(shape=example_input.shape)],
             convert_to="mlprogram",
             pass_pipeline=pipeline,
+            # TODO (rdar://131396853) Re-enable model load
+            skip_model_load=True,
         )
         assert get_op_types_in_program(model_converted._get_mil_internal()) == [
             "conv",
@@ -436,6 +476,8 @@ class TestGraphPassManagement:
             inputs=[ct.TensorType(shape=example_input.shape)],
             convert_to="mlprogram",
             pass_pipeline=pipeline,
+            # TODO (rdar://131396853) Re-enable model load
+            skip_model_load=True,
         )
         # The fp16 cast is skipped for conv and linear as we specified them in the pass options.
         assert get_op_types_in_program(model_converted._get_mil_internal()) == [
@@ -491,7 +533,7 @@ class TestGraphPassManagement:
             get_op_types_in_program(
                 model_converted._get_mil_internal(), skip_const_ops=False
             ).count("const")
-            == 23
+            == 25
         )
 
     def test_pass_unsupported_option(self):
