@@ -44,6 +44,9 @@ EXPECTED_MODULES = [
     "libmodelpackage",
     "libmilstoragepython",
     "optimize",
+    "StateType",
+    "ReshapeFrequency",
+    "SpecializationStrategy",
 ]
 
 
@@ -52,7 +55,7 @@ class TestApiVisibilities:
 
     def test_top_level(self):
         if not ct.utils._is_macos():
-             EXPECTED_MODULES.remove("libcoremlpython")
+            EXPECTED_MODULES.remove("libcoremlpython")
         _check_visible_modules(_get_visible_items(ct), EXPECTED_MODULES)
 
     def test_utils(self):
@@ -64,14 +67,20 @@ class TestApiVisibilities:
             "evaluate_regressor",
             "evaluate_transformer",
             "make_pipeline",
+            "materialize_dynamic_shape_mlmodel",
             "load_spec",
             "rename_feature",
             "save_spec",
+            "save_multifunction",
+            "MultiFunctionDescriptor",
+            "randomize_weights",
+            "bisect_model",
         ]
         _check_visible_modules(_get_visible_items(ct.utils), expected)
 
     def test_models(self):
         expected = [
+            "array_feature_extractor",
             "CompiledMLModel",
             "MLModel",
             "datatypes",
@@ -100,6 +109,7 @@ class TestApiVisibilities:
             "user_defined_metadata",
             "version",
             "weights_dir",
+            "make_state",
         ]
         _check_visible_modules(_get_visible_items(ct.models.MLModel), expected)
 
@@ -161,6 +171,7 @@ class TestApiVisibilities:
             "mil",
             "sklearn",
             "xgboost",
+            "StateType",
         ]
         _check_visible_modules(_get_visible_items(ct.converters), expected)
 
@@ -178,6 +189,7 @@ class TestApiVisibilities:
             "OpPalettizerConfig",
             "OptimizationConfig",
             "OpThresholdPrunerConfig",
+            "experimental",
             "linear_quantize_weights",
             "palettize_weights",
             "prune_weights",
