@@ -5,6 +5,8 @@
 
 import tempfile
 
+import pytest
+
 from coremltools.converters.mil.testing_utils import get_op_types_in_program
 from coremltools.test.optimize.coreml.test_passes import (
     TestCompressionPasses as _TestCompressionPasses,
@@ -212,6 +214,7 @@ class TestOptimizeTorchAPIOverview:
         _test_config(self.get_global_config())
         _test_config(self.get_fine_grain_config())
 
+    @pytest.mark.xfail(reason="rdar://132361333 Palettization Test Case Time Out", run=False)
     def test_programmatic_example_1(self):
         import torch
 
@@ -231,7 +234,7 @@ class TestOptimizeTorchAPIOverview:
 
         # Initialize the palettizer
         config = DKMPalettizerConfig(
-            global_config=ModuleDKMPalettizerConfig(n_bits=4, cluster_dim=4)
+            global_config=ModuleDKMPalettizerConfig(n_bits=4, cluster_dim=2)
         )
 
         palettizer = DKMPalettizer(model, config)
@@ -258,7 +261,7 @@ class TestOptimizeTorchAPIOverview:
             traced_model,
             inputs=[ct.TensorType(shape=example_input.shape)],
             pass_pipeline=ct.PassPipeline.DEFAULT_PALETTIZATION,
-            minimum_deployment_target=ct.target.iOS16,
+            minimum_deployment_target=ct.target.iOS18,
         )
         assert coreml_model is not None
         output_file = tempfile.NamedTemporaryFile(suffix=".mlpackage").name
@@ -295,7 +298,7 @@ class TestOptimizeTorchAPIOverview:
                     config = config.set_module_name(
                         name,
                         ModuleLinearQuantizerConfig(
-                            weight_observer=ObserverType.mix_max, weight_per_channel=True
+                            weight_observer=ObserverType.min_max, weight_per_channel=True
                         ),
                     )
                 else:
@@ -329,6 +332,46 @@ class TestOptimizeTorchAPIOverview:
         output_file = tempfile.NamedTemporaryFile(suffix=".mlpackage").name
         coreml_model.save(output_file)
 
+    def test_quantize_submodule(self):
+        import torch
+        from torchvision.models import mobilenet_v3_small
+
+        import coremltools as ct
+        from coremltools.optimize.torch.quantization import LinearQuantizer
+
+        class Model(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.model1 = mobilenet_v3_small()
+                self.model2 = mobilenet_v3_small()
+
+            def forward(self, x):
+                return self.model1(x), self.model2(x)
+
+        model = Model()
+        data = torch.randn(1, 3, 224, 224)
+        example_inputs = (data,)
+
+        quantizer = LinearQuantizer(model.model1)
+        model.model1 = quantizer.prepare(example_inputs=example_inputs)
+        model(data)
+        model.model1 = quantizer.finalize()
+
+        model = model.eval()
+        traced_model = torch.jit.trace(model, example_inputs=example_inputs)
+        coreml_model = ct.convert(
+            traced_model,
+            convert_to="mlprogram",
+            inputs=[ct.TensorType(shape=data.shape)],
+            minimum_deployment_target=ct.target.iOS18,
+            skip_model_load=True,
+        )
+        assert coreml_model is not None
+        quant_ops = coreml_model._mil_program.functions["main"].find_ops(
+            op_type="constexpr_blockwise_shift_scale"
+        )
+        assert len(quant_ops) > 0
+
 
 class TestConvertingCompressedSourceModels:
     """
