@@ -10,15 +10,16 @@ import unittest
 import numpy as np
 import pytest
 
+from ..utils import load_boston
 import coremltools as ct
 from coremltools._deps import _HAS_LIBSVM, _HAS_SKLEARN
+from coremltools.converters.mil import mil
 from coremltools.converters.mil.mil import Builder as mb
 from coremltools.converters.mil.mil import Function, Program
 from coremltools.models.pipeline import PipelineClassifier, PipelineRegressor
 from coremltools.models.utils import _is_macos
 
 if _HAS_SKLEARN:
-    from sklearn.datasets import load_boston
     from sklearn.linear_model import LinearRegression
     from sklearn.pipeline import Pipeline
     from sklearn.preprocessing import OneHotEncoder
@@ -43,12 +44,8 @@ class LinearRegressionPipelineCreationTest(unittest.TestCase):
         """
         Set up the unit test by loading the dataset and training a model.
         """
-
-        if not (_HAS_SKLEARN):
-            return
-
         scikit_data = load_boston()
-        feature_names = scikit_data.feature_names
+        feature_names = scikit_data["feature_names"]
 
         scikit_model = LinearRegression()
         scikit_model.fit(scikit_data["data"], scikit_data["target"])
@@ -63,7 +60,7 @@ class LinearRegressionPipelineCreationTest(unittest.TestCase):
 
     def test_pipeline_regression_creation(self):
 
-        input_names = self.scikit_data.feature_names
+        input_names = self.scikit_data["feature_names"]
         output_name = "target"
         p_regressor = PipelineRegressor(input_names, "target")
         p_regressor.add_model(self.scikit_spec)
@@ -120,7 +117,7 @@ class LibSVMPipelineCreationTest(unittest.TestCase):
 
         libsvm_model = svmutil.svm_train(prob, param)
         libsvm_spec = libsvm_converter.convert(
-            libsvm_model, scikit_data.feature_names, "target"
+            libsvm_model, scikit_data["feature_names"], "target"
         ).get_spec()
 
         # Save the data and the model
@@ -129,7 +126,7 @@ class LibSVMPipelineCreationTest(unittest.TestCase):
 
     def test_pipeline_classifier_creation(self):
 
-        input_names = self.scikit_data.feature_names
+        input_names = self.scikit_data["feature_names"]
         p_classifier = PipelineClassifier(input_names, [1, 0])
         p_classifier.add_model(self.libsvm_spec)
 
@@ -168,10 +165,7 @@ class LinearRegressionPipeline(unittest.TestCase):
         """
         Set up the unit test by loading the dataset and training a model.
         """
-        if not _HAS_SKLEARN:
-            return
         scikit_data = load_boston()
-        feature_names = scikit_data.feature_names
 
         scikit_model = Pipeline(steps=[("linear", LinearRegression())])
         scikit_model.fit(scikit_data["data"], scikit_data["target"])
@@ -181,7 +175,7 @@ class LinearRegressionPipeline(unittest.TestCase):
         self.scikit_model = scikit_model
 
     def test_pipeline_regression_creation(self):
-        input_names = self.scikit_data.feature_names
+        input_names = self.scikit_data["feature_names"]
         output_name = "target"
 
         p_regressor_model = converter.convert(self.scikit_model, input_names, "target")
@@ -238,7 +232,7 @@ class TestMakePipeline:
         weight_tensor = np.arange(input_length * output_length, dtype='float32')
         weight_tensor = weight_tensor.reshape(output_length, input_length)
 
-        prog = Program()
+        prog = mil.Program()
         func_inputs = {input_name: mb.placeholder(shape=(input_length,))}
         with Function(func_inputs) as ssa_fun:
             input = ssa_fun.inputs[input_name]
@@ -293,7 +287,7 @@ class TestMakePipeline:
 
     @staticmethod
     def test_compute_unit():
-        # Case 1 - Infering compute_unit
+        # Case 1 - Inferring compute_unit
         m1 = TestMakePipeline._make_model("x", 20, "y1", 10,
                                           compute_units=ct.ComputeUnit.CPU_ONLY)
         m2 = TestMakePipeline._make_model("y1", 10, "y2", 2,
@@ -319,7 +313,7 @@ class TestMakePipeline:
     @staticmethod
     def test_second_model_needs_pipeline_input():
         # First model takes one parameter
-        p1 = Program()
+        p1 = mil.Program()
         func_inputs = {'x1': mb.placeholder(shape=(2,))}
         with Function(func_inputs) as ssa_fun:
             x1 = ssa_fun.inputs['x1']
@@ -330,7 +324,7 @@ class TestMakePipeline:
 
         # Second model takes two parameters. One will be from previous model in pipeline.
         # The other as pipeline input.
-        p2 = Program()
+        p2 = mil.Program()
         func_inputs = {
             'y1': mb.placeholder(shape=(2,)),
             'x2': mb.placeholder(shape=(2,)),
