@@ -11,6 +11,7 @@ import numpy as np
 import pandas as pd
 import pytest
 
+from ..utils import load_boston
 from coremltools._deps import (_HAS_LIBSVM, _HAS_SKLEARN, MSG_LIBSVM_NOT_FOUND,
                                MSG_SKLEARN_NOT_FOUND)
 from coremltools.models.utils import (_is_macos, _macos_version,
@@ -22,7 +23,6 @@ if _HAS_LIBSVM:
     from coremltools.converters import libsvm
 
 if _HAS_SKLEARN:
-    from sklearn.datasets import load_boston
     from sklearn.preprocessing import OneHotEncoder
     from sklearn.svm import SVR
 
@@ -57,7 +57,7 @@ class SvrScikitTest(unittest.TestCase):
             model = SVR()
             spec = sklearn_converter.convert(model, "data", "out")
 
-        # Check the expected class during covnersion.
+        # Check the expected class during conversion.
         with self.assertRaises(TypeError):
             model = OneHotEncoder()
             spec = sklearn_converter.convert(model, "data", "out")
@@ -173,8 +173,8 @@ class EpsilonSVRLibSVMTest(unittest.TestCase):
         num_inputs = len(data["data"][0])
         spec = libsvm.convert(self.libsvm_model, input_length=num_inputs + 1)
 
-        # Not enought input names.
-        input_names = ["this", "is", "not", "enought", "names"]
+        # Not enough input names.
+        input_names = ["this", "is", "not", "enough", "names"]
         with self.assertRaises(ValueError):
             libsvm.convert(self.libsvm_model, input_names=input_names)
         with self.assertRaises(ValueError):
@@ -188,7 +188,7 @@ class EpsilonSVRLibSVMTest(unittest.TestCase):
         )
 
     def test_conversion_bad_inputs(self):
-        # Check the expected class during covnersion.
+        # Check the expected class during conversion.
         with self.assertRaises(TypeError):
             model = OneHotEncoder()
             spec = libsvm.convert(model, "data", "out")
